/** Java class "Homography_2D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;
/**
 * <p>
 * This class contains constructors to build a Homography_2D. Its matrix has 9 
 * entries, but 8 needs to be set due to homogenity
 * A homography_2D consists of:  	
 * <ul>
 * 		<li> Transformation(2) </li>
 * 		<li> Rotation(1), </li>
 * 		<li> Scale (2), </li>
 * 		<li> Shear (1), </li>
 * 		<li> projective part (2) </li>
 * </ul>
 * @author Norbert Fischer
 * 
 * @version 1.1 (26.1.2004) Achim Knigs
 * 		Changed Constructor Homography_2D(double hom[], Matrix cov) to accept 9 
 * 		values for the Matrix and to accept them by counting up the rows, befor
 * 		the colums. 
 */
public class Homography_2D extends Transformation {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	/**
	 * dimension must be set. Otherwise it's not possible to gain
	 * matrix from vector!!!
	 * This DIM = 3 is valid FOR ALL SUBCLASSES xxx_2D !   
	 */
	protected static final int DIM = 3;

	/**
	 * standard constructor. Creates an identity matrix
	 */
	public Homography_2D() {
		setV(LinearAlgebraFactory.getMatrixId(DIM).vec());
		setCov(LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Creates a homography by an affinity and the projective part.
	 * Error propagation is done automatically 
	 * @param aff   contains the affinity part
	 * @param projective    a vector with dimension 2. contains the 
	 * 					    projective part of homography_2D
	 */
	public Homography_2D(Affinity_2D aff, Vector projective) {
		this(aff, projective, null);
	}

	/**
	 * Creates a homography with covariane by an affinity and 
	 * the projective part. 
	 * No error propagation will be performed.
	 * @param aff   contains the affinity part
	 * @param projective  a vector with dimension 2. contains the 
	 * 					  projective part of homography_2D
	 * @param cov   9x9 matrix
	 */
	public Homography_2D(Affinity_2D aff, Vector projective, Matrix cov) {
		if (projective.getDim() != DIM - 1) {
			throw new IllegalArgumentException(
				"Projective vector must " + "have two entries!");
		}
		Vector tmpV = aff.getV();
		tmpV.set(2, projective.get(0));
		tmpV.set(5, projective.get(1));
		setV(tmpV);

		//FFG
		/*
		 * null comes from Homography_2D(Affinity_3D aff, Vector projective)
		 * It's a kind of flag to show, that FFG should be performed
		 */
		if (cov == null) {
			//Because this homography CONSISTS of aff and a projetive part, no
			//error propagation could done (between what, if so?)
			setCov(aff.getCov());
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else {
			checkCovDim(cov, DIM * DIM);
			setCov(cov);
		}
	}

	/**
	 * sets Homography_2D by hand. positions are: <br>
	 * <pre>
	 *              |hom[0]  hom[1]  hom[2] |
	 *              |hom[3]  hom[4]  hom[5] |
	 *              |hom[6]  hom[7]  hom[8] |
	 * </pre>
	 * @param hom    a double array containing the values. Must be 
	 * 			     dimension 9
	 */
	public Homography_2D(double hom[]) {
		this(hom, LinearAlgebraFactory.getMatrix(DIM * DIM, DIM * DIM));
	}

	/**
	 * sets Homography_2D with covariance by hand. positions are: <br>
	 * <pre>
	 *              |hom[0]  hom[1]  hom[2] |
	 *              |hom[3]  hom[4]  hom[5] |
	 *              |hom[6]  hom[7]  hom[8] |
	 * </pre>
	 * @param hom    a double array containing the values. Must be 
	 * 			     dimension 8
	 * @param cov    9x9 matrix
	 */
	public Homography_2D(double hom[], Matrix cov) {
		if (hom.length != (DIM * DIM)) {
			throw new IllegalArgumentException(
				"Array must " + "have " + DIM*DIM + " values!");
		}
		Vector tmpV = LinearAlgebraFactory.getVector(DIM * DIM);

		tmpV.set(0,hom[0]);
		tmpV.set(1,hom[3]);
		tmpV.set(2,hom[6]);
		tmpV.set(3,hom[1]);
		tmpV.set(4,hom[4]);
		tmpV.set(5,hom[7]);
		tmpV.set(6,hom[2]);
		tmpV.set(7,hom[5]);
		tmpV.set(8,hom[8]);
		setV(tmpV);
		
		checkCovDim(cov, DIM * DIM);
		setCov(cov);
	}
	
	/**
	 * Constructs a homography out of a given homography matrix and
	 * a covariance matrix for the given homography.
	 * @param hom   a 3x3 homography matrix 
	 * @param cov   9x9 covariance matrix
	 */
	public Homography_2D(Matrix hom, Matrix cov) {
		if ((hom.getRowDimension() != DIM)
				|| (hom.getColumnDimension() != DIM)) {
				throw new IllegalArgumentException(
					"Homography_2D dimensions must be "
						+ DIM
						+ "x"
						+ DIM
						+ ".");
			}
		
		setV(hom.vec());
		
		checkCovDim(cov, DIM * DIM);
		setCov(cov);
	}

	
	/** Creates homography from four homologous points
	 * All points need to have covariance matrices of rank at least two.
	 *  
	 * @param x11 first point in first image
	 * @param x12 first point in second image
	 * @param x21 second point in first image
	 * @param x22 second point in second image
	 * @param x31 third point in first image
	 * @param x32 third point in second image
	 * @param x41 fourth point in first image
	 * @param x42 fourth point in second image
	 * 
	 */
	public Homography_2D(Point_2D x11, Point_2D x12, Point_2D x21, Point_2D x22, Point_2D x31, Point_2D x32, Point_2D x41, Point_2D x42) {
		TriRelationalProperty[] prop = new TriRelationalProperty[4];
		prop[0] = new TriRelationalProperty(x11, x12, Homography_2D.class);
		prop[1] = new TriRelationalProperty(x21, x22, Homography_2D.class);
		prop[2] = new TriRelationalProperty(x31, x32, Homography_2D.class);
		prop[3] = new TriRelationalProperty(x41, x42, Homography_2D.class);
		if (!estimate(prop))
			throw new IllegalArgumentException("Convergence not reached!");
	}
	
	/** Creates a homography out of two given homographies: H=h1*h2
	 * @param h1 the first homography
	 * @param h2 the second homography
	 * 
	 */
	public Homography_2D(Homography_2D h1, Homography_2D h2) {
		setV((h1.getTransformationMatrix().
				  mult(h2.getTransformationMatrix()).vec()));
				  
		//FFG	
		if (h1.cov!=null && h2.cov!=null)	
		{
			setCov(errorProp(h1,h2));
		}
	}

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom() {
		return 8;
	}

	/**
	 * Transforms an entity with this transformation. The new entity contains 
	 * error propagation of transform and old entity 
	 * @param e   Entity to transform
	 * @return A new Entity with applied transformation and error propagation
	 */
	public Entity transform(Entity e) {
		Matrix h = null;
		
		// Covariance for transformation matrix
		Matrix covariance = null;

		/*
		 * Check instance and choose right transformation matrix and transform
		 * covariance matrix into the right way
		 */
		if (e instanceof Point_2D) {
			h = getTransformationMatrix();
			covariance = getCov();
		} else if (e instanceof Line_2D) {
			// The transformation Matrix for lines is H-t
			Matrix Hinv = getTransformationMatrix().invert();
			h = Hinv.trans();
			// Jacobi for transpose
			Matrix A=getTransposeTransMatrix(getDimRow());
			// Jacobi for inversion
			Matrix J=h.kronecker(h.trans());
			covariance =
				A.mult(J.mult(getCov()).mult(J.trans())).mult(A.trans());
		}
		
		return transform(h, covariance, e, e.getClass());
	}

	/** Condition that element with a set of conditioning factors.
	 *  Because the stored homography matrix is the one for 2D points,
	 *  the condition-factor-matrix for Point_2D is used.
	 * @param f conditioning factors ( two factors for transformations!)
	 */
	public void doCondition(double[] f) {
		Point_2D dummy= new Point_2D();
		Matrix W0=dummy.getConditionMatrix(f[0]);
		Matrix W1=dummy.getConditionMatrix(f[1]);
		Matrix Id=LinearAlgebraFactory.getMatrixId(getDimRow());
		
		// H0=W(fstrich)*H, vec(ABC)=(Ct kron A) vec(B) 
		Matrix A = Id.kronecker(W1);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());

		// H1=H*W-1(f) 
		A = W0.invert().trans().kronecker(Id);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());
	}

	/** The the Jacobian after the first entity of a trilinear relation.
	 *  second=Transformation*first
	 *  For this computation the second entity and the unkown=transformation is needed.
	 * @param e the second Entity of the relation.
	 *
	 */
	public Matrix getJacobianAfterFirst(Entity e) {
		if (e.getClass().equals(Point_2D.class)) {
			Point_2D p = (Point_2D) e;
			return p.SkewSym().mult(getTransformationMatrix());
		}
		if (e.getClass().equals(Line_2D.class)) {
			Line_2D l= (Line_2D) e;
			return (getTransformationMatrix().trans().mult(l.getV())).SkewSym().mult(-1.0);
		}
		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());
	}

	/** The the Jacobian after the second entity of a trilinear relation.
	 *  second=Transformation*first
	 * For this computation the first entity and the unkown=transformation is needed.
	 * @param e the first Entity of the relation.
	 *
	 */
	public Matrix getJacobianAfterSecond(Entity e) {
		if (e.getClass().equals(Point_2D.class)) {
			Point_2D p = (Point_2D) e;
			return getTransformationMatrix().mult(p.getV()).SkewSym().mult(-1.0);
		}
		if (e.getClass().equals(Line_2D.class)) {
			Line_2D l= (Line_2D) e;
			return l.SkewSym().mult(getTransformationMatrix().trans());
		}
		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());
	}
	
	/** Get the number of rows of the transformation matrix.
	 * @return the number of rows of the transformation matrix.
	 */
	public int getDimRow() {
		return DIM;
	}
	
	/** Get the number of columns of the transformation matrix.
	 * @return the number of columns of the transformation matrix.
	 */
	public int getDimCol() {
		return DIM;
	}


} // end Homography_2D
