/** Java class "Homography_3D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * <p>
 * This class contains constructors to build a Homography_3D. Its matrix has 16 
 * entries, but 15 needs to be set due to homogenity
 * A Homography_3D consists of:  	
 * <ul>
 * 		<li> Transformation(3) </li> 
 * 		<li> Rotation(3), </li>
 * 		<li> Scale (3), </li>
 * 		<li> Shear (3), </li>
 * 		<li> projective part (3) </li>
 * </ul>
 * @author Norbert Fischer
 * 
 * @version 1.1 (26.1.2004) Achim Knigs
 * 		Changed Constructor Homography_3D(double hom[], Matrix cov) to accept 16
 * 		parameters and count up the rows befor the colums. 
 */
public class Homography_3D extends Transformation {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	/**
	 * dimension must be set. Otherwise it's not possible to gain
	 * matrix from vector!!!
	 * This DIM = 4 is valid FOR ALL SUBCLASSES xxx_3D !   
	 */
	protected static final int DIM = 4;

	/**
	 * standard constructor. Creates an identity matrix
	 */
	public Homography_3D() {
		setV(LinearAlgebraFactory.getMatrixId(DIM).vec());
		setCov(LinearAlgebraFactory.getMatrix(DIM, DIM));
	}

	/**
	 * Creates a homography by an affinity and the projective part 
	 * @param aff   contains the affinity part
	 * @param projective   a vector with dimension 3. Contains the 
	 * 					  projective part of Homography_3D
	 */
	public Homography_3D(Affinity_3D aff, Vector projective) {
		this(aff, projective, null);
	}

	/**
	 * Creates a homography by an affinity and the projective part
	 * with covariance. No error propagation will be performed 
	 * @param aff   contains the affinity part
	 * @param projective   a vector with dimension 3. Contains the 
	 * 					  projective part of Homography_3D
	 * @param cov   16x16 matrix
	 */
	public Homography_3D(Affinity_3D aff, Vector projective, Matrix cov) {
		if (projective.getDim() != DIM - 1) {
			throw new IllegalArgumentException(
				"Projective vector must " + "have three entries!");
		}
		Vector tmpV = aff.getV();
		tmpV.set(3, projective.get(0));
		tmpV.set(7, projective.get(1));
		tmpV.set(11, projective.get(2));
		setV(tmpV);

		//FFG
		/*
		 * null comes from Homography_3D(Affinity_3D aff, Vector projective)
		 * It's a kind of flag to show, that FFG should be performed
		 */
		if (cov == null) {
			//Because this homography CONSISTS of aff and a projetive part, no
			//error propagation could done (between what, if so?)
			setCov(aff.getCov());
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else {
			checkCovDim(cov, DIM * DIM);
			setCov(cov);
		}
	}

	/**
	 * sets Homography_3D by hand. positions are:
	 * <pre>
	 *     |hom[00]	hom[01]	hom[02]	hom[03]|
	 *     |hom[04] hom[05] hom[06]	hom[07]|
	 *     |hom[08] hom[09] hom[10] hom[11]|
	 *     |hom[12] hom[13] hom[14] hom[15]|
	 * </pre>
	 * @param hom    a double array containing the values. Must be 
	 * 			     dimension 16
	 */
	public Homography_3D(double hom[]) {
		//Why write the same code twice, eh? :-)
		this(hom, LinearAlgebraFactory.getMatrix(DIM * DIM, DIM * DIM));
	}

	/**
	 * sets Homography_3D by hand with covariance. positions are:
	 * <pre>
	 *     |hom[00]	hom[01]	hom[02]	hom[03]|
	 *     |hom[04] hom[05] hom[06]	hom[07]|
	 *     |hom[08] hom[09] hom[10] hom[11]|
	 *     |hom[12] hom[13] hom[14] hom[15]|
	 * </pre>
	 * @param hom   a double array containing the values. Must be 
	 * 			    dimension 16
	 * @param cov   16x16 matrix
	 */
	public Homography_3D(double hom[], Matrix cov) {
		if (hom.length != (DIM * DIM)) {
			throw new IllegalArgumentException(
				"Array must have " + DIM*DIM + " values!");
		}
		Vector tmpV = LinearAlgebraFactory.getVector(DIM * DIM);
		tmpV.set(0,hom[0]);
		tmpV.set(1,hom[4]);
		tmpV.set(2,hom[8]);
		tmpV.set(3,hom[12]);
		tmpV.set(4,hom[1]);
		tmpV.set(5,hom[5]);
		tmpV.set(6,hom[9]);
		tmpV.set(7,hom[13]);
		tmpV.set(8,hom[2]);
		tmpV.set(9,hom[6]);
		tmpV.set(10,hom[10]);
		tmpV.set(11,hom[14]);
		tmpV.set(12,hom[3]);
		tmpV.set(13,hom[7]);
		tmpV.set(14,hom[11]);
		tmpV.set(15,hom[15]);
		setV(tmpV);
		
		checkCovDim(cov, DIM * DIM);
		setCov(cov);
	}
	
	/**
	 * Constructs a homography out of a given homography matrix and
	 * a covariance matrix for the given homography.
	 * @param hom   a 4x4 homography matrix 
	 * @param cov   16x16 covariance matrix
	 */
	public Homography_3D(Matrix hom, Matrix cov) {
		if ((hom.getRowDimension() != DIM)
				|| (hom.getColumnDimension() != DIM)) {
				throw new IllegalArgumentException(
					"Homography_3D dimensions must be "
						+ DIM
						+ "x"
						+ DIM
						+ ".");
			}
		
		setV(hom.vec());
		
		checkCovDim(cov, DIM * DIM);
		setCov(cov);
	}
	
	/** Creates a homography out of two given homographies: H=h1*h2
	 * @param h1 the first homography
	 * @param h2 the second homography
	 * 
	 */
	public Homography_3D(Homography_3D h1, Homography_3D h2) {
		setV((h1.getTransformationMatrix().
				  mult(h2.getTransformationMatrix()).vec()));
				  
		//FFG	
		if (h1.cov!=null && h2.cov!=null)	
		{
			setCov(errorProp(h1,h2));
		}
	}


	/**
	 * Creates a Homography_3D by 15 parameters:
	 * <ul>
	 * 			<li> Translation (3) </li> 
	 * 			<li> Rotation (3)  </li>
	 * 			<li> scale (3)  </li>
	 * 			<li> shear (3) </li>
	 * 			<li> projective part of homography (3) </li>
	 * </ul>
	 * error propagation is done automatically.
	 * @param trans   a given Translation_3D
	 * @param rot   a given Rotation_3D
	 * @param scale   a Vector with dimension 3 containing the scale
	 * @param shear   Vector containing 6 entries with order: 
	 * 			<pre>             xy, xz, yz, yx, zx, zy </pre>
	 * 		e.g. xy, xz means: x-axis is sheared, with proportional
	 * 			 			   amount of y and z: x' = x + xy*y + xz*z <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 * @param projective   a Vector with dimension 3
	 */
	public Homography_3D(
		Translation_3D trans,
		Rotation_3D rot,
		Vector scale,
		Vector shear,
		Vector projective) {
		//Why write the same code twice, eh? :-)
		this(new Motion_3D(trans, rot), scale, shear, projective, null);
	}

	/**
	 * Creates a Homography_3D by 15 parameters PLUS covariance Matrix:
	 * <ul>
	 * 			<li> Translation (3) </li> 
	 * 			<li> Rotation (3)  </li>
	 * 			<li> scale (3)  </li>
	 * 			<li> shear (3) </li>
	 * 			<li> projective part of homography (3) </li>
	 * </ul>
	 * No error propagation will be performed
	 * @param trans   a given Translation_3D
	 * @param rot   a given Rotation_3D
	 * @param scale   a Vector with dimension 3 containing the scale
	 * @param shear   Vector containing 6 entries with order: 
	 * 			<pre>             xy, xz, yz, yx, zx, zy </pre>
	 * 		e.g. xy, xz means: x-axis is sheared, with proportional
	 * 			 			   amount of y and z: x' = x + xy*y + xz*z <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 * @param projective   a Vector with dimension 3
	 * @param cov   16x16 matrix
	 */
	public Homography_3D(
		Translation_3D trans,
		Rotation_3D rot,
		Vector scale,
		Vector shear,
		Vector projective,
		Matrix cov) {
		this(new Motion_3D(trans, rot), scale, shear, projective, cov);
	}

	/**
	 * Creates a Homography_3D by 15 parameters:
	 * <ul>
	 * 			<li> Motion (6) </li> 
	 * 			<li> scale (3) </li>
	 * 			<li> shear (3) </li>
	 * 			<li> projective part of homography (3) </li>
	 * </ul>
	 * Error propagation is done automatically.
	 * @param mot   a given Motion_3D
	 * @param scale   a Vector with dimension 3 containing the scale
	 * @param shear   Vector containing 6 entries with order: 
	 * 			<pre>             xy, xz, yz, yx, zx, zy </pre>
	 * 		e.g. xy, xz means: x-axis is sheared, with proportional
	 * 			 			   amount of y and z: x' = x + xy*y + xz*z <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 * @param projective   a Vector with dimension 3
	 */
	public Homography_3D(
		Motion_3D mot,
		Vector scale,
		Vector shear,
		Vector projective) {
		this(mot, scale, shear, projective, null);
	}

	/**
	 * Creates a Homography_3D by 15 parameters PLUS covariance Matrix:
	 * <ul>
	 * 			<li> Motion (6) </li> 
	 * 			<li> scale (3) </li>
	 * 			<li> shear (3) </li>
	 * 			<li> projective part of homography (3) </li>
	 * </ul>
	 * No error propagation will be performed
	 * @param mot   a given Motion_3D
	 * @param scale   a Vector with dimension 3 containing the scale
	 * @param shear   Vector containing 6 entries with order: 
	 * 			<pre>             xy, xz, yz, yx, zx, zy </pre>
	 * 		e.g. xy, xz means: x-axis is sheared, with proportional
	 * 			 			   amount of y and z: x' = x + xy*y + xz*z <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 * @param projective   a Vector with dimension 3
	 * @param cov   16x16 matrix
	 */
	public Homography_3D(
		Motion_3D mot,
		Vector scale,
		Vector shear,
		Vector projective,
		Matrix cov) {
		//check dimensions
		if (scale.getDim() != DIM - 1 || projective.getDim() != DIM - 1) {
			throw new IllegalArgumentException(
				"scale or projective vector has not " + "three entries!");
		}
		if (shear.getDim() != 6) {
			throw new IllegalArgumentException(
				"shear vector has not " + "six entries!");
		}

		//set Motion
		Matrix tmpM = mot.getTransformationMatrix();

		//set scale
		tmpM.set(0, 0, tmpM.get(0, 0) * scale.get(0));
		tmpM.set(1, 1, tmpM.get(1, 1) * scale.get(1));
		tmpM.set(2, 2, tmpM.get(2, 2) * scale.get(2));

		//set shear
		/*
		 * Found under: 
		 * http://www.mpi-sb.mpg.de/units/ag4/teaching/uebung/lecture02.pdf
		 * http://www.cs.fit.edu/wds/classes/cse5255/thesis/shear/shear.html#3dShear
		 */
		Matrix shearMat = LinearAlgebraFactory.getMatrixId(DIM);
		shearMat.set(0, 1, shear.get(0)); //xy: y amount to x 
		shearMat.set(0, 2, shear.get(1)); //xz: z amount to x
		shearMat.set(1, 2, shear.get(2)); //yx: x amount to y
		shearMat.set(1, 0, shear.get(3)); //yz: ...
		shearMat.set(2, 0, shear.get(4)); //zx:
		shearMat.set(2, 1, shear.get(5)); //zy: 

		tmpM=tmpM.mult(shearMat);

		//set projective part 
		tmpM.set(3, 0, projective.get(0));
		tmpM.set(3, 1, projective.get(1));
		tmpM.set(3, 2, projective.get(2));

		setV(tmpM.vec());

		//FFG
		/*
		 * null comes from constructors above
		 * It's a kind of flag to show, that FFG should be performed
		 */
		if (cov == null) {
			//Because this homography CONSISTS of aff and a projetive part, no
			//error propagation could done (between what, if so?)			
			setCov(mot.getCov());
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else {
			checkCovDim(cov, DIM * DIM);
			setCov(cov);
		}
	}

	///////////////////////////////////////
	// operations

	/** Get the number of rows of the transformation matrix.
	 * @return the number of rows of the transformation matrix.
	 */
	public int getDimRow() {
		return DIM;
	}
	
	/** Get the number of columns of the transformation matrix.
	 * @return the number of columns of the transformation matrix.
	 */
	public int getDimCol() {
		return DIM;
	}


	/** Condition that element with a set of conditioning factors.
	 *  Because the stored homography matrix is the one for 3D points,
	 *  the condition-factor-matrix for Point_3D is used.
	 * @param f conditioning factors ( two factors for transformations!)
	 */
	public void doCondition(double[] f) {
		Point_3D dummy = new Point_3D();
		Matrix W0 = dummy.getConditionMatrix(f[0]);
		Matrix W1 = dummy.getConditionMatrix(f[1]);
		Matrix Id = LinearAlgebraFactory.getMatrixId(getDimRow());
		
		// H0=W(fstrich)*H, vec(ABC)=(Ct kron A) vec(B) 
		Matrix A = Id.kronecker(W1);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());

		// H1=H*W-1(f) 
		A = W0.invert().trans().kronecker(Id);
		v = A.mult(v);
		cov = A.mult(cov).mult(A.trans());
	}

	/**
	 * Creates a transformation-matrix for points. Actually, it is the same like the
	 * normal transformation-matrix
	 * @return a (4 x 4)-Matrix
	 */
	public Matrix getPointMatrix() {
		return getTransformationMatrix();
	} // end getPointMatrix        

	/**
	 * Creates out of this transformation a transformation-matrix for Line_3D
	 * See Stephans Diss, pg 52
	 * @return a (6 x 6)-Matrix 
	 */
	public Matrix getLineMatrix() {
		//pairs of colums of Homography (indices from diss -1 !!)
		int pairArray[][] = { { 3, 0 }, {
				3, 1 }, {
				3, 2 }, {
				1, 2 }, {
				2, 0 }, {
				0, 1 }
		};

		//like everytime: less to write								
		Matrix h = getTransformationMatrix();
		Vector h_k = LinearAlgebraFactory.getVector(4);
		Matrix lineMatrix = LinearAlgebraFactory.getMatrix(6, 6);

		for (int i = 0; i < 6; i++) {
			/*
			 * read jth - column (Indices from ith-pair)
			 * and create Pi-matrix
			 */
			Matrix Pi =
				Point_3D.Pi( LinearAlgebraFactory.getVector(
					h.get(0, pairArray[i][0]),
					h.get(1, pairArray[i][0]),
					h.get(2, pairArray[i][0]),
					h.get(3, pairArray[i][0])));

			//set second vector  H^{(k)} with kth-column
			h_k.set(0, h.get(0, pairArray[i][1]));
			h_k.set(1, h.get(1, pairArray[i][1]));
			h_k.set(2, h.get(2, pairArray[i][1]));
			h_k.set(3, h.get(3, pairArray[i][1]));

			//now calc ith-column of Line_3D-Homography
			Vector result_column = Pi.mult(h_k);

			//insert new column into Line_3D-Homography
			for (int j = 0; j < 6; j++) {
				lineMatrix.set(j, i, result_column.get(j));
			}
		}

		return lineMatrix;
	} // end getLineMatrix        

	/**
	 * Creates out of this transformation a transformation-matrix for Plane_3D
	 *  H_plane = H^{-T}
	 * @return a (4 x 4)-Matrix 
	 */
	public Matrix getPlaneMatrix() {
		return getTransformationMatrix().invert().trans();
	} // end getPlaneMatrix        

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom() {
		return 15;
	}

	/**
	 * Transforms an entity with this transformation. The new entity contains 
	 * error propagation of tranform and old entity
	 * @param e   Entity to transform  
	 * @return A new Entity with applied transformation and error propagation
	 */
	public Entity transform(Entity e) {

		Matrix h = null; //special Transformation matrix for Entity
		Matrix covariance = null;
		//because h is linked with entity, cov-matrix needs
		//to be "transformed"		

		/*
		 * Problem: It is necessary to know the instance of Entity e because each 
		 * 			entity needs a special error propagation. Especially Line_3D
		 * 			needs a very unique treatment :-)
		 */
		if (e instanceof Point_3D) {
			h = getPointMatrix();
			covariance = getCov();
		} else if (e instanceof Line_3D) { //See Stephan's Diss, page 52  
			h = getLineMatrix();

			/*
			 * now create jacobian for error propagation
			 * This is due to the formula Hl(i)=Pi(H(j)).mult(H(k)).
			 * For every column of Hl (i=1,6) there exist the above formula
			 * with (j,k)= (4,1), ..., (1,2). So for every column, the jacobian
			 * Pi(H(j)) is no zero and the Jacobian -PI(H(k)) is not zero, because
			 * Hl(i)=Pi(H(j)).mult(H(k))=-Pi(H(k)).mult(H(j))
			 * 
			 */
			Matrix jacobian = LinearAlgebraFactory.getMatrix(36, 16);
			Vector v = getV();
			Matrix Pi = null;
			
			// Pi for forth column PI(H(4))
			Pi = Point_3D.Pi(LinearAlgebraFactory.getVector(v.get(12), v.get(13), v.get(14), v.get(15)));
			jacobian.slice(0, 5, 0, 3, Pi); //slice from  (0,0) to  (5,3)
			jacobian.slice(6, 11, 4, 7, Pi); //slice from  (6,4) to (11,7)
			jacobian.slice(12, 17, 8, 11, Pi); //slice from (12,8) to (17,11)

	        //	Pi for second column PI(H(2))
			Pi = Point_3D.Pi(LinearAlgebraFactory.getVector(v.get(4), v.get(5), v.get(6), v.get(7)));
			jacobian.slice(18, 23, 8, 11, Pi); //slice from (18,8) to (23,11)
			jacobian.slice(6, 11, 12, 15, Pi.mult(-1)); 
			jacobian.slice(30, 35, 0, 3, Pi.mult(-1)); 

			//PI(H(3))
			Pi = Point_3D.Pi(LinearAlgebraFactory.getVector(v.get(8), v.get(9), v.get(10), v.get(11)));
			jacobian.slice(24, 29, 0, 3, Pi); //slice from (24,0) to (29,3)
			jacobian.slice(12, 17, 12, 15, Pi.mult(-1)); 
			jacobian.slice(18, 23,  4,  7, Pi.mult(-1));
			
			// PI(H(1))
			Pi = Point_3D.Pi(LinearAlgebraFactory.getVector(v.get(0), v.get(1), v.get(2), v.get(3)));
			jacobian.slice(30, 35, 4,  7, Pi); //slice from (30,4) to (35,7)
			jacobian.slice(0,   5,12, 15, Pi.mult(-1));
			jacobian.slice(24, 29, 8, 11, Pi.mult(-1)); 
			
			//let's do error propagation! [36x16] * [16x16] * [16x36]
			covariance = jacobian.mult(getCov()).mult(jacobian.trans());
		} else if (e instanceof Plane_3D) {
			h = getPlaneMatrix();
			
			// Jacobi for transpose
			Matrix A=getTransposeTransMatrix(getDimRow());
			// Jacobi for inversion
			Matrix J=h.kronecker(h.trans());
			covariance =
				A.mult(J.mult(getCov()).mult(J.trans())).mult(A.trans());
		}
		
		return transform(h, covariance, e, e.getClass());
	}


	/** The the Jacobian after the first entity of a trilinear relation.
	 *  For this computation the second entity and the unkown=transformation is needed.
	 *  esecond= T*efirst
	 *  For Lines: The used equation comes from formula 3.105 in
	 *             S. Heuel dissertation. The formulas in table 3.9
	 *             are derived from 3.106 and therefore are slightly
	 *             different. Both possibilities are ok.
	 * @param e the second Entity of the relation.
	 */
	public Matrix getJacobianAfterFirst(Entity e) {
		if (e.getClass().equals(Point_3D.class)) {
			Point_3D p = (Point_3D) e;
			return p.Pi().mult(getTransformationMatrix());
		}
		if (e.getClass().equals(Line_3D.class)) {
			Line_3D l = (Line_3D) e;
			Matrix Id = LinearAlgebraFactory.getMatrixId(4);
			return Id.kronecker(l.GammaDual().mult(getTransformationMatrix())).mult(Line_3D.JGamma());
//			return l.GammaDual().trans().mult(
//				getTransformationMatrix()).kronecker(
//				Id).mult(
//				Line_3D.JGamma());
		}
		if (e.getClass().equals(Plane_3D.class)) {
			return Plane_3D.PiDual(
				getTransformationMatrix().trans().mult(e.getV())).mult(-1);
		}
		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());
	}

	/** The the Jacobian after the second entity of a trilinear relation.
	 * For this computation the first entity and the unkown=transformation is needed.
	 *  esecond= T*efirst
	 *  For Lines: The used equation comes from formula 3.105 in
	 *             S. Heuel dissertation. The formulas in table 3.9
	 *             are derived from 3.106 and therefore are slightly
	 *             different. Both possibilities are ok.
	 * @param e the first Entity of the relation.
	 */
	public Matrix getJacobianAfterSecond(Entity e) {
		if (e.getClass().equals(Point_3D.class)) {
			return Point_3D.Pi(getTransformationMatrix().mult(e.getV())).mult(-1);
		}
		if (e.getClass().equals(Line_3D.class)) {
			Line_3D l = (Line_3D) e;
			Matrix Id = LinearAlgebraFactory.getMatrixId(4);
			return l.Gamma().trans().mult(getTransformationMatrix().trans()).kronecker(Id).mult(Line_3D.JGamma()).mult(Line_3D.DualMatrixC());
//			return Id
//				.kronecker(l.Gamma().mult(getTransformationMatrix().trans()))
//				.mult(Line_3D.JGamma())
//				.mult(Line_3D.DualMatrixC());
		}
		if (e.getClass().equals(Plane_3D.class)) {
			Plane_3D A = (Plane_3D) e;
			return A.PiDual().mult(getTransformationMatrix().trans());
		}
		throw new IllegalArgumentException(
			"Illegal trilinear relation between "
				+ getClass()
				+ " and "
				+ e.getClass());

	}

} // end Homography_3D
