/** Java class "Line_2D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Implementation of Line_2D
 * 
 * @author Norbert Fischer
 * 
 * @version 1.1 (25.1.2004)
 * 		Changed: Changed Constructor Line_2D(Point,Point) 
 * 		to not use Construction Class anymore.
 * 		Included calculations into Constructor itself. By Achim Knigs
 */
public class Line_2D extends Object_2D {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 3;
	private static final int DOF = 2;

	///////////////////////////////////////
	// operations

	/**
	 * Constructor for a new Line_2D
	 * @param _v a Vector with dimension 3. Contains the coordinates of the new Line_2D
	 * @param _cov 3x3 covariance matrix
	 */
	public Line_2D(Vector _v, Matrix _cov) {
		checkElement(_v, DIM); //has v the right dimension?
		checkCovDim(_cov, DIM); //also the matrix?
		v = _v.copy();
		cov = _cov.copy();
	}

	/**
	 * Creates a line be two Point_2D. Error propagation is done automatically
	 * @param x   first Point_2D
	 * @param y   second Point_2D
	 */
	public Line_2D(Point_2D x, Point_2D y) {
		Line_2D l = new Line_2D();
		l.setV( x.SkewSym().mult(y.getV()) );
		// Set Covariance Matrix:
		l.setCov( calcCov( x.SkewSym(), x.getCov(), 
						   y.SkewSym(), y.getCov()));
		setV(l.getV());
		checkElement(l.getV(), DIM);
		setCov(l.getCov());
	} // end Line_2D        

	/**
	 * Creates a new Line_2D. This line has no error, covariance matrix is set to 0
	 * @param u homogeneous part
	 * @param v homogeneous part
	 * @param w euklidean part
	 */
	public Line_2D(double u, double v, double w) {
		this(u, v, w, LinearAlgebraFactory.getMatrix(DIM, DIM));
	} // end Line_2D        

	/**
	 * Creates a new Line_2D with covariance matrix.
	 * @param u   homogeneous part
	 * @param v   homogeneous part
	 * @param w   euclidean part
	 * @param cov   3x3 Matrix
	 */
	public Line_2D(double u, double v, double w, Matrix cov) {

		checkCovDim(cov, DIM);
		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, u);
		vec.set(1, v);
		vec.set(2, w);

		setV(vec);
		checkElement(vec, DIM);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Line_2D        

	/**
	 * Angle-distance constructor for Line_2D. Note that Line_2D has 
	 * degree of freedom 2, so this is constructor with least parameters
	 * @param angle   angle between normal of line and x-axis
	 * @param distance   distance from origin
	 */
	public Line_2D(double angle, double distance) {
		this(angle, distance, LinearAlgebraFactory.getMatrix(DIM, DIM));
	} // end Line_2D        

	/**
	 * Angle-distance constructor for Line_2D with covariance. Note that 
	 * Line_2D has degree of freedom 2, so this is constructor with least 
	 * parameters
	 * @param angle   angle between normal of line and x-axis
	 * @param distance   distance from origin
	 * @param cov   3x3 Matrix
	 * TODO: Give 2x2 covariance of angle and distance instead
	 *       of covariance of line parameters
	 */
	public Line_2D(double angle, double distance, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, Math.cos(angle));
		v.set(1, Math.sin(angle));
		v.set(2, -distance);

		setV(v);
		setCov(cov.copy());
	} // end Line_2D        

	/**
	 * Default constructor
	 */
	public Line_2D() {
	} // end Line_2D

	/**
	 * Estimates a new line.
	 * @param rprops an array of constraints.
	 */
	public Line_2D(RelationalProperty[] rprops) {
		if (!estimate(rprops))
			throw new IllegalArgumentException("Convergence not reached!");
	}

	/**
	 * returns the euclidean part of this line
	 * @return a Vector with 1 element
	 */
	public Vector getEuclideanPart() {
		Vector vec = LinearAlgebraFactory.getVector(1);
		vec.set(0, getV().get(2));

		return vec;
	} // end getEuclideanPart        

	/**
	 * returns the homogeneous part of this plane
	 * @return a Vector with 2 elements
	 */
	public Vector getHomogeneousPart() {
		Vector vec = LinearAlgebraFactory.getVector(2);
		vec.set(0, getV().get(0));
		vec.set(1, getV().get(1));

		return vec;
	} // end getHomogeneousPart        

	/**
	 * calculates the distance between this line and a Point_2D
	 * @param  e   Point_2D to calculate distance to
	 * @return a positive double containing the distance 
	 */
	public double getDistance(Entity e) {
		if (!(e instanceof Point_2D))
			throw new IllegalArgumentException("Line_2D.distance: distance not possible");
		// enumerator
		double z = Math.abs(getV().scalarmult(e.getV()));

		// denominator
		double n = getHomogeneousPart()
				.mult(e.getHomogeneousPart().get(0))
				.getLength();

		return z / n;
	}

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	protected Matrix getConditionMatrix(double f) {
		Matrix w = LinearAlgebraFactory.getMatrix(3, 3);
		w.set(0, 0, 1);
		w.set(1, 1, 1);
		w.set(2, 2, f);

	    return w;
	}

	/**
	 * returns a copy of this line. Actually it clones a Line_2D.
	 */
	public Line_2D copy() {
		return new Line_2D(v, cov);
	}

	/**
	 * returns the dimension of this line=3
	 */
	public int getDimension() {
		return DIM;
	}

	/**
	 * returns dof=2 of this line
	 */
	public int getDegreesOfFreedom() {
		return DOF;
	}
	
	/**
	* returns the skew symmetric matrix  of this line
	* @return 3x3 Matrix with rank 2 
	*/
	public Matrix SkewSym() {
		return SkewSym(getV());
	}


	/**
	* returns the skew symmetic matrix of a vector
	* @return 3x3 Matrix with rank 2 
	*/
	public static Matrix SkewSym(Vector v) {
		Matrix S = LinearAlgebraFactory.getMatrix(3, 3);
		double a = v.get(0);
		double b = v.get(1);
		double c = v.get(2);
		S.set(0, 1, -c);
		S.set(0, 2, b);
		S.set(1, 0, c);
		S.set(1, 2, -a);
		S.set(2, 0, -b);
		S.set(2, 1, a);
		return S;
	}

	/**
	 * normalizes in that way, that the homogeneous part will be 1
	 */
	public void normalizeEuclidean() {
		Vector l_h = getHomogeneousPart();
		double factor = l_h.getLength();

		Matrix jacobi = LinearAlgebraFactory.getMatrixId(DIM);
		
		Matrix help=LinearAlgebraFactory.getMatrixId(2);
		
		help= help.minus(l_h.getMatrix().mult(l_h.getMatrix().trans()).mult(1.0/(factor*factor)));

		jacobi.slice(0, 1, 0,1, help);

		Vector tmp = l_h.mult(-getV().get(2) / (factor * factor));
		jacobi.set(2, 0, tmp.get(0));
		jacobi.set(2, 1, tmp.get(1));
		
		jacobi=jacobi.mult(1.0/factor);

		setCov(jacobi.mult(getCov()).mult(jacobi.trans()));

		setV(getV().mult(1.0 / factor));
	}

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_2D.class))
				return getV().getMatrix().trans();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_2D.class))
				return getV().SkewSym();

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_2D.class)) {
				Vector tmp = LinearAlgebraFactory.getVector(3);
				Vector lh = getHomogeneousPart();
				tmp.set(0, -1.0 * lh.get(1));
				tmp.set(1, lh.get(0));
				return tmp.getMatrix().trans();
			}

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_2D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(1, 3);
				Vector lh = getHomogeneousPart();
				tmp.set(0, 0, lh.get(0));
				tmp.set(0, 1, lh.get(1));
				return tmp;
			}
		}

		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public int getDofOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_2D.class))
				return 1;
		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_2D.class))
				return 2;

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_2D.class))
				return 1;

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_2D.class))
				return 1;

		}

		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());
	}

	/** Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation.
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_2D.class))
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else
					return getCov();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_2D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(3, 3);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}

			}
		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_2D.class)) {
				if (row!=0)
					throw new RuntimeException("Row out of range!");
				else {
					Matrix result = LinearAlgebraFactory.getMatrix(3, 3);
					int[] xxx = {0,1};
					int[] yyy = {1,0};
					result.slice(yyy, yyy, getCov().slice(xxx, xxx));
					return result;
				}
			}

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_2D.class)) {
				if (row!=0)
					throw new RuntimeException("Row out of range!");
				else {
					return getCov().slice(0,1,0,1);
				}
			}
		}

		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}
} // end Line_2D
