/** Java class "Line_3D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Implementation of Line_3D
 * 
 * 
 * @author Norbert Fischer
 * @version 1.3 (17.5.2004)
 * 		Changed some comments. Changed getDual() to give back the expected
 * 		Line, not scaled with -1
 * @version 1.2 (25.1.2004)
 * 		Changed Constructor Line_3D(Point,Point) and Line_3D(Plane,Plane) 
 *		to not use Construction Class
 * 		anymore, calculation is done in Constructor itself now. By Achim Knigs
 * @version 1.1 (9.2.2003)
 * 		Fixed: getJacobianHomogeneousPart() returned wrong matrix
 */
public class Line_3D extends Object_3D {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 6;
	private static final int DOF = 4;

	///////////////////////////////////////
	// operations

	/**
	 * Constructor for a new Line_3D
	 * @param _v a Vector with dimension 6. Contains the coordinates of the new Line_2D
	 * @param _cov 6x6 covariance matrix
	 */
	public Line_3D(Vector _v, Matrix _cov) {
		checkElement(_v, DIM);
		checkCovDim(_cov, DIM);
		v = _v.copy();
		cov = _cov.copy();
	}

	/**
	 * Creates a new Line_3D by specifying all elements directly. 
	 * covariance matrix is set to 0 so it has no error
	 * Be careful: The Line must fullfill the Pluecker condition !
	 *             This is NOT tested !
	 */
	public Line_3D(
		double l1,
		double l2,
		double l3,
		double l4,
		double l5,
		double l6) {

		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, l1);
		v.set(1, l2);
		v.set(2, l3);
		v.set(3, l4);
		v.set(4, l5);
		v.set(5, l6);

		checkElement(v, DIM);
		setV(v);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);

	} // end Line_3D        

	/**
	 * Creates a new Line_3D with covariance by specifying all elements directly. 
	 * @param cov  6x6 Matrix	
	 */
	public Line_3D(
		double l1,
		double l2,
		double l3,
		double l4,
		double l5,
		double l6,
		Matrix cov) {

		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, l1);
		v.set(1, l2);
		v.set(2, l3);
		v.set(3, l4);
		v.set(4, l5);
		v.set(5, l6);

		checkElement(v, DIM);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Line_3D        

	/**
	 * Creates a new Line_3D with covariance by specifying all elements directly. 
	 * @param cov  6x6 Matrix
	 * @param testPlueckerCondition if false do NOT test pluecker condition.
	 *        Be careful with that, because if Pluecker condition is not met,
	 *        this is not a 3D-Line !.
	 */
	public Line_3D(
		double l1,
		double l2,
		double l3,
		double l4,
		double l5,
		double l6,
		Matrix cov,
		boolean testPlueckerCondition) {

		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, l1);
		v.set(1, l2);
		v.set(2, l3);
		v.set(3, l4);
		v.set(4, l5);
		v.set(5, l6);

		checkElement(v, DIM);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());

		if (testPlueckerCondition)
			if (!checkPlueckerCondition(ZERO_THRESHOLD_CONSTR))
				throw new IllegalArgumentException("Pluecker condition is not ok !");
	} // end Line_3D        

	/**
	 * Creates a Line_3D by an intersection of two Plane_3D. Error propagation is
	 * done automatically
	 * @param plane1    first Plane_3D
	 * @param plane2    second Plane_3D
	 */
	public Line_3D(Plane_3D plane1, Plane_3D plane2) {
		Line_3D l = new Line_3D();
		l.setV(plane1.PiDual().mult(plane2.getV()));

		// Set Covariance Matrix:
		//PiDual(b) ist nach Seite 3 negativ, aber das hebt sich in der Formel raus
		l.setCov(
			calcCov(
				plane1.PiDual(),
				plane1.getCov(),
				plane2.PiDual(),
				plane2.getCov()));
		checkElement(l.getV(), DIM);
		setV(l.getV());
		setCov(l.getCov());
	} // end Line_3D

	/**
	 * estimates a new line.
	 * @param rprops an array of constraints.
	 */
	public Line_3D(BiRelationalProperty[] rprops) {
		if (!estimate(rprops)) {
			throw new IllegalArgumentException("Convergence not reached!");
		}
	}

	/**
	 * Creates a Line_3D from two Point_3D. Error propagation is done automatically
	 * @param point1   first Point_3D
	 * @param point2   second Point_3D
	 */
	public Line_3D(Point_3D point1, Point_3D point2) {
		Line_3D l = new Line_3D();
		l.setV(point1.Pi().mult(point2.getV()));

		// Set Covariance Matrix:
		//Pi(x) is negative (see sugr.handbook pg.3) but ep neutralizes this
		l.setCov(
			calcCov(
				point1.Pi(),
				point1.getCov(),
				point2.Pi(),
				point2.getCov()));

		checkElement(l.getV(), DIM);
		setV(l.getV());
		setCov(l.getCov());
	} // end Line_3D        

	/**
	 * Default constructor
	 */
	public Line_3D() {
	} // end Line_3D

	/** Check the pluecker condition.
	 * 
	 * @param zero_threshold a threshold for the scalarproduct beeing zero.
	 * @return true if pluecker condition is ok, otherwise false
	 * 
	 */
	public boolean checkPlueckerCondition(double zero_threshold) {
		if (Math.abs(getEuclideanPart().scalarmult(getHomogeneousPart()))
			> zero_threshold)
			return false;
		return true;
	}

	/**
	 * Create a new Line, which is the Dual of this line. Note that the 
	 * complete line is dualized, also the covariance matrix
	 * @return a Line_3D
	 */
	public Line_3D getDual() {
		Matrix dual = DualMatrixC();

		return new Line_3D(dual.mult(v), dual.mult(cov).mult(dual.trans()));
	} // end getDual        

	/**
	 * returns the euclidean part of this line
	 * @return a Vector with 3 elements
	 */
	public Vector getEuclideanPart() {
		Vector vec = LinearAlgebraFactory.getVector(3);
		vec.set(0, getV().get(3));
		vec.set(1, getV().get(4));
		vec.set(2, getV().get(5));

		return vec;
	} // end getEuclideanPart        

	/**
	 * returns the homogeneous part of this plane
	 * @return a Vector with 3 elements
	 */
	public Vector getHomogeneousPart() {
		Vector vec = LinearAlgebraFactory.getVector(3);
		vec.set(0, getV().get(0));
		vec.set(1, getV().get(1));
		vec.set(2, getV().get(2));

		return vec;
	} // end getHomogeneousPart 

	/**
	  * Compute the (euclidean) distance to another entity. This
	  * is of course not possible for all combinations.
	  * In the case of an illegal combination an IllegalArgumentException
	  * should be thrown.
	  * @return the distance to the given entity
	  * @param e   the entity to which a distance should be computed.
	 */
	public double getDistance(Entity e) {
		if (e instanceof Point_3D) // implemented in point
			return e.getDistance(this);
		if (e instanceof Line_3D) {
			// enumerator
			double z = Math.abs(getDual().getV().scalarmult(e.getV()));
			// denominator
			double n =
				Math.abs(
					getHomogeneousPart()
						.SkewSym()
						.mult(e.getHomogeneousPart())
						.getLength());
			return z / n;

		}
		throw new IllegalArgumentException(
			"Line_3D.distance: distance between "
				+ e.getClass()
				+ " not possible");
	}

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	protected Matrix getConditionMatrix(double f) {
		Matrix w = LinearAlgebraFactory.getMatrix(6, 6);
		w.set(0, 0, 1);
		w.set(1, 1, 1);
		w.set(2, 2, 1);
		w.set(3, 3, f);
		w.set(4, 4, f);
		w.set(5, 5, f);
		return w;
	}

	/**
	 * Returns a copy of this line. Actually it works like clone
	 */
	public Line_3D copy() {
		return new Line_3D(v, cov);
	}

	/**
	 * Returns dimension=6
	 */
	public int getDimension() {
		return DIM;
	}

	/**
	 * Returns dof=4 for this line
	 */
	public int getDegreesOfFreedom() {
		return DOF;
	}

	/**
	 * Returns the gamma-dual of a 6x1 vector. This matrix
	 * differs from that of the dissertation in the sign, because of
	 * other definition of C 
	 */
	public static Matrix GammaDual(Vector v) {
		Matrix GammaD = LinearAlgebraFactory.getMatrix(4, 4);
		double l1 = v.get(0);
		double l2 = v.get(1);
		double l3 = v.get(2);
		double l4 = v.get(3);
		double l5 = v.get(4);
		double l6 = v.get(5);

		GammaD.set(0, 1, -l3);
		GammaD.set(0, 2, l2);
		GammaD.set(0, 3, l4);

		GammaD.set(1, 0, l3);
		GammaD.set(1, 2, -l1);
		GammaD.set(1, 3, l5);

		GammaD.set(2, 0, -l2);
		GammaD.set(2, 1, l1);
		GammaD.set(2, 3, l6);

		GammaD.set(3, 0, -l4);
		GammaD.set(3, 1, -l5);
		GammaD.set(3, 2, -l6);
		return GammaD;
	} // end GammaDual

	/**
	 * Returns the gamma-dual of this line. This matrix
	 * differs from that of the dissertation in the sign, because of
	 * other definition of C 
	 */
	public Matrix GammaDual() {
		return GammaDual(getV());
	}

	/**
	 * Returns the gamma matrix of a 6x1 vector.
	 */
	public static Matrix Gamma(Vector v) {
		Matrix Gamma = LinearAlgebraFactory.getMatrix(4, 4);
		double l1 = v.get(0);
		double l2 = v.get(1);
		double l3 = v.get(2);
		double l4 = v.get(3);
		double l5 = v.get(4);
		double l6 = v.get(5);

		Gamma.set(0, 1, l6);
		Gamma.set(0, 2, -l5);
		Gamma.set(0, 3, -l1);

		Gamma.set(1, 0, -l6);
		Gamma.set(1, 2, l4);
		Gamma.set(1, 3, -l2);

		Gamma.set(2, 0, l5);
		Gamma.set(2, 1, -l4);
		Gamma.set(2, 3, -l3);

		Gamma.set(3, 0, l1);
		Gamma.set(3, 1, l2);
		Gamma.set(3, 2, l3);
		return Gamma;
	} // end Gamma

	/**
	 * Returns the gamma matrix of this line 
	 */
	public Matrix Gamma() {
		return Gamma(getV());
	}

	/**
	 * Returns the 16x6 jacobi-matrix for the line-equality-test
	 * This formula is from the dissertation (3.70). Note that
	 * the different definition of the sign of C influences this result !
	 * @return 16x6 jacobi-matrix
	 */
	public Matrix getLineLineJacobian() {
		return Gamma()
			.trans()
			.kronecker(LinearAlgebraFactory.getMatrixId(4))
			.mult(Line_3D.JGamma()).mult(Line_3D.DualMatrixC());

		/* This is an old code which deliver another Matrix, but the
		 * equal tests deliver (nearly) the same test values !
		Vector L = getV();
		Matrix J = LinearAlgebraFactory.getMatrix(16, 6);
		J.set(0, 0, -L.get(3));
		J.set(0, 4, -L.get(1));
		J.set(0, 5, -L.get(2));
		J.set(1, 1, -L.get(3));
		J.set(1, 3, L.get(1));
		J.set(2, 2, -L.get(3));
		J.set(2, 3, L.get(2));
		J.set(3, 1, -L.get(2));
		J.set(3, 2, L.get(1));
		J.set(4, 0, -L.get(4));
		J.set(4, 4, L.get(0));
		J.set(5, 1, -L.get(4));
		J.set(5, 3, -L.get(0));
		J.set(5, 5, -L.get(2));
		J.set(6, 2, -L.get(4));
		J.set(6, 4, L.get(2));
		J.set(7, 0, L.get(2));
		J.set(7, 2, -L.get(0));
		J.set(8, 0, -L.get(5));
		J.set(8, 5, L.get(0));
		J.set(9, 1, -L.get(5));
		J.set(9, 5, L.get(1));
		J.set(10, 2, -L.get(5));
		J.set(10, 3, -L.get(0));
		J.set(10, 4, -L.get(1));
		J.set(11, 0, -L.get(1));
		J.set(11, 1, L.get(0));
		J.set(12, 4, L.get(5));
		J.set(12, 5, -L.get(4));
		J.set(13, 3, -L.get(5));
		J.set(13, 5, L.get(3));
		J.set(14, 3, L.get(4));
		J.set(14, 4, -L.get(3));
		J.set(15, 0, -L.get(3));
		J.set(15, 1, -L.get(4));
		J.set(15, 2, -L.get(5));
		return J;
		*/
	}

	/** Get the reduced Matrix A and B from a relational 
	 * property. A is the Jacobian of this Entity (the unknown),
	 * B is the Jacobian of the observation (the relational property).
	 * The Jacobian of the oberservation depends (only) on the unknown and
	 * the Jacobian of the unknown depends (only) of the oberservation !
	 * This method is overwritten to handle some the special cases.
	 * 
	 * @param rprop the relational Property
	 * @return A in array element 0, B in array Element 1.
	 */
	protected Matrix[] getReducedAB(RelationalProperty rprop) {
		BiRelationalProperty r = (BiRelationalProperty) rprop;
		if (r.getClass().equals(Equal.class)) {
			// Choose an index so that |Li * Mi | >0
			// Am besten whlt man hier das Maximum von |Li*Mi| !!! (13.1.05 ChB)
			int i=-1;
			double max=-1;
			Vector lv = getV();
			Vector mv = r.getFirstEntity().getV();
			for (int ii=0; ii<6; ii++) {
				double limi = Math.abs(lv.get(ii) * mv.get(ii));
				if (limi>max) {
					max=limi;
					i=ii;
				}
			}
			if (i==-1)
				throw new IllegalArgumentException(
					"Consistent Reduction of 3D Line Jacobian not possible: Change coordinate system, such"
						+ "that |Li*Mi|>0 for at least one i");

			Matrix A = ((Line_3D) r.getFirstEntity()).getReducedDeltaMatrix(i);
			Matrix B = getReducedDeltaMatrix(i).mult(-1.0);
			// -1 for sign preserving!
			Matrix[] AB = reduceMatrix(A, B, r.getDof());
			return AB;
		} 
//		else if (r.getClass().equals(Incident.class)) {
//			if (r.getFirstEntity().getClass().equals(Point_3D.class)) {
//				Matrix A = r.getJacobian();
//				Matrix B = getJacobianOfRelation(r);
//				Matrix[] AB = reduceFirstMatrix(B, A, r.getDof());
//				Matrix xxx = AB[1];
//				AB[1]=AB[0];
//				AB[0]=xxx;				
//				return AB;
//			}
//			else if (r.getFirstEntity().getClass().equals(Plane_3D.class)) {
//				Matrix A = r.getJacobian();
//				Matrix B = getJacobianOfRelation(r);
//				Matrix[] AB = reduceFirstMatrix(B, A, r.getDof());
//				Matrix xxx = AB[1];
//				AB[1]=AB[0];
//				AB[0]=xxx;				
//				return AB;				
//			}
//			else
//				return super.getReducedAB(rprop);
//		}
		else
			return super.getReducedAB(rprop);
	}

	/** Compute Matrix Delta_i of this line. The matrix delta_i
	 *  is delta_i(L)=L*E_i.trans + L_i * I
	 *  (defined in eq. 3.68 of PhD from Stephan Heuel, ipb.)
	 *  This is used for the reduced Jacobian matrixes.
	 * @param i : The index i=0, ..., 5 for which Matrix delta is built.
	 * @return the matrix delta_i
	 */
	public Matrix getDeltaMatrix(int i) {
		Vector unit = LinearAlgebraFactory.getVector(6);
		unit.set(i, 1.0);
		Matrix help = getV().getMatrix().mult(unit.getMatrix().trans());
		Matrix um = LinearAlgebraFactory.getMatrix(6, 6);
		for (int j = 0; j < 6; j++)
			um.set(j, j, getV().get(i));

		help = help.minus(um);
		return help;
	}

	/** Compute reduced Matrix Delta_i of this line. 
	 *  The reduced Matrix two rows are omitted: The i.th (which
	 *  is always zero) and the (i+3) modulo 6, because for this
	 *  row the pluecker condition is not met.
	 *  This is used for the reduced Jacobian matrixes.
	 * @param i : The index i=0, ..., 5 for which Matrix delta is built.
	 * @return the matrix delta_i
	 */
	public Matrix getReducedDeltaMatrix(int i) {
		Matrix result = LinearAlgebraFactory.getMatrix(4, 6);
		Matrix delta = getDeltaMatrix(i);
		int k = 0;
		for (int j = 0; j < 6; j++) {
			if (j != i && (i + 3) % 6 != j) {
				result.setRow(k, delta.getRow(j));
				k++;
			}

		}
		return result;
	}

	/**
	 * Normalizes in that way, that the homogeneous part will be 1
	 */
	public void normalizeEuclidean() {
		Vector l_h = getHomogeneousPart();
		double factor = l_h.getLength();

		Matrix jacobi = LinearAlgebraFactory.getMatrix(DIM, DIM);

		Matrix tmp1 = l_h.getMatrix().mult(l_h.getMatrix().trans());
		tmp1 = tmp1.mult(1.0 / (factor * factor));
		tmp1 = LinearAlgebraFactory.getMatrixId(3).minus(tmp1);

		Matrix tmp2 =
			getEuclideanPart().getMatrix().mult(l_h.getMatrix().trans());
		tmp2 = tmp2.mult(-1.0 / (factor * factor));

		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				jacobi.set(i, j, tmp1.get(i, j));
				jacobi.set(i + 3, j, tmp2.get(i, j));
			}
		}
		jacobi.set(3, 3, 1.0);
		jacobi.set(4, 4, 1.0);
		jacobi.set(5, 5, 1.0);

		jacobi = jacobi.mult(1.0 / factor);

		setCov(jacobi.mult(getCov().mult(jacobi.trans())));
		setV(getV().mult(1.0 / factor));
	}

	/** Get the value of the constraint (h(beta) or g(y))  if this entity 
	 * is an unknown or an observation.
	 * This is the length constraint and for 3D-Lines also the
	 * plcker condition.
	 * @return the value of the constraint as a vector.
	 */
	protected Vector getConstraintValue() {
		Vector vhelp = LinearAlgebraFactory.getVector(2);
		vhelp.set(0, 0.5* (v.scalarmult(v)-1.0));
		vhelp.set(1, 0.5* getHomogeneousPart().scalarmult(getEuclideanPart()));
		return vhelp;
	}

	/** Get the Jacobian of the constraint (H, G) if this entity 
	 *  is an unkown or an observation
	 * 
	 * @return the Jacobian as a matrix.
	 */
	public Matrix getJacobianOfConstraint() {
		Matrix H = LinearAlgebraFactory.getMatrix(2, 6);
		H.slice(0, 0, 0, 5, getV().getMatrix().trans());
		H.slice(1, 1, 0, 2, getEuclideanPart().getMatrix().trans());
		H.slice(1, 1, 3, 5, getHomogeneousPart().getMatrix().trans());
		return H;
	}

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_3D.class))
				return getDual().getV().getMatrix().trans();
			if (partnerType.equals(Plane_3D.class))
				return Gamma().trans();
			if (partnerType.equals(Point_3D.class))
				return GammaDual().trans();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_3D.class))
				return getLineLineJacobian();
			//				return Delta_imax();

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(6, 6);
				tmp.slice(0, 2, 0, 2, getHomogeneousPart().SkewSym());
				return tmp;
			}
			if (partnerType.equals(Plane_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(1, 4);
				tmp.slice(0, 0, 0, 2, getHomogeneousPart().getMatrix().trans());
				return tmp;
			}

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(1, 6);
				tmp.slice(0, 0, 0, 2, getHomogeneousPart().getMatrix().trans());
				return tmp;
			}
			if (partnerType.equals(Plane_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(4, 4);
				tmp.slice(0, 2, 0, 2, getHomogeneousPart().SkewSym());
				return tmp;
			}

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public int getDofOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_3D.class))
				return 1;
			if (partnerType.equals(Plane_3D.class))
				return 2;
			if (partnerType.equals(Point_3D.class))
				return 2;

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_3D.class))
				return 4;

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_3D.class))
				return 2;
			if (partnerType.equals(Plane_3D.class))
				return 1;

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_3D.class))
				return 1;
			if (partnerType.equals(Plane_3D.class))
				return 2;
		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/** Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation.
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_3D.class))
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else
					return getDual().getCov();
			if (partnerType.equals(Plane_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 5, 4, 0 };
							int[] yyy = { 1, 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 5, 3, 1 };
							int[] yyy = { 0, 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 4, 3, 2 };
							int[] yyy = { 0, 1, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 0, 1, 2 };
							int[] yyy = { 0, 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}
			if (partnerType.equals(Point_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1, 3 };
							int[] yyy = { 1, 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 2, 0, 4 };
							int[] yyy = { 0, 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 1, 0, 5 };
							int[] yyy = { 0, 1, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 3, 4, 5 };
							int[] yyy = { 0, 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}
		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Line_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(6, 6);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 3, 1, 2 };
							int[] yyy = { 0, 4, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 3, 1 };
							int[] yyy = { 1, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 3, 2 };
							int[] yyy = { 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 4 :
						{
							int[] xxx = { 4, 0 };
							int[] yyy = { 0, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 5 :
						{
							int[] xxx = { 4, 0, 2 };
							int[] yyy = { 1, 3, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 6 :
						{
							int[] xxx = { 4, 2 };
							int[] yyy = { 2, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 7 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 8 :
						{
							int[] xxx = { 5, 0 };
							int[] yyy = { 0, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 9 :
						{
							int[] xxx = { 5, 1 };
							int[] yyy = { 1, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 10 :
						{
							int[] xxx = { 5, 0, 1 };
							int[] yyy = { 2, 3, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 11 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 12 :
						{
							int[] xxx = { 5, 4 };
							int[] yyy = { 4, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 13 :
						{
							int[] xxx = { 5, 3 };
							int[] yyy = { 3, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 14 :
						{
							int[] xxx = { 4, 3 };
							int[] yyy = { 3, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 15 :
						{
							int[] xxx = { 3, 4, 5 };
							int[] yyy = { 0, 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
				/*
								int maxi = choose_i();
								Matrix result = LinearAlgebraFactory.getMatrix(6,6);
								if (row==maxi)
									return result;
								else if (row>5)
									throw new RuntimeException("Row out of range!");
								else {
									int[] xxx = {maxi, row};
									int[] yyy = {row, maxi};
									result.slice(yyy, yyy, getCov().slice(xxx, xxx));
									return result;
								}
				*/
			}
		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Line_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(6, 6);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 1 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 2 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 3 :
					case 4 :
					case 5 :
						break;
					default :
						throw new RuntimeException("Row out of range!");
				}
				return result;
			}
			if (partnerType.equals(Plane_3D.class)) {
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else {
					Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
					result.slice(0, 2, 0, 2, getCov().slice(0, 2, 0, 2));
					return result;
				}
			}

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Line_3D.class)) {
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else {
					Matrix result = LinearAlgebraFactory.getMatrix(6, 6);
					result.slice(0, 2, 0, 2, getCov().slice(0, 2, 0, 2));
					return result;
				}
			}
			if (partnerType.equals(Plane_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 1 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 2 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
						}
						break;
					case 3 :
						break;
					default :
						throw new RuntimeException("Row out of range!");
				}
				return result;
			}
		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());
	}

	/** Get the Matrix JGamma. vec(GammaDual(L)) = JGamma*C*L. This matrix
	 * differs from that of the dissertation because of
	 * other definition of C  */
	public static Matrix JGamma() {
		Matrix Jg = LinearAlgebraFactory.getMatrix(16, 6);
		/*
		Jg.set(1,2, -1);
		Jg.set(2,1,  1);
		Jg.set(3,3,  1);
		Jg.set(4,2,  1);
		Jg.set(6,0, -1);
		Jg.set(7,5,  1);
		Jg.set(8,1, -1);
		Jg.set(9,0,  1);
		Jg.set(11,5,  1);
		Jg.set(12,3, -1);
		Jg.set(13,4, -1);
		Jg.set(14,5, -1);
		*/
		Jg.set(1, 5, -1);
		Jg.set(2, 4, 1);
		Jg.set(3, 0, 1);
		Jg.set(4, 5, 1);
		Jg.set(6, 3, -1);
		Jg.set(7, 1, 1);
		Jg.set(8, 4, -1);
		Jg.set(9, 3, 1);
		Jg.set(11, 2, 1);
		Jg.set(12, 0, -1);
		Jg.set(13, 1, -1);
		Jg.set(14, 2, -1);
		return Jg;
	}

	/** Get the dualization matrix C with LDual=C*L. This matrix
	 *  differs from that of the dissertation in the sign !
	 * 
	 */
	public static Matrix DualMatrixC() {
		Matrix C = LinearAlgebraFactory.getMatrix(6, 6);
		C.set(0, 3, -1);
		C.set(1, 4, -1);
		C.set(2, 5, -1);
		C.set(3, 0, -1);
		C.set(4, 1, -1);
		C.set(5, 2, -1);
		return C;
	}

	/** Threshold for zero used in constructors for checking pluecker
	 *  condition. This threshold is not very accurate.
	 */
	private static final double ZERO_THRESHOLD_CONSTR = 1e-04;

} // end Line_3D
