/*
 * Created on 13.03.2003
 */
package sugr;

import sugr.linalg.*;
/**
 * This matrix contains translation(2) and rotation(1) in 2D
 * @author Norbert Fischer
 */
public class Motion_2D extends Affinity_2D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. Creates an identity matrix
	 */
	public Motion_2D() {}
	
	/**
	 * creates a motion by a given Translation_2D and a Rotation_2D
	 * Error propagation is done automatically
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 */
	public Motion_2D (Translation_2D trans, Rotation_2D rot)
	{
		this(trans, rot, null);
	}

	/**
	 * creates a motion with covariance by a given Translation_2D 
	 * and a Rotation_2D 
	 * No error propagation will be performed if cov is not null.
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 * @param cov   9x9 matrix
	 */
	public Motion_2D (Translation_2D trans, Rotation_2D rot,
					  Matrix cov)
	{
		setV((trans.getTransformationMatrix().
		  mult(rot.getTransformationMatrix()).vec()));
		  
		//FFG
		if (cov==null)	
		{
			setCov(errorProp(trans, rot));
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else
		{
			checkCovDim(cov, DIM*DIM);
			setCov(cov);
		}
	}

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 3;
	}	
	
}
