/** Java class "Parallel.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Contains methods if two elements are parallel or not. You can retrieve then a lot 
 * of information about this test, for example: chi-value used, 
 * the testvalue and so on.
 * 
 * @author Norbert Fischer
 * @version 
 * <b> 1.1 (8.2.2003) : </b> <br>
 * 		- compute_distance -> Line_2D/Line_2D 
 * 			Covariance-calculation is now right ;-) <br>
 * 
 * <b> 1.1.1 (11.2.2003)</b> <br>
 * 		Fixed: compute_distance -> Line_2D/Line_2D
 * 			 Covariance-calculation changed from l.getCov() to
 * 			 l.getJacobeanHomogeneousPart(). Analog m <br>
 * 		Fixed: compute_distance -> Plane_3D/Plane_3D
 * 			 Covariance-calculation changed from a.getCov() to
 * 			 a.getJacobeanHomogeneousPart(). Analog b <br>
 * 
 * <b> 1.2 (20.2.2003)</b> <br>
 * 		Fixed: computation of error propagation (ep) was wrong in
 * 			Line_2D/Line2D, Line_3D/Plane_3D. ep needs also the homogeneous parts
 * 			of the elements - just adding covariances is not enough ;-) <br>
 * 
 * <b> 1.2.1 (24.2.2003)</b> <br>
 * 		Changed: works now on copies of entities. This preserves the 
 * 			original input values from changing!!!
 * <b> 2.0 (June 2003)</b> <br>
 * 		Is now derived from BiRelationalProperty !!!
 */
public class Parallel extends BiRelationalProperty {

	///////////////////////////////////////
	// operations

	/**
	 * <p>
	 * Return Jacobian Matrix A of the first Entity and the relation type.
	 * Be careful: For estimation , the Jacobian must have another
	 * sign for Jacobian of observations and unknowns in
	 * case of lines and planes. So deliver
	 * (-1)* the Jacobian of the Entity ! (See table 3.6. on page 73 of
	 * diss of Stephan Heuel!)
	 * </p><p>
	 * 
	 * @return Jacobian Matrix A
	 * </p>
	 */
	public Matrix getJacobian() {
		Matrix J=first.getJacobianOfRelation(Parallel.class, secondEntityType);
		if ((first.getClass().equals(Line_3D.class) && secondEntityType.equals(Line_3D.class)) ||
		    (first.getClass().equals(Plane_3D.class) && secondEntityType.equals(Plane_3D.class)))
		return J.mult(-1.0);
		
		return J;
	}

	/**
	 * <p>
	 * Compute degrees of freedom of that Relation
	 * </p><p>
	 * 
	 * @return degrees of freedom of that Relation
	 * </p>
	 */
	public int getDof() {
		return first.getDofOfRelation(Parallel.class, secondEntityType);
	}

	/**
	 * Build up an Parallel-Relation between two Line_2D.
	 * @param l  first Line_2D
	 * @param secondClass must always be a Line_2D
	 */
	public Parallel(Line_2D l, Class secondClass) {
		super(l, secondClass);
	} // end Parallel        

	/**
	 * Build up an Parallel-Relation between two Line_3D.
	 * @param l  first Line_3D
	 * @param secondClass could be a Line_3D or Plane_3D
	 */
	public Parallel(Line_3D l, Class secondClass) {
		super(l, secondClass);
	} // end Parallel        

	/**
	 * Build up an Parallel-Relation between two Plane_3D.
	 * @param a  first Plane_3D
	 * @param secondClass could be a Plane_3D or Line_3D
	 */
	public Parallel(Plane_3D a, Class secondClass) {
		super(a, secondClass);
	} // end Parallel        

	/** <p>
	 * Returns the Covariance Matrix of the row-vector of the Jacobian Matrix A of the first Entity and the relation type.
	 * </p>
	 *
	 * @param row Row vector, for which the Covariance matrix is to be returned
	 * @return covariance of the row vector of the jacobian
	 */
	public Matrix getCovarianceOfJacobianRow(int row) {
		return first.getCovarianceOfJacobianRowOfRelation(
			Parallel.class,
			secondEntityType,
			row);
	}

} // end Parallel
