/** Java class "Plane_3D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Implementation of Plane_3D
 * 
 * @author Norbert Fischer
 * @version 1.2 (25.1.2004)
 * 		Changed Constructor Plane_3D(Point, Line) and Plane_3D(Point,Point,Point)
 * 		to not use Construction Class anymore. Calculation is now done in 
 * 		Constructor itself. By Achim Knigs
 * @version 1.1 (10.2.2003)
 * 		Fixed: getJacobianHomogeneousPart() returned wrong matrix
 * 
 */
public class Plane_3D extends Object_3D {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 4;
	private static final int DOF = 3;

	///////////////////////////////////////
	// operations

	/**
	 * Constructor for a new Plane_3D
	 * @param _v a Vector with dimension 4. Contains the coordinates of the new Line_2D
	 * @param _cov 4x4 covariance matrix
	 */
	public Plane_3D(Vector _v, Matrix _cov) {
		checkElement(_v, DIM);
		checkCovDim(_cov, DIM);
		v = _v.copy();
		cov = _cov.copy();
	}

	/**
	 * constructs a new plane by setting all entries directly. 
	 * covariance matrix is set to 0 so it has no error
	 */
	public Plane_3D(double x, double y, double z, double w) {

		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, w);

		checkElement(v, DIM);
		setV(v);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Plane_3D        

	/**
	 * constructs a new plane by setting all entries directly. With covariance 
	 * @param cov   4x4 Matrix
	 */
	public Plane_3D(double x, double y, double z, double w, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, w);

		checkElement(v, DIM);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Plane_3D        

	/**
	 * constructs a new Plane_3D by specifying the homogeneous part. Note that you
	 * are setting the normal of the plane. Euclidean part is set to 1. <br> 
	 * covariance matrix is set to 0 so it has no error
	 */
	public Plane_3D(double x, double y, double z) {
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, 1);
		setV(v);
		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Plane_3D        

	/**
	 * constructs a new Plane_3D by specifying the homogeneous part. Note that you
	 * are setting the normal of the plane. Euclidean part is set to 1. <br> 
	 * With covariance.
	 */
	public Plane_3D(double x, double y, double z, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, 1);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Plane_3D        

	/**
	 * constructs a new plane by a join between point and line. Error propagation is
	 * done automatically
	 * @param point   a Point_3D
	 * @param line    a Line_3D
	 */
	public Plane_3D(Point_3D point, Line_3D line) {
		Plane_3D a = new Plane_3D();
		a.setV( point.PiDual().trans().mult(line.getV()) );
        
		// Set Covariance Matrix:
		//Pi(x) is negative (see sugr.handbook pg.3) but ep neutralizes this
		a.setCov( calcCov( point.PiDual().trans(), point.getCov(), 
						   line.GammaDual().trans(), line.getCov())  );
		
		checkElement(a.getV(), DIM);
		setV(a.getV());
		setCov(a.getCov());
	} // end Plane_3D        

	/**
	 * constructs a new plane by a join between point and line. Error propagation is
	 * done automatically
	 * @param point   a Point_3D
	 * @param line    a Line_3D
	 */
	public Plane_3D(Line_3D line, Point_3D point) {
		this(point, line);
	} // end Plane_3D        

	/**
	 * estimates a new plane by a join of two coplanar lines
	 * @param L1   a Line_3D
	 * @param L2   a Line_3D
	 * TODO: test fails, because covariance differs from statistically
	 * 		computed one, check estimation
	 */
	public Plane_3D(Line_3D L1, Line_3D L2) {
		double f = getConditionFactorF(L1, L2);
		if (f != 1) {
			L1.doCondition(f);
			L2.doCondition(f);
		}

		RelationalProperty[] rprops = new RelationalProperty[2];

		rprops[0] = new Incident(L1, Plane_3D.class);
		rprops[1] = new Incident(L2, Plane_3D.class);

		if ((L1.getCov().vec().scalarmult(L1.getCov().vec()) == 0)
			|| (L2.getCov().vec().scalarmult(L2.getCov().vec()) == 0))
			approximate(rprops);
		else {
			RelationalProperty[] yhat = new RelationalProperty[rprops.length];
			double[] sigma0 = new double[1];
			int[] iterations = new int[1];
			if (!estimate(rprops, true, MAXITER, yhat, sigma0, iterations, false))
				throw new IllegalArgumentException("Convergence not reached!");
		}

		if (f != 1) {
			L1.doCondition(1 / f);
			L2.doCondition(1 / f);
			doCondition(1 / f);
		}
	} // end Plane_3D 

	/**
	 * Estimates a new plane
	 * @param rprops an array of constraints.
	 */
	public Plane_3D(RelationalProperty[] rprops) {
		if (!estimate(rprops))
			throw new IllegalArgumentException("Convergence not reached!");
	}

	/**
	 * constructs a plane by a join of three points. Error 
	 * propagation will be done automatically
	 */
	public Plane_3D(Point_3D point1, Point_3D point2, Point_3D point3) {
		//using other constructor of Plane_3D and constructor of Line_3D.
		Plane_3D a = new Plane_3D(point1,new Line_3D(point2,point3));
		checkElement(a.getV(), DIM);
		setV(a.getV());
		setCov(a.getCov());
	} // end Plane_3D        

	/**
	 * default constructor
	 */
	public Plane_3D() {
	} // end Plane_3D        

	/**
	 * returns the euclidean part of this plane
	 * @return a Vector with one element
	 */
	public Vector getEuclideanPart() {
		Vector vec = LinearAlgebraFactory.getVector(1);
		vec.set(0, getV().get(3));

		return vec;
	} // end getEuclideanPart        

	/**
	 * returns the homogeneous part of this plane
	 * @return a Vector with three elements
	 */
	public Vector getHomogeneousPart() {
		Vector vec = LinearAlgebraFactory.getVector(3);
		vec.set(0, getV().get(0));
		vec.set(1, getV().get(1));
		vec.set(2, getV().get(2));

		return vec;
	} // end getHomogeneousPart        

	/**
	  * Compute the (euclidean) distance to another entity. This
	  * is of course not possible for all combinations.
	  * In the case of an illegal combination an IllegalArgumentException
	  * should be thrown.
	  * @return the distance to the given entity
	  * @param e  the entity to which a distance should be computed.
	 */
	public double getDistance(Entity e) {
		if (e instanceof Point_3D) {
			double tmp = Math.abs(e.getV().scalarmult(getV()));
			return tmp
				/ getHomogeneousPart()
					.mult(e.getHomogeneousPart().get(0))
					.getLength();
		}
		throw new IllegalArgumentException(
			"Plane_3D.distance: distance between "
				+ e.getClass()
				+ " not possible");
	}

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	protected Matrix getConditionMatrix(double f) {
		Matrix w = LinearAlgebraFactory.getMatrix(4, 4);
		w.set(0, 0, 1);
		w.set(1, 1, 1);
		w.set(2, 2, 1);
		w.set(3, 3, f);
		return w;
	}

	/**
	 * returns a copy of this plane. Actually it works like clone
	 */
	public Plane_3D copy() {
		return new Plane_3D(v, cov);
	}

	/**
	 * returns dimension=4 for this plane
	 */
	public int getDimension() {
		return DIM;
	}

	/**
	 * returns dof=3 of this plane
	 */
	public int getDegreesOfFreedom() {
		return DOF;
	}

	/**
	 * returns the pi-jacobian of this plane
	 */
	public Matrix Pi() {
		return Pi(getV());
	}

	/**
	 * returns the pi-jacobian of a 4x1 vector
	 */
	public static Matrix Pi(Vector v_in) {
		Matrix Pi = LinearAlgebraFactory.getMatrix(6, 4);
		double u = v_in.get(0);
		double v = v_in.get(1);
		double w = v_in.get(2);
		double t = v_in.get(3);
		Pi.set(0, 0, t);
		Pi.set(0, 3, -u);

		Pi.set(1, 1, t);
		Pi.set(1, 3, -v);

		Pi.set(2, 2, t);
		Pi.set(2, 3, -w);

		Pi.set(3, 1, -w);
		Pi.set(3, 2, v);

		Pi.set(4, 0, w);
		Pi.set(4, 2, -u);

		Pi.set(5, 0, -v);
		Pi.set(5, 1, u);
		return Pi;
	}
	
	/**
	 * returns the pidual-jacobian of the dual of this plane
	 */
	public Matrix PiDual() {
		return PiDual(getV());
	}


	/**
	 * Returns the pidual-jacobian of the dual of a 4x1 vector
	 * Uses now the PiDual function of Point to ensure consistency (T.Laebe, 15.11.05) 
	 * 
	 */
	public static Matrix PiDual(Vector v) {
		return Point_3D.PiDual(v);
	} // end PiDual  

	/**
	 * normalizes in that way, that the homogeneous part will be 1
	 */
	public void normalizeEuclidean() {
		Vector a_h = getHomogeneousPart();
		double factor = a_h.getLength();

		Matrix jacobi = LinearAlgebraFactory.getMatrix(DIM, DIM);

		Matrix tmp1 =
			a_h.getMatrix().mult(a_h.getMatrix().trans()).mult(
				1.0 / (factor * factor));
		tmp1 = LinearAlgebraFactory.getMatrixId(3).minus(tmp1);

		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				jacobi.set(i, j, tmp1.get(i, j));
			}
		}
		jacobi.set(3, 3, 1);

		Vector tmp = a_h.mult(-getV().get(3) / (factor * factor));
		jacobi.set(3, 0, tmp.get(0));
		jacobi.set(3, 1, tmp.get(1));
		jacobi.set(3, 2, tmp.get(2));
		jacobi=jacobi.mult(1/factor); //line added look at diss Heuel p.104 Table 4.1
		
		setCov(jacobi.mult(getCov()).mult(jacobi.trans()));
		setV(getV().mult(1.0 / factor));
	}

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_3D.class))
				return getV().getMatrix().trans();
			if (partnerType.equals(Line_3D.class))
				return Pi().trans();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Plane_3D.class))
				return PiDual();

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Plane_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(4, 4);
				tmp.slice(0, 2, 0, 2, getHomogeneousPart().SkewSym());
				return tmp;
			}
			if (partnerType.equals(Line_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(1, 6);
				tmp.slice(0, 0, 0, 2, getHomogeneousPart().getMatrix().trans());
				return tmp;
			}
		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Plane_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(1, 4);
				tmp.slice(0, 0, 0, 2, getHomogeneousPart().getMatrix().trans());
				return tmp;
			}
			if (partnerType.equals(Line_3D.class)) {
				Matrix tmp = LinearAlgebraFactory.getMatrix(4, 6);
				tmp.slice(
					0,
					2,
					0,
					2,
					getHomogeneousPart().SkewSym().mult(-1.0));
				return tmp;
			}

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public int getDofOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_3D.class))
				return 1;
			if (partnerType.equals(Line_3D.class))
				return 2;
		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Plane_3D.class))
				return 3;

		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Plane_3D.class))
				return 2;
			if (partnerType.equals(Line_3D.class))
				return 1;

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Plane_3D.class))
				return 1;
			if (partnerType.equals(Line_3D.class))
				return 2;

		}
		throw new IllegalArgumentException(
			"Illegal Relation in getDofOfRelation(): "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/** Get the reduced Matrix A and B from a relational 
	 * property. A is the Jacobian of this Entity (the unknown),
	 * B is the Jacobian of the observation (the relational property).
	 * The Jacobian of the oberservation depends (only) on the unknown and
	 * the Jacobian of the unknown depends (only) of the oberservation !
	 * This method is overwritten to handle some the special cases.
	 * 
	 * @param rprop the relational Property
	 * @return A in array element 0, B in array Element 1.
	 */
	protected Matrix[] getReducedAB(RelationalProperty rprop) {
		BiRelationalProperty r = (BiRelationalProperty) rprop;
		if (r.getClass().equals(Incident.class)) {
			if (r.getFirstEntity().getClass().equals(Line_3D.class)) {
				Matrix A = r.getJacobian();
				Matrix B = getJacobianOfRelation(r);
				Matrix[] AB = reduceFirstMatrix(A, B, r.getDof());
				return AB;
			}
			else
				return super.getReducedAB(rprop);
		}
		else
			return super.getReducedAB(rprop);
	}
		
	/** Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation.
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Point_3D.class))
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else
					return getCov();
			if (partnerType.equals(Line_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(6, 6);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 3, 2, 1 };
							int[] yyy = { 0, 4, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 3, 2, 0 };
							int[] yyy = { 1, 3, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 3, 1, 0 };
							int[] yyy = { 2, 3, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 0, 1, 2 };
							int[] yyy = { 0, 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}
		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Plane_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 3, 0 };
							int[] yyy = { 0, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 4 :
						{
							int[] xxx = { 3, 1 };
							int[] yyy = { 1, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 5 :
						{
							int[] xxx = { 3, 2 };
							int[] yyy = { 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}
		} else if (relationType.equals(Parallel.class)) {
			if (partnerType.equals(Plane_3D.class))
				throw new RuntimeException("Not yet implemented!");
			if (partnerType.equals(Line_3D.class))
				throw new RuntimeException("Not yet implemented!");

		} else if (relationType.equals(Orthogonal.class)) {
			if (partnerType.equals(Plane_3D.class))
				throw new RuntimeException("Not yet implemented!");
			if (partnerType.equals(Line_3D.class))
				throw new RuntimeException("Not yet implemented!");

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());
	}
} // end Plane_3D
