/** Java class "Point_2D.java" generated from Poseidon for UML.
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 *
 * Implementation of Point_2D
 *
 *
 * @author Norbert Fischer
 * @version 1.1 (25.1.2004)
 * 		Changed Constructor Point_2D(Line,Line) changed to not use Construction Class
 * 		anymore. Calculation is now done in Constructor itself.
 */
public class Point_2D extends Object_2D {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 3;
	private static final int DOF = 2;

	///////////////////////////////////////
	// operations

	/**
	 * Constructor for a new Point_2D
	 * @param _v a Vector with dimension 3. Contains the coordinates of the new Line_2D
	 * @param _cov 3x3 covariance matrix
	 */
	public Point_2D(Vector _v, Matrix _cov) {
		checkElement(_v, DIM);
		checkCovDim(_cov, DIM);
		v = _v.copy();
		cov = _cov.copy();
	}

	/**
	 * Constructor for a euclidean point. homogeneous part is set to 1,
	 * covariance matrix is set to 0
	 * @param x  x-part of point
	 * @param y  y-part of point
	 */
	public Point_2D(double x, double y) {

		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, 1);

		setV(v);
		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Point_2D

	/**
	 * Constructor for a euclidean point with covariance. homogeneous part is 
	 * set to 1.
	 * @param x  x-part of point
	 * @param y  y-part of point
	 * @param cov  3x3 Matrix
	 */
	public Point_2D(double x, double y, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, 1);

		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Point_2D

	/**
	 * Sets all elements of point directly. Covariance is set to 0
	 */
	public Point_2D(double u, double v, double w) {

		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, u);
		vec.set(1, v);
		vec.set(2, w);

		checkElement(vec, DIM);
		setV(vec);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Point_2D

	/**
	 * Sets all elements of point directly. With Covariance.
	 * @param cov   3x3 Matrix
	 */
	public Point_2D(double u, double v, double w, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, u);
		vec.set(1, v);
		vec.set(2, w);

		checkElement(vec, DIM);
		setV(vec);

		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Point_2D
	
	/** Construct a Point with known accuracies for x and y.
	 *  If you have the 2 coordinates of a 2D-Point and accuracies for
	 *  these two coordinates (ignoring correlations) then you can
	 *  use this (shortcut) constructor to generate a SUGR-Point with
	 *  covariance matrix. The generated covariance is
	 *  | sigma_x^2    0         0|
	 *  |     0     sigma_y^2    0|
	 *  |     0        0         0|
	 *  and the homogeneous part is set to 1.
	 *  
	 *  @param x first coordinate of point
	 *  @param y second coordinate of point
	 *  @param sigma_x accuracy of x (standard deviation)
	 *  @param sigma_y accuracy of y (standard deviation)
	 */
	public Point_2D(double x, double y, double sigma_x, double sigma_y) {
		Matrix cov=LinearAlgebraFactory.getMatrix(DIM,DIM);
		cov.set(0,0, sigma_x*sigma_x);
		cov.set(1,1, sigma_y*sigma_y);

		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, x);
		vec.set(1, y);
		vec.set(2, 1);
		setV(vec);
		setCov(cov);
	}

	/**
	 * constructs a new point_2D by an intersection between two lines. Error
	 * propagation is done automatically
	 * @param l   first Line_2D
	 * @param m   second Line_2D
	 */
	public Point_2D(Line_2D l, Line_2D m) {
		//Construct v via x=J*v
		Point_2D x = new Point_2D();
		x.setV( l.SkewSym().mult(m.getV()) );
		// Set Covariance Matrix:
		x.setCov( calcCov( l.SkewSym(), l.getCov(), 
					m.SkewSym(), m.getCov())  );
		
		checkElement(x.getV(), DIM);
		setV(x.getV());
		setCov(x.getCov());
	} // end Point_2D

	/**
	 * default constructor
	 */
	public Point_2D() {
	} // end Point_2D

	/**
	 * Estimates a new Point
	 * @param rprops an array of constraints.
	 */
	public Point_2D(RelationalProperty[] rprops) {
		if (!estimate(rprops))
			throw new IllegalArgumentException("Convergence not reached!");
	}

	/**
	 * returns the euclidean part of this point
	 * @return a Vector with 2 elements
	 */
	public Vector getEuclideanPart() {
		Vector vec = LinearAlgebraFactory.getVector(2);
		vec.set(0, getV().get(0));
		vec.set(1, getV().get(1));

		return vec;
	} // end getEuclideanPart

	/**
	 * returns the homogeneous part of this plane
	 * @return a Vector with one element
	 */
	public Vector getHomogeneousPart() {
		Vector vec = LinearAlgebraFactory.getVector(1);
		vec.set(0, getV().get(2));

		return vec;
	} // end getHomogeneousPart

	/**
	  * Compute the (euclidean) distance to another entity. This
	  * is of course not possible for all combinations.
	  * In the case of an illegal combination an IllegalArgumentException
	  * should be thrown.
	  * @return the distance to the given entity
	  * @param e   the entity to which a distance should be computed.
	 */
	public double getDistance(Entity e) {
		if (e instanceof Point_2D) {
			Vector tmp = e.getEuclideanPart().mult(getHomogeneousPart().get(0));
			tmp =
				tmp.minus(
					getEuclideanPart().mult(e.getHomogeneousPart().get(0)));
			return tmp.getLength()
				/ Math.abs(
					getHomogeneousPart().get(0)
						* e.getHomogeneousPart().get(0));
		}
		if (e instanceof Line_2D) // implemented in Line_2D
			return e.getDistance(this);

		throw new IllegalArgumentException(
			"Point_2D.distance: distance between "
				+ e.getClass()
				+ " not possible");
	}

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	protected Matrix getConditionMatrix(double f) {
		Matrix w = LinearAlgebraFactory.getMatrix(3,3);
		w.set(0, 0, f);
		w.set(1, 1, f);
		w.set(2, 2, 1);
	    return w;
	}

	/**
	 * returns a copy of this point. Actually it works like clone
	 */
	public Point_2D copy() {
		return new Point_2D(v, cov);
	}

	/**
	 * returns dimension=3 for this point
	 */
	public int getDimension() {
		return DIM;
	}

	/**
	 * returns dof=2 of this point
	 */
	public int getDegreesOfFreedom() {
		return DOF;
	}

	/**
	 * Returns the skew symmetric matrix of a 3x1 vector.
	 * @return 3x3 Matrix with rank 2
	 */
	public static Matrix SkewSym(Vector v) {
		Matrix S = LinearAlgebraFactory.getMatrix(3, 3);
		double a = v.get(0);
		double b = v.get(1);
		double c = v.get(2);
		S.set(0, 1, -c);
		S.set(0, 2, b);
		S.set(1, 0, c);
		S.set(1, 2, -a);
		S.set(2, 0, -b);
		S.set(2, 1, a);
		return S;
	}
	
	/**
	 * Returns the skew symmetric matrix of this point
	 * @return 3x3 Matrix with rank 2
	 */
	public Matrix SkewSym() {
      return SkewSym(getV());
	}

	/**
	 * normalizes in that way, that the homogeneous part will be 1
	 */
	public void normalizeEuclidean() {
		double factor = getV().get(2);

		Matrix jacobi = LinearAlgebraFactory.getMatrixId(DIM);

		jacobi.set(2, 2, 0);
		jacobi.set(0, 2, -getV().get(0) / factor);
		jacobi.set(1, 2, -getV().get(1) / factor);
		jacobi = jacobi.mult(1.0 / factor);
		
		setCov(jacobi.mult(getCov()).mult(jacobi.trans()));

		setV(getV().mult(1.0 / factor));
	}

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_2D.class))
				return getV().getMatrix().trans();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_2D.class))
				return getV().SkewSym();

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public int getDofOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_2D.class))
				return 1;

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_2D.class))
				return 2;
		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/** Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation.
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Line_2D.class))
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else
					return getCov();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_2D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(3, 3);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}

			}
		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}
} // end Point_2D
