/** Java class "Relation.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */

package sugr;

import sugr.linalg.*;

import java.lang.reflect.*;

/**
 * <p>
 * class which contains methods to test two elements
 * 
 * @author Norbert Fischer
 * @version 1.1 (10.2.2003):
 * 		- Fixed: getDistance() return t instead of return 0.0
 * 		- doTesting() changed so that computation of Testvalue t
 * 			throws no matrix-dimension exception (but have a look
 * 			on it if it's right
 * 
 * @version 1.1.1 (12.2.2003)
 * 		- Changed: testValue() return the squareroot of t, just like
 * 			Perl-sugr [relation.pm , line 208]
 * 
 * <b> 1.2 (16.2.2003) </b> <br>
 * 		- Fixed: corrected compute_PseudoInverse() (a trans() was too much)
 * 		- Removed: compute_PseudoInverse() into linalg-classes
 * 					(now matrix-method pseudoInverse() )
 * 		- Fixed: doTesting, calculation of t = d^t Sigma_dd^+ d
 * 				now really used the pseudoInverse ;-)
 * 		- Added: setAlpha_WithUsedDOF(double alpha)	
 * 				needs to remember the old used dof because the call in 
 * 				RelationApplet.java doesn't know the real dof
 * 		- Fixed: chi2 was not actualized when a new alpha was set
 * 
 * <b> 1.3 (19.2.2003)</b> <br>
 * 		- Changed: Relation is from now on abstract. It shouldn't be possible
 * 				to create a relation-instance, because it has no real relation
 * 				(e.g. Point_2D/Point_2D + Equality) to test. Therefore it makes
 * 				no sense to getDistance(), getalpha(), getChi() without any entities
 * 		- Changed: compute_distance() is abstract. It should only be implemented in 
 * 				subclasses with know comparision and elements
 * 		- Removed: testChiSquare (int dof): dof is now element of this class and
 * 				does not need to be set exterior
 * 		- changed: testChiSquare (double alpha, int dof) changed into
 * 				testChiSquare (double alpha) 
 * 		- Removed: setAlpha_WithUsedDOF(...) because dof is now part of Relation
 * 		- Changed: setAlpha(double _alpha) has functionality of removed method above
 * 		- Changed: dof needs to be set and this is done in doTesting()
 * 				because every subclass is calling this method (less work, you know ;-)
 * 		- Changed: startvalue of alpha is set to 0.05
 * 
 * <b> 2.0 (17.06.2003, T. Laebe) </b> <br>
 *      One part of the Relation is now BiRelationalProperty.
 * </p>
 */
public class Relation {

	///////////////////////////////////////
	// attributes

	/** The first part and the type of the relation. */
	private BiRelationalProperty rprop;

	/** The second part of the relation. */
	private Entity second;

	/**
	 * <p>
	 * Represents test value t = d^T %SIGMA_dd^+ d (see skript for more information)
	 * </p>
	 */
	protected double t;

	/**
	 * <p>
	 * Represents values for the chi-Test
	 * </p>
	 */
	private static final double C0 = 2.515517;
	private static final double C1 = 0.802853;
	private static final double C2 = 0.010328;
	private static final double D1 = 1.432788;
	private static final double D2 = 0.189269;
	private static final double D3 = 0.001308;

	/**
	 * <p>
	 * Represents the result of the chi-Test
	 * </p>
	 */
	protected double chi2;

	/**
	 * <p>
	 * Represents significance niveau. this is a start value, you may change this
	 * </p>
	 */
	protected double alpha = 0.95;

	/**
	 * If true, the entities are conditioned prior to the testing
	 * TODO: The conditioning does not work! It is turned off by default until the error is fixed! 
	 */
	protected boolean condition = false;

	/**
	 * Represents the non-squared, non-normalized test value. This may be used for checking the
	 * sign of a scalar-valued test.
	 */
	protected Vector d;

	/**
	 * Represents the covariance matrix of the non-squared, non-normalized test value d.
	 */
	protected Matrix Sigma;

	///////////////////////////////////////
	// operations

	/**
	 * <p>
	 * Constructs a concrete relation.
	 * </p><p>
	 * 
	 * 
	 * @param _first the first entity of the relation
	 * </p><p>
	 * @param _second the second entity of the relation 
	 * </p><p>
	 * @param relationType the type of the Relation (a derived class from
	 *        BiRelationalProperty
	 * </p>
	 */
	public Relation(Entity _first, Entity _second, Class relationType)
		throws IllegalArgumentException {
		second = _second;
		createRelationalProperty(_first,_second.getClass(), relationType);
		doTesting(condition);

	} // end Relation
	
	/**
	 * <p>
	 * Constructs a concrete relation.
	 * </p><p>
	 * 
	 * 
	 * @param _first the first entity of the relation
	 * </p><p>
	 * @param _second the second entity of the relation 
	 * </p><p>
	 * @param relationType the type of the Relation (a derived class from
	 *        BiRelationalProperty
 	 * @param doConditioning  if true, conditioning is done.
	 * </p>
	 */
	public Relation(Entity _first, Entity _second, Class relationType, boolean doConditioning)
		throws IllegalArgumentException {
		second = _second;
		condition=doConditioning;
		createRelationalProperty(_first,_second.getClass(), relationType);
		doTesting(condition);
	} // end Relation        
        

	/**
	 * Constructs a concrete relation.
	 * 
	 * 
	 * @param _rprop a relational property which consists of
	 *        the first entity of the relation and (inclusivly) the
	 *        type of the relation.
	 * @param _second the second entity of the relation 
	 */
	public Relation(BiRelationalProperty _rprop, Entity _second) {
		rprop = _rprop;
		second = _second;

		doTesting(condition);
	}

	/**
	 * Constructs a concrete relation.
	 * 
	 * 
	 * @param _rprop a relational property which consists of
	 *        the first entity of the relation and (inclusivly) the
	 *        type of the relation.
	 * @param _second the second entity of the relation 
	 * @param doConditioning  if true, conditioning is done.
	 */
	public Relation(
		BiRelationalProperty _rprop,
		Entity _second,
		boolean doConditioning) {
		rprop = _rprop;
		second = _second;
		condition = doConditioning;

		doTesting(condition);
	}

	/**
	 * <p>
	 * returns true if a relation between two elements (e.g l||m ) holds, 
	 * and false if not
	 * </p><p>
	 * 
	 * @return a boolean with ...
	 * </p>
	 */
	public boolean decision() {
		if (t < chi2)
			return true;
		return false;
	} // end decision        

	/**
	 * <p>
	 * returns the testvalue t= d^t SIGMA_dd^+ d
	 * [see sugr-handbook p.17: 
	 * 		"Statistically testing uncertain geometric relations", Ver2]
	 * </p>
	 */
	public double testValue() {
		// your code here
		return t;
	} // end testValue        

	/**
	 * <p>
	 * this calls the chi-routine. 
	 * Implemented after Stephan Heuels Perl:sugr [Basic.pm, line 686 ff]
	 * </p><p>
	 * 
	 * @param _alpha significance niveau
	 * @param dof degrees of freedom
	 * </p><p>
	 * @return a double with ...
	 * </p>
	 */
	public static double getChiSquareDistributionValue(
		double _alpha,
		int dof) {
		boolean negate = false;
		if (_alpha < 0.5) {
			_alpha = 1 - _alpha;
			negate = true;
		}

		double t = Math.sqrt(Math.log(1.0 / ((1 - _alpha) * (1 - _alpha))));

		//
		// x_alpha : alpha-Fraktil der standardisierten Normalverteilung,
		// Approximation nach Seite 104, Gl. 241.8 , Koch. Gilt fuer
		// alpha zwischen 0.5 und 1 ====> fuer alpha<0.5 muss x_alpha negiert werden,
		// siehe Koch Gleichung (241.7), SH und A. Brunn 17.9.98
		double x_alpha =
			t
				- ((C0 + C1 * t + C2 * t * t)
					/ (1 + D1 * t + D2 * t * t + D3 * t * t * t));
		if (negate)
			x_alpha = -x_alpha;

		//
		// Approximation fuer Chi2 Fraktilwerte: nach Gl. 261.11 , Seite 118,
		// Koch **
		//
		return dof
			* Math.pow(
				x_alpha * Math.sqrt(2.0 / (9.0 * dof))
					+ 1.0
					- 2.0 / (9.0 * dof),
				3);
	} // end testChiSquare        

	/**
	 * <p>
	 * returns the significance niveau
	 * </p><p>
	 * 
	 * @return a double with ...
	 * </p>
	 */
	public double getAlpha() {
		return alpha;
	} // end getAlpha        

	/**
	 * <p>
	 * sets the significance niveau. must be between 0 and 1
	 * </p><p>
	 * 
	 * </p><p>
	 * 
	 * @param _alpha  sets the new alpha-value
	 * </p>
	 */
	public void setAlpha(double _alpha) {
		if ((_alpha < 0) && (_alpha > 1))
			throw new IllegalArgumentException("alpha must be between 0 and 1!");
		alpha = _alpha;
		this.chi2 = getChiSquareDistributionValue(alpha, rprop.getDof());

	} // end setAlpha

	/** Set whether conditioning should be done.
	 * @param doConditioning  if true, conditioning is done.
	 */
	public void setCondition(boolean doConditioning) {
		if (doConditioning != condition) {
			condition = doConditioning;
			doTesting(doConditioning);
		} else
			condition = doConditioning;
	}

	/** Return if conditioning is on.
	 * 
	 * @return true, if conditioning is on.
	 */
	public boolean getCondition() {
		return condition;
	}

	/**
	 * <p>
	 * returns chi-testvalue
	 * </p><p>
	 * 
	 * @return a double with ...
	 * </p>
	 */
	public double testThreshold() {
		return chi2;
	} // end getChi2

	/**
	 * <p>
	 * returns the non-squared, non-normalized test value
	 * </p><p>
	 * 
	 * @return non-squared, non-normalized test value
	 * </p>
	 */
	public Vector getD() {
		return d;
	}

	/**
	 * <p>
	 * returns the covariance matrix of d
	 * </p><p>
	 * 
	 * @return covariance matrix of d
	 * </p>
	 */
	public Matrix getSigma() {
		return Sigma;
	}

	/**
	 * <p>
	 * This is the main routine. it tests two entities wether a given relation hold 
	 * or not. This method should be called in every constructor of
	 * this class and of derived classes. 
	 * </p><p>
	 * 
	 * </p>
	 */
	protected void doTesting(boolean condition) {
		//Sets dof for this special relation 
		int dof = rprop.getDof();

		Entity e1, e2;

		if (condition) {
			e1 = (Entity) rprop.getFirstEntity().clone();
			e2 = (Entity) second.clone();

			//What is done? See page 15 for info
			double f = Entity.getConditionFactorF(e1, e2);
			//See page 12 for info
			if (f != 1) {
				e1.doCondition(f);
				e2.doCondition(f);
			}
			//Change the null-space of the Cov-Matrices
			e1.changeNullspace();
			e2.changeNullspace();
		} else {
			e1 = rprop.getFirstEntity();
			e2 = second;
		}

		// Hier wird mit den reduzierten Jakobimatrizen, genau wie in der Schtzung, getestet. Evtl. ist auch eine Nullraumangleichung durch sphrische Normierung wie in der Schtzung notwendig... (Chb 13.1.05)
		Matrix[] AB = e2.getReducedAB(rprop);
		d = AB[0].mult(e2.getV());
		Sigma =
			AB[0].mult(e2.getCov()).mult(AB[0].trans()).plus(
				AB[1].mult(e1.getCov()).mult(AB[1].trans()));
		t = d.scalarmult((Sigma.invert()).mult(d));

// Das ist der alte Code mit der Pseudoinversen (entfernt 13.1.05 ChB)
//		Matrix A = rprop.getJacobian();
//		Matrix B = e2.getJacobianOfRelation(rprop.getClass(), e1.getClass());
//		d = A.mult(e2.getV());
//		Sigma =
//			A.mult(e2.getCov()).mult(A.trans()).plus(
//				B.mult(e1.getCov()).mult(B.trans()));
//		t = d.scalarmult((Sigma.pseudoInverse(dof)).mult(d));
				
		chi2 = getChiSquareDistributionValue(alpha, dof);
	}

	/** Create the relational property out of an entity and a relation type
	 *  @param _first Entity
	 *  @param _second type of the second Entity.
	 *  @param relationType the type of the relation.
	 *   
	 *
	 */
	private void createRelationalProperty(
		Entity _first,
		Class _second,
		Class relationType)
		throws IllegalArgumentException {
		try {
			Class[] parameterTypes = new Class[2];
			parameterTypes[0] = _first.getClass();
			parameterTypes[1] = Class.class;
			Constructor constructor =
				relationType.getDeclaredConstructor(parameterTypes);
			Object[] parameters = new Object[2];
			parameters[0] = _first;
			parameters[1] = _second;
			Object r = constructor.newInstance(parameters);
			rprop = (BiRelationalProperty) r;

		} catch (NoSuchMethodException e) {
			throw new IllegalArgumentException(
				"Illegal Relation "
					+ relationType.toString()
					+ " between "
					+ _first.getClass().toString()
					+ " and "
					+ _second.toString());
		} catch (InvocationTargetException e) {
			throw new IllegalArgumentException(
				"Illegal Relation "
					+ relationType.toString()
					+ " between "
					+ _first.getClass().toString()
					+ " and "
					+ _second.toString());
		} catch (IllegalAccessException e) {
			throw new IllegalArgumentException(
				"Illegal Relation "
					+ relationType.toString()
					+ " between "
					+ _first.getClass().toString()
					+ " and "
					+ _second.toString());
		} catch (InstantiationException e) {
			throw new IllegalArgumentException(
				"Illegal Relation "
					+ relationType.toString()
					+ " between "
					+ _first.getClass().toString()
					+ " and "
					+ _second.toString());
		}
	}

} // end Relation
