/**
 * Created on 13.03.2003
 *
 */
package sugr;

import sugr.linalg.*;

/**
 * Class containing rotation (1) constructors. 
 * @author Norbert Fischer
 */
public class Rotation_2D extends Motion_2D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard rotation matrix. Creates an identity matrix
	 */
	public Rotation_2D() {}
	
	/**
	 * Creates a rotation matrix which rotates around angle 
	 * @param angle   angle to rotate, given in radians 
	 */
	public Rotation_2D(double angle)
	{
		this(Math.cos(angle), Math.sin(angle));
	}

	/**
	 * Creates a rotation matrix which rotates around angle 
	 * @param angle   angle to rotate, given in radians
	 * @param cov   9x9 Matrix 
	 */
	public Rotation_2D(double angle, Matrix cov)
	{
		this(Math.cos(angle), Math.sin(angle), cov);
	}

	
	/**
	 * set rotation matrix by hand. the matrix has then following form:
	 * <pre>
	 *      |a -b 0|
	 *      |b  a 0|
 	 *      |0  0 1|
 	 * </pre> 
	 * @param a   equivalent to cos(angle) 
	 * @param b   equivalent to sin(angle)
	 */
	public Rotation_2D(double a, double b)
	{
		this(a,b, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}
	
	/**
	 * set rotation matrix with covariance by hand. the matrix 
	 * has then following form:
	 * <pre>
	 *      |a -b 0|
	 *      |b  a 0|
 	 *      |0  0 1|
 	 * </pre> 
	 * @param a   equivalent to cos(angle) 
	 * @param b   equivalent to sin(angle)
	 * @param cov   9x9 matrix
	 */
	public Rotation_2D(double a, double b, Matrix cov)
	{
		Matrix rot = LinearAlgebraFactory.getMatrix(DIM, DIM);
		rot.set(0,0, a);
		rot.set(1,1, a);
		rot.set(0,1, -b);
		rot.set(1,0,  b);
		rot.set(2,2, 1);
		setV(rot.vec());
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}
	
	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 1;
	}	
	
}
