/*
 * Created on 14.04.2005
 *
 */
package sugr.linalg;

/**
 * @author beder
 *
 */
public class LinearAlgebraFactoryColt {

	///////////////////////////////////////
	// operations

	/**
	 * Return a Matrix no_rows x no_cols filled with zeros.
	 * @param no_rows   Number of rows the matrix should have
	 * @param no_cols   Number of columns the matrix should have
	 * @return a no_rows x no_cols - Matrix
	 */
	public static Matrix getMatrix(int no_rows, int no_cols) {
		return new MatrixImplColt(no_rows, no_cols);
	} // end getMatrix        

	/**
	 * Return a Vector size x 1 filled with zeros.
	 * @return a Vector with dimension size 
	 * @param size   Dimension the vector should have
	 */
	public static Vector getVector(int size) {
		return new VectorImplColt(size);
	} // end getVector        

	/**
	 * create a new matrix by a given double precision floating point array.
	 * e.g.:
	 * <pre>
	 *    double[][] array = {  {0,1,2,3}, 
	 *                          {0,1,2,3}, 
	 *                          {0,1,2,3}};
	 *    Matrix A = LinearAlgebraFactory.getMatrix(array);
	 * </pre>
	 * Matrix A has dimension 3x4.
	 * @param values   given in double
	 * @return a Matrix
	 */
	public static Matrix getMatrix(double[][] values) {
		return new MatrixImplColt(values);
	} // end getMatrix        

	/**
	 * create new Matrix given Matrix. Functionally it works like copy
	 * @param A  Matrix to clone 
	 * @return a new Matrix with same entries as given Matrix
	 */
	public static Matrix getMatrix(Matrix A) {
		return new MatrixImplColt(A);
	} // end getMatrix

	/**
	 * Create new diagonal matrix.
	 * @param size the number of rows and columns of the matrix
	 * @param d the value the diagonal elements should have  
	 * @return a new Matrix
	 */
	public static Matrix getDiagMatrix(int size, double d) {
		Matrix m = new MatrixImplColt(size, size);
		for (int i = 0; i < size; i++)
			m.set(i, i, d);

		return m;
	} // end getMatrix

	/**
	 * create a new Vector by a given double precision floating point array.
	 * @param values    given in double
	 * @return a Vector with dimension = #entries in array
	 */
	public static Vector getVector(double[] values) {
		return new VectorImplColt(values);
	} // end getVector
	
	/**
	 * create a new Vector by 2 doubles
	 * @param a first double value
	 * @param b second double value
	 * @return a Vector with dimension 2
	 */
	public static Vector getVector(double a, double b) {
		double [] d={a,b};
	    
		return new VectorImplColt(d);
	} // end getVector 
	
	/**
	 * create a new Vector by 3 doubles
	 * @param a first double value
	 * @param b second double value
	 * @param c third double value
	 * @return a Vector with dimension 3
	 */
	public static Vector getVector(double a, double b, double c) {
	    double [] d={a,b,c};
	    
		return new VectorImplColt(d);
	} // end getVector
	
	/**
	 * create a new Vector by 4 doubles
	 * @param a first double value
	 * @param b second double value
	 * @param c third double value
	 * @param d fourth double value
	 * @return a Vector with dimension 4
	 */
	public static Vector getVector(double a, double b, double c, double d) {
		double [] da={a,b,c,d};
	    
		return new VectorImplColt(da);
	} // end getVector        
        
        

	/**
	 * create new Vector by given Vector. Functionally it works like copy
	 * @param v    Vector to clone 
	 * @return a Vector
	 */
	public static Vector getVector(Vector v) {
		return new VectorImplColt(v);
	} // end getVector     

	/**
	 * creates an identity Matrix 
	 * @param dim   dimension the Matrix should have
	 * @return a identity-Matrix with dimension dim  
	 */
	public static Matrix getMatrixId(int dim) {
		Matrix tmp = new MatrixImplColt(dim, dim);
		for (int i = 0; i < dim; i++)
			tmp.set(i, i, 1);

		return tmp;
	} // end getMatrixId        

	/**
	 * creates a Matrix from a given Vector. May be useful for some matrix-multiplication
	 * action
	 * @param v   Vector to create Matrix from
	 * @return a Matrix with dimension (v.dim() x 1) 
	 */
	public static Matrix getMatrix(Vector v) {
		Matrix tmp = LinearAlgebraFactory.getMatrix(v.getDim(), 1);
		for (int i = 0; i < v.getDim(); i++) {
			tmp.set(i, 0, v.get(i));
		}
		return tmp;
	} // end getMatrix        
	
	
}
