/** Java interface "Vector.java" generated from Poseidon for UML.
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr.linalg;

import java.io.Serializable;

/**
 * Interface to vector-methods
 * <br><br>
 * Note that s stands for this vector (source), r means result vector and letter
 * a is chosen for scalar values, in all cases a double precision floating point.
 * 
 * @author Daniel Goldbach
 * @author Norbert Fischer
 */
public interface Vector extends Cloneable,Serializable {

	///////////////////////////////////////
	// operations

	/**
	 * Add two vectors: r = s+v
	 * 
	 * @return s+v
	 * @param v   vector to add
	 */
	public Vector plus(Vector v);

	/**
	 * Substracts to vectors: r = s-v
	 * @return s-v
	 * @param v   vector to substract
	 */
	public Vector minus(Vector v);

	/**
	 * scalarmultiplication (or dot product)of two vectors: a = s*v, where a is 
	 * a scalar value  
	 * @return s*v, where a = s_0*v_0 + s_1*v_1 + ... + s_n*v_n as double
	 * @param v   vector to scalarmultiplicate
	 */
	public double scalarmult(Vector v);

	/**
	 * calulates the cross product: r = s x v. 
	 * @return a Vector orthogonal to both s and v
	 * @param v   vector to create cross product with
	 */
	public Vector crossProduct(Vector v);

	/**
	 * Multiplication with scalar: r = a*s 
	 * @return a Vector multiplicated with a scalar
	 * @param f   scalar value
	 */
	public Vector mult(double f);

	/**
	 * Copies s into a new vector r 
	 * @return a Vector r independent from s with s=r 
	 */
	public Vector copy();

	/**
	 * Set an element of s
	 * @param pos   Position to set a new value. First position is 0
	 * @param value   new value to set
	 */
	public void set(int pos, double value);
	
	/** Set a part of a vector
	 * 
	 * @param pos begining position (inclusive)
	 * @param v vector which hold the values to be set. All the values from the
	 *        vector are copied to the target.
	 */
	public void set(int pos, Vector v);

	/**
	 * Gets an element from s 
	 * @param pos   Position to get element from. Starts with 0
	 * @return the element as an double
	 */
	public double get(int pos);

	/** Get a part of the vector.
		 * @param i0  index of first vector element
		 * @param i1  index of the last vector element
		 * @return a vector which is a part of the original vector.
		 */
	public Vector get(int i0, int i1);
	

	/**
	 * Returns the dimension of s 
	 * @return a int containing the dimension
	 */
	public int getDim();

	/**
	 * Calculates the length of s.  
	 * @return a = sqrt(s*s)
	 */
	public double getLength();

	/**
	 * Calculates the Frobenius norm of a vector, which is the same
	 * as the getLength().  
	 * @return a = sqrt(s*s)
	 */
	public double normF();

	/**
	 * Calculates the maximum norm of a vector  
	 * @return a = max|v_i|
	 */
	public double normM();
	
	
	/**
	 * Prints this matrix to stdout <br>
	 * Note:It may be useful to select w > d. A common value is w=5, d=3
	 * @param w   describes the width of a single element
	 * @param d   number of decimals after comma
	 */
	public void print(int w, int d);
	
	/**
	 * Prints this matrix to stdout <br>
	 */
	public void print();

	/**
	 * Prints this matrix in Matlab format to stdout <br>
	 */
	public void printMatlab();

	/** Delivers a string with the elements of the matrix.
	 * @return a string for printing
	 */
	public String toString();

	/**
	 * Test, whether s and v are identical or not 
	 * @param obj   vector to test
	 * @return boolean value   true, if identical, otherwise false
	 */
	//overrides equals from Object. Makes it easier to test
	public boolean equals(Object obj);

	/**
	 * Creates a matrix from s <br>
	 * <small> This is a "kind" of typecast. Useful for matrix-/vectormultiplication
	 * </small>
	 * @return vector as a matrix
	 */
	public Matrix getMatrix();

	/**
	 * Creates a skew-symmetrix matrix from s. s must have dimension 3 to do that.
	 * <pre>
	 * e.g.:         | a |                              |  0 -c   b |
	 *       if  s = | b | , then s.Skewsym() returns   |  c  0  -a |
	 *               | c |                              | -b  a   0 |
	 * </pre>
	 * This matrix can be read as a cross product.
	 * @return returns skewsym from 3-dim vector. matrix has rank 2 
	 */
	public Matrix SkewSym();
	
	/** Clone a vector.
	 * @return a copy of the vector.
	 */
	public Object clone(); 

} // end Vector
