/*
 * Created on 19.07.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class Affinity_3DTest extends TestCase {

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Affinity_3D aff = new Affinity_3D();
		int dof = aff.getDegreesOfFreedom();
		if(dof!=12)fail();
	}

	/**
	 * Test for void Affinity_3D()
	 */
	public void testAffinity_3D() {
		Affinity_3D aff = new Affinity_3D();
		if(aff==null)fail();
	}

	/**
	 * Test for void Affinity_3D(double[])
	 */
	public void testAffinity_3DdoubleArray() {
		double[] dArr = new double[12];
		for(int i=0;i<12;i++){
			dArr[i]=i+1;
		}
		Affinity_3D aff = new Affinity_3D(dArr);
		double[][] affArr = new double[4][4];
		for(int i=0;i<12;i++){
			affArr[i%3][i/3]=i+1;
		}
		for(int i=0;i<3;i++){
			affArr[3][i]=0;
		}
		affArr[3][3]=1;
		Matrix transMat = aff.getTransformationMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(affArr[i][j]!=transMat.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void Affinity_3D(double[], Matrix)
	 */
	public void testAffinity_3DdoubleArrayMatrix() {
        //array for covariance Matrix
        double[][] covArray=new double[16][16];
        //random Values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else {
                	if(i==j)covArray[i][j]=Math.random()*1e-1;
                	else covArray[i][j]=covArray[j][i];
                }
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		double[] dArr = new double[12];
		for(int i=0;i<12;i++){
			dArr[i]=i+1;
		}
		Affinity_3D aff = new Affinity_3D(dArr,covMat);
		double[][] affArr = new double[4][4];
		for(int i=0;i<12;i++){
			affArr[i%3][i/3]=i+1;
		}
		for(int i=0;i<3;i++){
			affArr[3][i]=0;
		}
		affArr[3][3]=1;
		Matrix transMat = aff.getTransformationMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(affArr[i][j]!=transMat.get(i,j))fail();
			}
		}
		Matrix zeroM=aff.getCov().minus(covMat);
		double zeroD=zeroM.vec().getLength();
		if(zeroD>1e-14)fail();
	}

	/**
	 * Test for void Affinity_3D(Rotation_3D, Translation_3D, Vector, Vector)
	 * Test for error propagation must not be done here, as scale and shear do not have errors.
	 */
	public void testAffinity_3DRotation_3DTranslation_3DVectorVector() {
		//set up rotation and translation
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI,Math.PI);
		Translation_3D trans = new Translation_3D(1,2,3);
		//set up scale
		Vector scale = LinearAlgebraFactory.getVector(2,4,6);
		double[][] scaleArray = {{2,0,0,0},{0,4,0,0},{0,0,6,0},{0,0,0,1}};
		Matrix scaleMat = LinearAlgebraFactory.getMatrix(scaleArray);
		//set up shear
		double[] shearArray = {2,3,5,7,5,11};
		Vector shear = LinearAlgebraFactory.getVector(shearArray);
		double[][] shearArrArr={{1,2,3,0},{7,1,5,0},{5,11,1,0},{0,0,0,1}};
		Matrix shearMat = LinearAlgebraFactory.getMatrix(shearArrArr);
		//calculating Affinity_3D with SUGR
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear);
		Matrix affMatSugr = aff.getTransformationMatrix();
		//calculating via hand
		Matrix affMatHand = rot.getTransformationMatrix().mult(trans.getTransformationMatrix())
								.mult(scaleMat).mult(shearMat);
		//check result
		Matrix zeroM = affMatSugr.minus(affMatHand);
		double zeroD = zeroM.vec().getLength();
		if (zeroD>1e-12)fail();
	}

	/**
	 * Test for void Affinity_3D(Rotation_3D, Translation_3D, Vector, Vector, Matrix)
	 */
	public void testAffinity_3DRotation_3DTranslation_3DVectorVectorMatrix() {
        //set up covariance Matrix
        double[][] covArray=new double[16][16];
        //random values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else {
                	if(i==j)covArray[i][j]=Math.random()*1e-1;
                	else covArray[i][j]=covArray[j][i];
                }
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);

		//set up rotation and translation
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI,Math.PI);
		Translation_3D trans = new Translation_3D(1,2,3);
		//set up scale
		Vector scale = LinearAlgebraFactory.getVector(2,4,6);
		double[][] scaleArray = {{2,0,0,0},{0,4,0,0},{0,0,6,0},{0,0,0,1}};
		Matrix scaleMat = LinearAlgebraFactory.getMatrix(scaleArray);
		//set up shear
		double[] shearArray = {2,3,5,7,5,11};
		Vector shear = LinearAlgebraFactory.getVector(shearArray);
		double[][] shearArrArr={{1,2,3,0},{7,1,5,0},{5,11,1,0},{0,0,0,1}};
		Matrix shearMat = LinearAlgebraFactory.getMatrix(shearArrArr);
		//calculating Affinity_3D with SUGR
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear,covMat);
		Matrix affMatSugr = aff.getTransformationMatrix();
		//calculating via hand
		Matrix affMatHand = rot.getTransformationMatrix().mult(trans.getTransformationMatrix())
								.mult(scaleMat).mult(shearMat);
		//check result
		Matrix zeroM = affMatSugr.minus(affMatHand);
		double zeroD = zeroM.vec().getLength();
		if (zeroD>1e-12)fail();
		zeroM = aff.getCov().minus(covMat);
		zeroD = zeroM.vec().normF();
		if(zeroD>1e-14)fail();
	}
}