/*
 * Created on 04.03.2004
 *
 */
package sugr.test;
import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class EntityTest extends TestCase {

	/**
	 * Constructor for ElementTest.
	 * @param arg0
	 */
	public EntityTest(String arg0) {
		super(arg0);
	}

	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 1 000 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 * If MAXITER is set to 1 000 000, it may happen in an 'Out of Memory'-Error!
	 */
	public static int MAXITER = 1000000;

	/**
	 * double PRECISION for testing statistics. 
	 */
	public static double PRECISION =0.15;

	/**
	 * Test for double getNormOfEuclideanPart()
	 */
	public void testGetNormOfEuclideanPart(){
		Point_3D point = new Point_3D(8,4,1,1);
		double euNorm = point.getNormOfEuclideanPart();
		if(euNorm!=9)fail();
	}

	/**
	 * Test for double getNormOfHomogeneousPart()
	 */
	public void testGetNormOfHomogeneousPart(){
		Plane_3D plane = new Plane_3D(8,4,1,5);
		double homNorm = plane.getNormOfHomogeneousPart();
		if(homNorm!=9)fail();
	}

	/**
	 * Test for non-statistical part of void normalizeEuclideanComparable()
	 */
	public void testNormalizeEuclideanComparable(){
		//Test1
		Point_3D point = new Point_3D(-8,-4,-1,-1);
		Vector vec = LinearAlgebraFactory.getVector(8,4,1,1);
		point.normalizeEuclideanComparable();
		if (!TestStatistics.check(point.getV(),vec,PRECISION))fail();
		//Test2
		Plane_3D plane = new Plane_3D(-3,-4,0,1);
		vec = LinearAlgebraFactory.getVector(0.6,0.8,-0.0,-0.2);
		plane.normalizeEuclideanComparable();
		if (!TestStatistics.check(plane.getV(),vec,PRECISION))fail();
	}

	/**
	 * Test for error propagation of void normalizeEuclideanComparable()
	 */
	public void testNormalizeEuclideanComparableStatistics(){
		double[][] covArray = {{0.04,0.01,0.01,0.01},{0.01,0.04,0.01,0.01},{0.01,0.01,0.04,0.01},{0.01,0.01,0.01,0.04}};
		Matrix cov = LinearAlgebraFactory.getMatrix(covArray);
		Vector vec = LinearAlgebraFactory.getVector(3,4,2,5);
		//Test with Point_3D
		Point_3D point = new Point_3D(vec,cov);
		point.normalizeEuclideanComparable();
		TestStatistics statistic = new TestStatistics(4);
		RandomVector rv = new RandomVector(vec,cov);
		Point_3D point2;
		for (int i = 0; i < MAXITER; i++) {
			point2 = new Point_3D(rv.getNextRandom(),cov);
			point2.normalizeEuclideanComparable();
			statistic.addObservation(point2);
		}
		if (!TestStatistics.check(point.getV(),statistic.getMue(),
									point.getCov(),statistic.getSigma(),PRECISION))
			fail();
		
		//Test with Plane_3D
		Plane_3D plane = new Plane_3D(vec,cov);
		plane.normalizeEuclideanComparable();
		statistic = new TestStatistics(4);
		rv = new RandomVector(vec,cov);
		Plane_3D plane2;
		for (int i = 0; i < MAXITER; i++) {
			plane2 = new Plane_3D(rv.getNextRandom(),cov);
			plane2.normalizeEuclideanComparable();
			statistic.addObservation(plane2);
		}
		if (!TestStatistics.check(plane.getV(),statistic.getMue(),
									plane.getCov(),statistic.getSigma(),PRECISION))
			fail();
		
		//Test with Line_2D
		double[][] covArray2D = {{0.01,0,0},{0,0.01,0},{0,0,0.01}};
		cov = LinearAlgebraFactory.getMatrix(covArray2D);
		vec = LinearAlgebraFactory.getVector(3,4,2);
		Line_2D line = new Line_2D(vec,cov);
		line.normalizeEuclideanComparable();
		statistic = new TestStatistics(3);
		rv = new RandomVector(vec,cov);
		Line_2D line2;
		for (int i = 0; i < MAXITER; i++) {
			line2 = new Line_2D(rv.getNextRandom(),cov);
			line2.normalizeEuclideanComparable();
			statistic.addObservation(line2);
		}
		if (!TestStatistics.check(line.getV(),statistic.getMue(),
									line.getCov(),statistic.getSigma(),PRECISION))
			fail();
	}
}