/*
 * Created on 17.05.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * @author Bastian Oehler
 *  
 */
public class Homography_2DTest extends TestCase {

	/**
	 * Constructor for Homography_2DTest.
	 * 
	 * @param arg0
	 */
	public Homography_2DTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		Matrix matr = LinearAlgebraFactory.getDiagMatrix(9, 1e-5);
		Homography_2D homogr = new Homography_2D(doub, matr);
		int dof = homogr.getDegreesOfFreedom();
		if (dof != 8)
			fail();
	}

	/**
	 * Test for Entity transform(Entity) TODO: IllegalArgumentException???
	 */
	public void testTransformEntity() {
		int MAXITER = (int) 200000;

		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
				"Testing transformEntity", "with Point_2D and Line_2D", 0,
				2 * MAXITER);

		double[] homArr = { 3, 5, 7, 1, 9, 34, 16, 12, 1 };
		double[][] covArrHom = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (i == j)
					covArrHom[i][j] = 1e-2 * Math.random();
				else if (i < j)
					covArrHom[i][j] = 1e-4 * Math.random();
				else
					covArrHom[i][j] = covArrHom[j][i];
			}
		}
		Matrix covMatHom = LinearAlgebraFactory.getMatrix(covArrHom);
		Vector homVec = LinearAlgebraFactory.getVector(homArr);
		Homography_2D homogr = new Homography_2D(homArr, covMatHom);
		homogr.setV(homVec);
		boolean broken = false;

		{
			//init point
			double[][] covArrPoint = { { 0.011, 0.003, 0.002 },
					{ 0.003, 0.012, 0.004 }, { 0.002, 0.004, 0.013 } };
			Matrix pointCov = LinearAlgebraFactory.getMatrix(covArrPoint);
			Vector pointVec = LinearAlgebraFactory.getVector(32, 93, 10);
			Point_2D point = new Point_2D(pointVec, pointCov);
			point.normalizeEuclidean();
			//transform Point_2D point with Homography_2D homogr
			Point_2D transPoint = (Point_2D) homogr.transform(point);
			//check result of transformation
			for (int i = 0; i < 3; i++)
				if (transPoint.getV().get(i) != homogr
						.getTransformationMatrix().mult(point.getV()).get(i))
					fail();
			//test statistics
			TestStatistics stats = new TestStatistics(3);
			RandomVector rv = new RandomVector(pointVec, pointCov);
			RandomVector rvHom = new RandomVector(homVec, covMatHom);
			Point_2D pointStats;
			Point_2D pointStatsTrans;
			Homography_2D homStats = new Homography_2D(homArr, covMatHom);
			for (int i = 0; i < MAXITER; i++) {
				pointStats = new Point_2D(rv.getNextRandom(), pointCov);
				homStats.setV(rvHom.getNextRandom());
				pointStatsTrans = (Point_2D) homStats.transform(pointStats);
				stats.addObservation(pointStatsTrans);
				//break if cancel button is pressed
				if (pm.isCanceled()) {
					broken = true;
					break;
				}
				//update progress monitor.
				pm.setProgress(i);
			}
			if (broken)
				pm.close();
			Point_2D simuPoint = new Point_2D(stats.getMue(), stats.getSigma());
			simuPoint.normalizeEuclideanComparable();
			transPoint.normalizeEuclideanComparable();
			if (!TestStatistics.check(simuPoint, transPoint, 0.1))
				fail();
			//System.out.println(simuPoint);
			//System.out.println(transPoint);

		}


		{
			//transform gaussian lines with Homograpy_2D homogr
			double[][] covArrLine = { { 0.011, 0.003, 0.0002 },
					{ 0.003, 0.012, 0.004 }, { 0.0002, 0.004, 0.013 } };
			Matrix lineCov = LinearAlgebraFactory.getMatrix(covArrLine);

			Vector vecLine = LinearAlgebraFactory.getVector(0.5, 3, 1);
			Line_2D line = new Line_2D(vecLine, lineCov);
			Line_2D transLine = (Line_2D) homogr.transform(line);
			//calculate Vector of transformed line
			Matrix help = homogr.getTransformationMatrix().trans().invert();
			Vector transLineCalcVec = help.mult(line.getV());
			//check result of transformation
			if (!TestStatistics
					.check(transLine.getV(), transLineCalcVec, 1e-12))
				fail();
			//test statistics 
			if (!broken) {
				TestStatistics stats = new TestStatistics(3);
				RandomVector rvHom = new RandomVector(homVec,covMatHom);
				Homography_2D homStats = new Homography_2D(homArr,
						LinearAlgebraFactory.getMatrix(9, 9));
				homStats.setV(homVec);
				RandomVector rvLine = new RandomVector(vecLine, lineCov);
				Line_2D lineStats;
				Line_2D lineStatsTrans;
				for (int i = 0; i < MAXITER; i++) {
					lineStats = new Line_2D(rvLine.getNextRandom(), lineCov);
					homStats.setV(rvHom.getNextRandom());
					lineStatsTrans = (Line_2D) homStats.transform(lineStats);
					stats.addObservation(lineStatsTrans);
					//break if cancel button is pressed
					if (pm.isCanceled())
						break;
					//update progress monitor.
					pm.setProgress(MAXITER + i);
				}
				pm.close();
				Line_2D simuLine = new Line_2D(stats.getMue(), stats.getSigma());
				simuLine.normalizeEuclideanComparable();
				transLine.normalizeEuclideanComparable();
				//System.out.println("Statistik:\n" + simuLine);
				//System.out.println("SUGR:\n" + transLine);
				if (!TestStatistics.check(simuLine, transLine, 0.1))
					fail();
			}
		}

		//try illegal arguments
		try {
			Matrix lineCov = LinearAlgebraFactory.getDiagMatrix(6, 1e-3);
			Line_3D e = new Line_3D(3, 6, 7, 1, 3, 9, lineCov);
			homogr.transform(e);
			fail();
		} catch (NullPointerException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Point_3D e = new Point_3D(1, 2, 3, 4, cov);
			homogr.transform(e);
			fail();
		} catch (NullPointerException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Plane_3D e = new Plane_3D(4, 6, 2, 1, cov);
			homogr.transform(e);
			fail();
		} catch (NullPointerException e) {
		}
	}

	/**
	 * Test for int getDimRow()
	 */
	public void testGetDimRow() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		Matrix matr = LinearAlgebraFactory.getDiagMatrix(9, 1e-5);
		Homography_2D homogr = new Homography_2D(doub, matr);
		int rowDim = homogr.getDimRow();
		if (rowDim != 3)
			fail();
	}

	/**
	 * Test for int getDimCol()
	 */
	public void testGetDimCol() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		Matrix matr = LinearAlgebraFactory.getDiagMatrix(9, 1e-5);
		Homography_2D homogr = new Homography_2D(doub, matr);
		int colDim = homogr.getDimCol();
		if (colDim != 3)
			fail();
	}

	/**
	 * Test for Matrix getJacobianAfterFirst(Entity)
	 */
	public void testGetJacobianAfterFirst() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		double[][] covMat = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (j >= i)
					covMat[i][j] = 1e-2 * Math.random();
				else
					covMat[i][j] = -covMat[j][i];
			}
		}
		Matrix matr = LinearAlgebraFactory.getMatrix(covMat);
		Homography_2D homogr = new Homography_2D(doub, matr);

		//set up point
		Point_2D point = new Point_2D(2, 3, 1);
		//calculate Jacobian
		Matrix jacPHand = point.getV().SkewSym().mult(
				homogr.getTransformationMatrix());
		Matrix jacPSUGR = homogr.getJacobianAfterFirst(point);
		//check result
		if (!TestStatistics.check(jacPHand, jacPSUGR, 1e-10))
			fail();

		//set up line
		Line_2D line = new Line_2D(5, 3, 1);
		//calculate Jacobian
		Matrix jacLHand = homogr.getTransformationMatrix().trans().mult(
				line.getV()).SkewSym().mult(-1.0);
		Matrix jacLSUGR = homogr.getJacobianAfterFirst(line);
		//check result
		if (!TestStatistics.check(jacLHand, jacLSUGR, 1e-10))
			fail();

		//try illegal arguments
		try {
			Matrix lineCov = LinearAlgebraFactory.getDiagMatrix(6, 1e-3);
			Line_3D e = new Line_3D(3, 6, 7, 1, 3, 9, lineCov);
			homogr.getJacobianAfterFirst(e);
			fail();
		} catch (IllegalArgumentException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Point_3D e = new Point_3D(1, 2, 3, 4, cov);
			homogr.getJacobianAfterFirst(e);
			fail();
		} catch (IllegalArgumentException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Plane_3D e = new Plane_3D(4, 6, 2, 1, cov);
			homogr.getJacobianAfterFirst(e);
			fail();
		} catch (IllegalArgumentException e) {
		}
	}

	/**
	 * Test for Matrix getJacobianAfterSecond(Entity)
	 */
	public void testGetJacobianAfterSecond() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		double[][] covMat = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (j >= i)
					covMat[i][j] = 1e-2 * Math.random();
				else
					covMat[i][j] = -covMat[j][i];
			}
		}
		Matrix matr = LinearAlgebraFactory.getMatrix(covMat);
		Homography_2D homogr = new Homography_2D(doub, matr);

		//set up point
		Point_2D point = new Point_2D(2, 3, 1);
		//calculate Jacobian
		Matrix jacPHand = homogr.getTransformationMatrix().mult(point.getV())
				.SkewSym().mult(-1.0);
		Matrix jacPSUGR = homogr.getJacobianAfterSecond(point);
		//check result
		if (!TestStatistics.check(jacPHand, jacPSUGR, 1e-10))
			fail();

		//set up line
		Line_2D line = new Line_2D(5, 3, 1);
		//calculate Jacobian
		Matrix jacLHand = line.getV().SkewSym().mult(
				homogr.getTransformationMatrix().trans());
		Matrix jacLSUGR = homogr.getJacobianAfterSecond(line);
		//check result
		if (!TestStatistics.check(jacLHand, jacLSUGR, 1e-10))
			fail();

		//try illegal arguments
		try {
			Matrix lineCov = LinearAlgebraFactory.getDiagMatrix(6, 1e-3);
			Line_3D e = new Line_3D(3, 6, 7, 1, 3, 9, lineCov);
			homogr.getJacobianAfterSecond(e);
			fail();
		} catch (IllegalArgumentException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Point_3D e = new Point_3D(1, 2, 3, 4, cov);
			homogr.getJacobianAfterSecond(e);
			fail();
		} catch (IllegalArgumentException e) {
		}
		try {
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 1e-3);
			Plane_3D e = new Plane_3D(4, 6, 2, 1, cov);
			homogr.getJacobianAfterSecond(e);
			fail();
		} catch (IllegalArgumentException e) {
		}

	}

	/**
	 * Test for void Homography_2D()
	 */
	public void testHomography_2D() {
		Homography_2D emptyHomogr = new Homography_2D();
		if (emptyHomogr == null)
			fail();
	}

	/**
	 * Test for void Homography_2D(Affinity_2D, Vector)
	 */
	public void testHomography_2DAffinity_2DVector() {
		//initialize skew-symmetric covariance matrix
		double[][] covArr = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (j >= i)
					covArr[i][j] = 1e-2 * Math.random();
				else
					covArr[i][j] = covArr[j][i];
			}
		}
		Matrix cov = LinearAlgebraFactory.getMatrix(covArr);
		double[][] arr = { { 1, 3, 5 }, { 2, 4, 6 }, { 7, 8, 1 } };
		Matrix mat = LinearAlgebraFactory.getMatrix(arr);
		Affinity_2D aff = new Affinity_2D(1, 2, 3, 4, 5, 6, cov);
		Vector vec = LinearAlgebraFactory.getVector(7, 8);
		Homography_2D hom = new Homography_2D(aff, vec);
		for (int i = 0; i < 9; i++) {
			if (hom.getTransformationMatrix().get(i / 3, i % 3) != mat.get(
					i / 3, i % 3))
				fail();
		}
		Matrix transCov = hom.getCov();
		Matrix zeroM = transCov.minus(cov);
		double zeroD = zeroM.vec().getLength();
		if (zeroD > 1e-12)
			fail();
	}

	/** Test for Homography2D(Homography2D,Homography2D). */
	public void testHomography_2DHomography2DHomography2D() {
		Matrix cov1 = LinearAlgebraFactory.getMatrix(9, 9);
		cov1.set(0, 0, 0.001);
		cov1.set(1, 1, 0.001);
		cov1.set(2, 2, 2e-03);
		cov1.set(3, 3, 1e-03);
		cov1.set(4, 4, 1e-03);
		cov1.set(5, 5, 1e-03);
		cov1.set(6, 6, 1e-03);
		cov1.set(7, 7, 1e-03);
		cov1.set(8, 8, 1e-03);

		cov1.set(0, 1, 0.0002);
		cov1.set(1, 0, 0.0002);
		cov1.set(0, 2, 0.00021);
		cov1.set(2, 0, 0.00021);
		cov1.set(3, 1, 0.0002);
		cov1.set(1, 3, 0.0002);
		cov1.set(3, 2, 0.000021);
		cov1.set(2, 3, 0.000021);

		double[] h1a = { 1, 0.1, 2, 0.1, 1.2, 0.1, 0.3, 0.5, -1 };

		Homography_2D h1 = new Homography_2D(h1a, cov1);

		Matrix cov2 = LinearAlgebraFactory.getMatrix(9, 9);
		/*
		cov2.set(0, 0, 0.01);
		cov2.set(1, 1, 0.01);
		cov2.set(2, 2, 1e-02);
		cov2.set(3, 3, 1e-02);
		cov2.set(4, 4, 1e-02);
		cov2.set(5, 5, 2e-02);
		cov2.set(6, 6, 1e-02);
		cov2.set(7, 7, 1e-02);
		cov2.set(8, 8, 1e-02);

		cov2.set(0, 1, 0.0002);
		cov2.set(1, 0, 0.0002);
		cov2.set(0, 2, 0.00021);
		cov2.set(2, 0, 0.00021);
		cov2.set(3, 1, 0.0002);
		cov2.set(1, 3, 0.0002);
		cov2.set(3, 2, 0.00021);
		cov2.set(2, 3, 0.00021);
		*/

		double[] h2a = { 1, 3, 4, 6, 1, 5, 7, 8, 1 };

		Homography_2D h2 = new Homography_2D(h2a, cov2);

		// Test des Ergebnisses:
		Matrix soll = h1.getTransformationMatrix().mult(
				h2.getTransformationMatrix());
		Homography_2D hsoll = new Homography_2D();
		hsoll.setV(soll.vec());
		hsoll.setCov(LinearAlgebraFactory.getMatrix(9,9));
		hsoll.normalizeSphere();
		Homography_2D hist = new Homography_2D(h1, h2);
		hist.normalizeSphere();
		if (!TestStatistics.check(hsoll.getV(), hist.getV(), 1e-3))
			fail();

		//RandomVector rvh2 = new RandomVector(h2.getV(), cov2);
		RandomVector rvh1 = new RandomVector(h1.getV(), cov1);

		TestStatistics stats = new TestStatistics(9);

		int MAXITER = 1000000;

		Vector tmp;
		Homography_2D hs;
		Homography_2D h1s = (Homography_2D) h1.clone();
		Homography_2D h2s = (Homography_2D) h2.clone();
		//System.out.println("Start Statistik mit MAXITER=" + MAXITER);

		// Test only varying one homography because otherwise MAXITER
		// must be too large
		for (int i = 0; i < MAXITER; i++) {
			//tmp = rvh2.getNextRandom();
			//h2s.setV(tmp);
			tmp = rvh1.getNextRandom();
			h1s.setV(tmp);

			hs = new Homography_2D(h1s, h2s);
			stats.addObservation(hs);
			//System.out.print(" "+i);
		}

		Matrix sigma = stats.getSigma();
		//System.out.println("Statistik: " + stats.getMue());
		//System.out.println("Statistik: " + sigma);

		Homography_2D hsugr = new Homography_2D(h1, h2);
		//System.out.println("SUGR: " + hsugr);
		//System.out.println("DIFF Covariance: " + hsugr.getCov().minus(sigma));

		if (!TestStatistics.check(hsugr.getV(), stats.getMue(), hsugr.getCov(),
				sigma, 0.3)) fail();
	}

	/**
	 * Test for void Homography_2D(Affinity_2D, Vector, Matrix)
	 */
	public void testHomography_2DAffinity_2DVectorMatrix() {
		//initialize skew-symmetric covariance matrix
		double[][] covArr = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (j >= i)
					covArr[i][j] = 1e-2 * Math.random();
				else
					covArr[i][j] = covArr[j][i];
			}
		}
		Matrix cov = LinearAlgebraFactory.getMatrix(covArr);
		//init Matrix mat for test of aff -> hom
		double[][] arr = { { 1, 3, 5 }, { 2, 4, 6 }, { 7, 8, 1 } };
		Matrix mat = LinearAlgebraFactory.getMatrix(arr);
		//init Affinity aff
		Affinity_2D aff = new Affinity_2D(1, 2, 3, 4, 5, 6);
		Vector vec = LinearAlgebraFactory.getVector(7, 8);
		Homography_2D hom = new Homography_2D(aff, vec, cov);
		for (int i = 0; i < 9; i++) {
			if (hom.getTransformationMatrix().get(i / 3, i % 3) != mat.get(
					i / 3, i % 3))
				fail();
		}
		Matrix transCov = hom.getCov();
		Matrix zeroM = transCov.minus(cov);
		double zeroD = zeroM.vec().getLength();
		if (zeroD > 1e-12)
			fail();
	}

	/**
	 * Test for void Homography_2D(double[])
	 */
	public void testHomography_2DdoubleArray() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		Homography_2D homogr = new Homography_2D(doub);
		Matrix transMatr = homogr.getTransformationMatrix();
		for (int i = 0; i < 9; i++) {
			if (transMatr.get(i / 3, i % 3) != doub[i])
				fail();
		}
	}
	
	/**
	 * Test for void Homography_2D(Matrix, Matrix)
	 */
	public void testHomography_2DMatrixMatrix() {
		Matrix hom=LinearAlgebraFactory.getDiagMatrix(3, 1.0);
		hom.set(0,0, 1.0);
		hom.set(0,1, 2.0);
		hom.set(0,2, 3.0);
		hom.set(1,0, 4.0);
		hom.set(1,1, 5.0);
		hom.set(1,2, 6.0);
		hom.set(2,0, 7.0);
		hom.set(2,1, 8.0);
		hom.set(2,2, 9.0);
		
		Matrix cov=LinearAlgebraFactory.getDiagMatrix(9, 0.01);
		cov.set(0,0, 1.0);
		cov.set(0,1, 2.0);
		cov.set(0,2, 3.0);
		cov.set(1,0, 4.0);
		cov.set(1,1, 5.0);
		cov.set(1,2, 6.0);
		cov.set(2,0, 7.0);
		cov.set(2,1, 8.0);
		cov.set(2,2, 9.0);
		
		Homography_2D H=new Homography_2D(hom, cov);
		
		if (!TestStatistics.check(H.getCov().copy(), cov, 1e-10))
			fail();
		if (!TestStatistics.check(H.getTransformationMatrix().copy(), hom, 1e-10))
			fail();
	}


	/**
	 * Test for void Homography_2D(double[], Matrix)
	 */
	public void testHomography_2DdoubleArrayMatrix() {
		double[] doub = { 3, 5, 7, 1, 9, 34, 76, 12, 99 };
		double[][] covMat = new double[9][9];
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++) {
				if (j >= i)
					covMat[i][j] = Math.random();
				else
					covMat[i][j] = -covMat[j][i];
			}
		}
		Matrix matr = LinearAlgebraFactory.getMatrix(covMat);
		Homography_2D homogr = new Homography_2D(doub, matr);
		Matrix transMatr = homogr.getTransformationMatrix();
		for (int i = 0; i < 9; i++)
			if (transMatr.get(i / 3, i % 3) != doub[i])
				fail();
		Matrix transCov = homogr.getCov();
		Matrix zeroMatr = transCov.minus(matr);
		for (int i = 0; i < 81; i++)
			if (Math.abs(zeroMatr.get(i / 9, i % 9)) > Math
					.abs(covMat[(i / 9)][(i % 9)] * 1e-12))
				fail();
	}
}