/*
 * Created on 17.05.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 */
public class Homography_3DTest extends TestCase {

	/**
	 * Constructor for Homography_3DTest.
	 * @param arg0
	 */
	public Homography_3DTest(String arg0) {
		super(arg0);
	}

	public Matrix getPi (Vector vec){
		double[][] piArray = {{vec.get(3),0,0,-vec.get(0)}
								,{0,vec.get(3),0,-vec.get(1)}
								,{0,0,vec.get(3),-vec.get(2)}
								,{0,-vec.get(2),vec.get(1),0}
								,{vec.get(2),0,-vec.get(1),0}
								,{-vec.get(1),vec.get(0),0,0}};
		Matrix pi = LinearAlgebraFactory.getMatrix(piArray);
		return pi;
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Homography_3D hom = new Homography_3D();
		if (hom.getDegreesOfFreedom()!=15)fail();
	}

	/**
	 * Test for Entity transform(Point_3D) with uncertain Homography
	 */
	public void testTransformEntityPointUncertainHomography() {
		int MAXITER=(int)10000;
		//initialize Homography_3D hom with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>i)covArray[i][j]=1e-4*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Point_3D point
		Vector pointVec = LinearAlgebraFactory.getVector(4,7.1,19,1);
		//double[][] pointCovArray = {{0.01,0.0005,0.0008,0.00019},{0.0005,0.02,0.00063,0.00196},{0.0008,0.00063,0.0130,0.00073},{0.00019,0.000196,0.00073,0.0015}};
		//Matrix pointCov = LinearAlgebraFactory.getMatrix(pointCovArray);
		Matrix pointCov=LinearAlgebraFactory.getMatrix(4,4);
		Point_3D point= new Point_3D(pointVec,pointCov);
		//transform Point_3D with Homography_3D hom
		Point_3D pointTrans = (Point_3D)hom.transform(point);
		Vector pointCalcVec = hom.getTransformationMatrix().mult(point.getV());
		//check transformation performed by sugr
		for(int i=0;i<4;i++)if(pointTrans.getV().get(i)!=pointCalcVec.get(i))fail();

		//test statistics of transformed Point_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Point_3D uncertain Homography_3D",0,MAXITER);
		TestStatistics stats = new TestStatistics(4);
		//RandomVector rv = new RandomVector(pointVec,pointCov);
		RandomVector rvHom = new RandomVector(homVec,homCov);
		Homography_3D homStats = new Homography_3D(homArray,homCov);
		for (int i=0;i<MAXITER;i++){
			//pointStats=new Point_3D(rv.getNextRandom(),pointCov);
			homStats.setV(rvHom.getNextRandom());
			Point_3D pointStatsTrans=(Point_3D)homStats.transform(point);
			stats.addObservation(pointStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Point_3D simuPoint = new Point_3D(stats.getMue(),stats.getSigma());
		simuPoint.normalizeEuclideanComparable();
		pointTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPoint,pointTrans,0.5)){
			System.out.println("Test for transform(Point_3D) with uncertain Homography_3D");
			System.out.println("Simulation: "+simuPoint);
			System.out.println("sugr: "+pointTrans);
			fail();
		}
	}

	/**
	 * Test for Entity transform(Point_3D) with uncertain Point
	 */
	public void testTransformEntityPointUncertainPoint() {
		int MAXITER=(int)10000;
		//initialize Homography_3D hom with zero covariance matrix
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		Matrix homCov = LinearAlgebraFactory.getMatrix(16,16);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Point_3D point
		Vector pointVec = LinearAlgebraFactory.getVector(4,7.1,19,1);
		double[][] pointCovArray = {{0.01,0.0005,0.0008,0},{0.0005,0.02,0.00063,0},{0.0008,0.00063,0.0130,0},{0,0,0,0}};
		Matrix pointCov = LinearAlgebraFactory.getMatrix(pointCovArray);
		Point_3D point= new Point_3D(pointVec,pointCov);
		//transform Point_3D with Homography_3D hom
		Point_3D pointTrans = (Point_3D)hom.transform(point);
		Vector pointCalcVec = hom.getTransformationMatrix().mult(point.getV());
		//check transformation performed by sugr
		for(int i=0;i<4;i++)if(pointTrans.getV().get(i)!=pointCalcVec.get(i))fail();

		//test statistics of transformed Point_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Point_3D uncertain Point_3D",0,MAXITER);
		TestStatistics stats = new TestStatistics(4);
		RandomVector rv = new RandomVector(pointVec,pointCov);
		//RandomVector rvHom = new RandomVector(homVec,homCov);
		//Homography_3D homStats = new Homography_3D(homArray,homCov);
		Point_3D pointStats;
		for (int i=0;i<MAXITER;i++){
			pointStats=new Point_3D(rv.getNextRandom(),pointCov);
			//homStats.setV(rvHom.getNextRandom());
			Point_3D pointStatsTrans=(Point_3D)hom.transform(pointStats);
			stats.addObservation(pointStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Point_3D simuPoint = new Point_3D(stats.getMue(),stats.getSigma());
		simuPoint.normalizeEuclideanComparable();
		pointTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPoint,pointTrans,0.5)){
			System.out.println("Test for transform(Point_3D) with uncertain Point_3D");
			System.out.println("Simulation: "+simuPoint);
			System.out.println("sugr: "+pointTrans);
			fail();
		}
	}
	
	/**
	 * Test for Entity transform(Line_3D) with uncertain Homography
	 */ 
	public void testTransformEntityLineUncertainHomography() {
		int MAXITER=(int)100000;
		//initialize Homography_3D hom with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>i)covArray[i][j]=1e-5*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Line_3D line
		Point_3D p1=new Point_3D(2,5,3);
		Point_3D p2=new Point_3D(9,4,7);
		Line_3D line=new Line_3D(p1,p2);
		//transform Line_3D with Homography_3D hom
		Line_3D lineTrans = (Line_3D)hom.transform(line);
		Vector lineCalcVec = hom.getLineMatrix().mult(line.getV());
		//check transformation performed by sugr
		for(int i=0;i<6;i++)if(lineTrans.getV().get(i)!=lineCalcVec.get(i))fail();

		//test statistics of transformed Line_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Line_3D uncertain Homography_3D",0,MAXITER);
		TestStatistics stats = new TestStatistics(6);
		//RandomVector rv = new RandomVector(pointVec,pointCov);
		RandomVector rvHom = new RandomVector(homVec,homCov);
		Homography_3D homStats = new Homography_3D(homArray,homCov);
		for (int i=0;i<MAXITER;i++){
			//pointStats=new Point_3D(rv.getNextRandom(),pointCov);
			homStats.setV(rvHom.getNextRandom());
			Line_3D lineStatsTrans=(Line_3D)homStats.transform(line);
			stats.addObservation(lineStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Line_3D simuLine = new Line_3D(stats.getMue(),stats.getSigma());
		simuLine.normalizeEuclideanComparable();
		lineTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuLine,lineTrans,0.5)){
			System.out.println("Test for transform(Line_3D) with uncertain Homography_3D");
			System.out.println("Simulation: "+simuLine);
			System.out.println("sugr: "+lineTrans);
			fail();
		}
	}
	
	/**
	 * Test for Entity transform(Line_3D) with uncertain Line
	 */ 
	public void testTransformEntityLineUncertainLine() {
		int MAXITER=(int)100000;
		//initialize Homography_3D hom with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];

		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Line_3D line
		Matrix covp=LinearAlgebraFactory.getDiagMatrix(4, 0.001);
		covp.set(3,3,0);
		Point_3D p1=new Point_3D(2,5,3, covp);
		Point_3D p2=new Point_3D(9,4,7, covp);
		Line_3D line=new Line_3D(p1,p2);
		//transform Line_3D with Homography_3D hom
		Line_3D lineTrans = (Line_3D)hom.transform(line);
		Vector lineCalcVec = hom.getLineMatrix().mult(line.getV());
		//check transformation performed by sugr
		for(int i=0;i<6;i++)if(lineTrans.getV().get(i)!=lineCalcVec.get(i))fail();

		//test statistics of transformed Line_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Line_3D uncertain Line",0,MAXITER);
		TestStatistics stats = new TestStatistics(6);
		RandomVector rv = new RandomVector(line.getV(),line.getCov());
		//RandomVector rvHom = new RandomVector(homVec,homCov);
		Homography_3D homStats = new Homography_3D(homArray,homCov);
		homStats.setV(homVec);
		Line_3D lineStats;
		for (int i=0;i<MAXITER;i++){
			lineStats=new Line_3D(rv.getNextRandom(),line.getCov());
			//homStats.setV(rvHom.getNextRandom());
			Line_3D lineStatsTrans=(Line_3D)homStats.transform(lineStats);
			stats.addObservation(lineStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Line_3D simuLine = new Line_3D(stats.getMue(),stats.getSigma());
		simuLine.normalizeEuclideanComparable();
		lineTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuLine,lineTrans,0.5)){
			System.out.println("Test for transform(Line_3D) with uncertain Line");
			System.out.println("Simulation: "+simuLine);
			System.out.println("sugr: "+lineTrans);
			fail();
		}
	}

	
	/**
	 * Test for Entity transform(Plane_3D) with uncertain Homography
	 */ 
	public void testTransformEntityPlaneUncertainHomography() {
		int MAXITER=(int)100000;
		//initialize Homography_3D hom with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>i)covArray[i][j]=1e-4*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Plane_3D plane
		Plane_3D plane=new Plane_3D(2,5,3);
		//transform Line_3D with Homography_3D hom
		Plane_3D planeTrans = (Plane_3D)hom.transform(plane);
		Vector planeCalcVec = hom.getPlaneMatrix().mult(plane.getV());
		//check transformation performed by sugr
		for(int i=0;i<4;i++)if(planeTrans.getV().get(i)!=planeCalcVec.get(i))fail();

		//test statistics of transformed Line_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Plane_3D uncertain Homography_3D",0,MAXITER);
		TestStatistics stats = new TestStatistics(4);
		//RandomVector rv = new RandomVector(pointVec,pointCov);
		RandomVector rvHom = new RandomVector(homVec,homCov);
		Homography_3D homStats = new Homography_3D(homArray,homCov);
		for (int i=0;i<MAXITER;i++){
			//pointStats=new Point_3D(rv.getNextRandom(),pointCov);
			homStats.setV(rvHom.getNextRandom());
			Plane_3D planeStatsTrans=(Plane_3D)homStats.transform(plane);
			stats.addObservation(planeStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Plane_3D simuPlane = new Plane_3D(stats.getMue(),stats.getSigma());
		simuPlane.normalizeEuclideanComparable();
		planeTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPlane,planeTrans,0.5)){
			System.out.println("Test for transform(Plane_3D) with uncertain Homography_3D");
			System.out.println("Simulation: "+simuPlane);
			System.out.println("sugr: "+planeTrans);
			fail();
		}
	}

	/**
	 * Test for Entity transform(Plane_3D) with uncertain Plane
	 */ 
	public void testTransformEntityPlaneUncertainPlane() {
		int MAXITER=(int)100000;
		//initialize Homography_3D hom with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		Homography_3D hom = new Homography_3D(homArray,LinearAlgebraFactory.getMatrix(16,16));
		Vector homVec = LinearAlgebraFactory.getVector(homArray);
		hom.setV(homVec);
		//init Plane_3D plane with random covariance
		double[][] covArray = new double [4][4];
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(j>i)covArray[i][j]=1e-4*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix planeCov = LinearAlgebraFactory.getMatrix(covArray);
		Plane_3D plane=new Plane_3D(2,5,3,planeCov);
		//transform Line_3D with Homography_3D hom
		Plane_3D planeTrans = (Plane_3D)hom.transform(plane);
		Vector planeCalcVec = hom.getPlaneMatrix().mult(plane.getV());
		//check transformation performed by sugr
		for(int i=0;i<4;i++)if(planeTrans.getV().get(i)!=planeCalcVec.get(i))fail();

		//test statistics of transformed Line_3D pointTrans
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformEntity",
					"with Plane_3D uncertain Plane_3D",0,MAXITER);
		TestStatistics stats = new TestStatistics(4);
		RandomVector rvPlane = new RandomVector(plane.getV(),planeCov);
		Plane_3D planeStats;
		for (int i=0;i<MAXITER;i++){
			planeStats=new Plane_3D(rvPlane.getNextRandom(),planeCov);
			Plane_3D planeStatsTrans=(Plane_3D)hom.transform(planeStats);
			stats.addObservation(planeStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Plane_3D simuPlane = new Plane_3D(stats.getMue(),stats.getSigma());
		simuPlane.normalizeEuclideanComparable();
		planeTrans.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPlane,planeTrans,0.5)){
			System.out.println("Test for transform(Plane_3D) with uncertain Plane_3D");
			System.out.println("Simulation: "+simuPlane);
			System.out.println("sugr: "+planeTrans);
			fail();
		}
	}

	/**
	 * Test for int getDimRow()
	 */
	public void testGetDimRow() {
		Homography_3D hom = new Homography_3D();
		if(hom.getDimRow()!=4)fail();
	}

	/**
	 * Test for int getDimCol()
	 */
	public void testGetDimCol() {
		Homography_3D hom = new Homography_3D();
		if(hom.getDimCol()!=4)fail();
	}

	/**
	 * Test for Matrix getJacobianAfterFirst()
	 */
	public void testGetJacobianAfterFirst() {
		//set up homography_3D with random covariance Matrix
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>i)covArray[i][j]=1e-4*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		
		//set up Point_3D
		Point_3D po=new Point_3D(4,15,2);
		Matrix jacPoHand=po.Pi().mult(hom.getTransformationMatrix());
		Matrix jacPoSUGR=hom.getJacobianAfterFirst(po);
		//check result
		if(!TestStatistics.check(jacPoSUGR,jacPoHand,1e-10))fail();
		
		// Test for Lines ---------------------------
		// this is the same as as for getJacobian after Second
		//set up Line_3D
		Line_3D L=new Line_3D(1,2,3,2,-1,0); // note that Pluecker 
		                                     // condition is fullfilled !
		Line_3D Lstrich=(Line_3D) hom.transform(L);
		
		// For the original and transformed line the conditions
		// Jacobian*L, Jacobian*Lstrich must be zero !!
		Vector cond1=hom.getJacobianAfterFirst(Lstrich).mult(L.getV());
		Vector cond2=hom.getJacobianAfterSecond(L).mult(Lstrich.getV());
		Vector zero =LinearAlgebraFactory.getVector(cond1.getDim());
		if(!TestStatistics.check(cond1,zero,1e-10)) fail();
		if(!TestStatistics.check(cond2,zero,1e-10)) fail();

		// For all other lines the values of the conditions
		// must be the same
		for(int i=0; i<10000; i++) {
			Line_3D Lr=new Line_3D(Math.random(),Math.random(),Math.random(),
					Math.random(),Math.random(),Math.random());
			Lr.ensureConstraints();
			Line_3D Lrstrich=new Line_3D(Math.random(),Math.random(),Math.random(),
					Math.random(),Math.random(),Math.random());
			Lstrich.ensureConstraints(); // to fullfill Pluecker condition
		
		    Vector F=hom.getJacobianAfterFirst(Lrstrich).mult(Lr.getV());
		    Vector G=hom.getJacobianAfterSecond(Lr).mult(Lrstrich.getV());

		    // This is is same condition, see S. Heuel Diss. formula
		    // 3.105
		    Vector K=Lrstrich.GammaDual().mult(hom.getTransformationMatrix()).mult(Lr.Gamma()).vec();
		
		    // All three vectors must be the same !
		    if(!TestStatistics.check(F,G,1e-10)) fail();
		    if(!TestStatistics.check(F,K,1e-10)) fail();
		    if(!TestStatistics.check(G,K,1e-10)) fail();
		}
		
		// Test for Planes ----------------------------------------------
		//set up Plane_3D
		Plane_3D pl=new Plane_3D(3,9,12);
		Matrix jacPlHand=Plane_3D.PiDual(hom.getTransformationMatrix().trans().mult(pl.getV())).mult(-1.0);
		Matrix jacPlSUGR=hom.getJacobianAfterFirst(pl);
		//check result
		if(!TestStatistics.check(jacPlSUGR,jacPlHand,1e-10))fail();
		
		//check illegal Argument exceptions
		try{
			Point_2D p2=new Point_2D(2,5);
			hom.getJacobianAfterFirst(p2);
			fail();
		}catch(IllegalArgumentException e){}
		try{
			Line_2D l2=new Line_2D(3,7);
			hom.getJacobianAfterFirst(l2);
			fail();
		}catch(IllegalArgumentException e){}
	}

	/**
	 * Test for Matrix getJacobianAfterSecond()
	 */
	public void testGetJacobianAfterSecond() {
		//set up homography_3D with random covariance Matrix
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>i)covArray[i][j]=1e-4*Math.random();
				else if (j==i)covArray[i][j]=1e-2*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix homCov = LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,homCov);
		
		//set up Point_3D
		Point_3D po=new Point_3D(4,15,2);
		Matrix jacPoHand=Point_3D.Pi(hom.getTransformationMatrix().mult(po.getV())).mult(-1.0);
		Matrix jacPoSUGR=hom.getJacobianAfterSecond(po);
		//check result
		if(!TestStatistics.check(jacPoSUGR,jacPoHand,1e-10))fail();

		// Test for Lines; that is the same test as for
		// getJacobian after First
		//set up Line_3D
		Line_3D L=new Line_3D(1,2,3,2,-1,0); // note that Pluecker 
		                                     // condition is fullfilled !
		Line_3D Lstrich=(Line_3D) hom.transform(L);
		
		// For the original and transformed line the conditions
		// Jacobian*L, Jacobian*Lstrich must be zero !!
		Vector cond1=hom.getJacobianAfterFirst(Lstrich).mult(L.getV());
		Vector cond2=hom.getJacobianAfterSecond(L).mult(Lstrich.getV());
		Vector zero =LinearAlgebraFactory.getVector(cond1.getDim());
		if(!TestStatistics.check(cond1,zero,1e-10)) fail();
		if(!TestStatistics.check(cond2,zero,1e-10)) fail();

		// For all other lines the values of the conditions
		// must be the same
		for(int i=0; i<10000; i++) {
			Line_3D Lr=new Line_3D(Math.random(),Math.random(),Math.random(),
					Math.random(),Math.random(),Math.random());
			Lr.ensureConstraints();
			Line_3D Lrstrich=new Line_3D(Math.random(),Math.random(),Math.random(),
					Math.random(),Math.random(),Math.random());
			Lstrich.ensureConstraints(); // to fullfill Pluecker condition
		
		    Vector F=hom.getJacobianAfterFirst(Lrstrich).mult(Lr.getV());
		    Vector G=hom.getJacobianAfterSecond(Lr).mult(Lrstrich.getV());

		    // This is is same condition, see S. Heuel Diss. formula
		    // 3.105
		    Vector K=Lrstrich.GammaDual().mult(hom.getTransformationMatrix()).mult(Lr.Gamma()).vec();
		
		    // All three vectors must be the same !
		    if(!TestStatistics.check(F,G,1e-10)) fail();
		    if(!TestStatistics.check(F,K,1e-10)) fail();
		    if(!TestStatistics.check(G,K,1e-10)) fail();
		}

		
		//set up Plane_3D
		Plane_3D pl=new Plane_3D(3,9,12);
		Matrix jacPlHand=pl.PiDual().mult(hom.getTransformationMatrix().trans());
		Matrix jacPlSUGR=hom.getJacobianAfterSecond(pl);
		//check result
		if(!TestStatistics.check(jacPlSUGR,jacPlHand,1e-10))fail();
		
		//check illegal Argument exceptions
		try{
			Point_2D p2=new Point_2D(2,5);
			hom.getJacobianAfterSecond(p2);
			fail();
		}catch(IllegalArgumentException e){}
		try{
			Line_2D l2=new Line_2D(3,7);
			hom.getJacobianAfterSecond(l2);
			fail();
		}catch(IllegalArgumentException e){}
	}

	/**
	 * Test for void Homography_3D()
	 */
	public void testHomography_3D() {
		Homography_3D emptyHomogr=new Homography_3D();
		if(emptyHomogr==null)fail();
	}

	/**
	 * Test for void Homography_3D(Affinity_3D, Vector)
	 */
	public void testHomography_3DAffinity_3DVector() {
		//init Affinity_3D
		//init random covariance Matrix for Affinity_3D 
		double[][] affCovArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)affCovArray[i][j]=1e-3*Math.random();
				else affCovArray[i][j]=affCovArray[j][i];
			}
		}
		Matrix affCovMat=LinearAlgebraFactory.getMatrix(affCovArray);
		double[] affArray = {12,65,97,35,57,72,67,63,15,35,91,34};
		Affinity_3D aff = new Affinity_3D(affArray,affCovMat);
		//init Vector
		Vector vec = LinearAlgebraFactory.getVector(15,90,17);
		//construct Homography_3D
		Homography_3D hom = new Homography_3D(aff,vec);
		
		//check Homography_3D
		Matrix homMat = hom.getTransformationMatrix();
		Matrix affMat = aff.getTransformationMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if (i<3){
					if (homMat.get(i,j)!=affMat.get(i,j))fail(); 
				} else {
					if ((j<3)&&(homMat.get(i,j)!=vec.get(j)))fail();
				}
			}
		}
		//check if covariance matrices of hom and aff are the same
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(affCovMat.get(i,j)!=hom.getCov().get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void Homography_3D(Affinity_3D, Vector, Matrix)
	 */
	public void testHomography_3DAffinity_3DVectorMatrix() {
		//init Affinity_3D
		double[] affArray = {12,65,97,35,57,72,67,63,15,35,91,34};
		Matrix id16=LinearAlgebraFactory.getMatrixId(16);
		Affinity_3D aff = new Affinity_3D(affArray,id16);
		//init Covariance Matrix
		double[][] homCovArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)homCovArray[i][j]=1e-3*Math.random();
				else homCovArray[i][j]=homCovArray[j][i];
			}
		}
		Matrix homCovMat=LinearAlgebraFactory.getMatrix(homCovArray);
		//init Vector
		Vector vec = LinearAlgebraFactory.getVector(15,90,17);
		//construct Homography_3D
		Homography_3D hom = new Homography_3D(aff,vec,homCovMat);
		
		//check Homography_3D
		Matrix homMat = hom.getTransformationMatrix();
		Matrix affMat = aff.getTransformationMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if (i<3){
					if (homMat.get(i,j)!=affMat.get(i,j))fail(); 
				} else {
					if ((j<3)&&(homMat.get(i,j)!=vec.get(j)))fail();
				}
			}
		}
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(homCovMat.get(i,j)!=hom.getCov().get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void Homography_3D(double[])
	 */
	public void testHomography_3DdoubleArray() {
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		Homography_3D hom = new Homography_3D(homArray);
		Matrix trMatAC = hom.getTransformationMatrix();
		for(int i=0;i<16;i++){
			if(homArray[i]!=trMatAC.get(i/4,i%4))fail();
		}
	}

	/**
	 * Test for void Homography_3D(double[], Matrix)
	 */
	public void testHomography_3DdoubleArrayMatrix() {
		//init Homography_3D with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)covArray[i][j]=1e-3*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,covMat);
		//check construction
		Matrix trMatAC = hom.getTransformationMatrix();
		Matrix covMatAC = hom.getCov();
		for(int i=0;i<16;i++){
			if(homArray[i]!=trMatAC.get(i/4,i%4))fail();
		}
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(covArray[i][j]!=covMatAC.get(i,j))fail();
			}
		}
	}
	
	/**
	 * Test for void Homography_2D(Matrix, Matrix)
	 */
	public void testHomography_3DMatrixMatrix() {
		Matrix hom=LinearAlgebraFactory.getDiagMatrix(4, 1.0);
		hom.set(0,0, 1.0);
		hom.set(0,1, 2.0);
		hom.set(0,2, 3.0);
		hom.set(1,0, 4.0);
		hom.set(1,1, 5.0);
		hom.set(1,2, 6.0);
		hom.set(2,0, 7.0);
		hom.set(2,1, 8.0);
		hom.set(2,3, 9.0);
		
		Matrix cov=LinearAlgebraFactory.getDiagMatrix(16, 0.01);
		cov.set(0,0, 1.0);
		cov.set(0,1, 2.0);
		cov.set(0,2, 3.0);
		cov.set(1,0, 4.0);
		cov.set(1,1, 5.0);
		cov.set(1,2, 6.0);
		cov.set(2,0, 7.0);
		cov.set(2,1, 8.0);
		cov.set(2,2, 9.0);
		
		Homography_3D H=new Homography_3D(hom, cov);
		H.getTransformationMatrix().print();
		
		if (!TestStatistics.check(H.getCov().copy(), cov, 1e-10))
			fail();
		if (!TestStatistics.check(H.getTransformationMatrix().copy(), hom, 1e-10))
			fail();
	}
	
	/** Test for Homography3D(Homography3D,Homography3D). */
	public void testHomography_3DHomography3DHomography3D() {
		Matrix cov1 = LinearAlgebraFactory.getMatrix(16, 16);
		cov1.set(0, 0, 0.001);
		cov1.set(1, 1, 0.001);
		cov1.set(2, 2, 2e-03);
		cov1.set(3, 3, 1e-03);
		cov1.set(4, 4, 1e-03);
		cov1.set(5, 5, 1e-03);
		cov1.set(6, 6, 1e-03);
		cov1.set(7, 7, 1e-03);
		cov1.set(8, 8, 1e-03);
		cov1.set(9, 9, 1e-03);
		cov1.set(10, 10, 1e-03);
		cov1.set(11, 11, 1e-03);
		cov1.set(12, 12, 1e-03);
		cov1.set(13, 13, 1e-03);
		cov1.set(14, 14, 1e-03);
		cov1.set(15, 15, 3e-04);


		for(int i=0; i<16; i++)
			for(int j=0; j<16; j++) 
				if (i!=j) cov1.set(i,j, 1e-06 *i*j + 1e-06);
			
		for(int i=0; i<16; i++)
			for(int j=0; j<16; j++) 
				cov1.set(i,j, cov1.get(j,i));
		


		double[] h1a = { 1, 0.1, 0.3, 2, 0.1, 1.2, 0.1, 0.3, 0.5, -1, 0.4, -0.6, 1.2, 1.0, 0.1, 1.0 };

		Homography_3D h1 = new Homography_3D(h1a, cov1);

		Matrix cov2 = LinearAlgebraFactory.getMatrix(16, 16);
		/*
		cov2.set(0, 0, 0.01);
		cov2.set(1, 1, 0.01);
		cov2.set(2, 2, 1e-02);
		cov2.set(3, 3, 1e-02);
		cov2.set(4, 4, 1e-02);
		cov2.set(5, 5, 2e-02);
		cov2.set(6, 6, 1e-02);
		cov2.set(7, 7, 1e-02);
		cov2.set(8, 8, 1e-02);

		cov2.set(0, 1, 0.0002);
		cov2.set(1, 0, 0.0002);
		cov2.set(0, 2, 0.00021);
		cov2.set(2, 0, 0.00021);
		cov2.set(3, 1, 0.0002);
		cov2.set(1, 3, 0.0002);
		cov2.set(3, 2, 0.00021);
		cov2.set(2, 3, 0.00021);
		*/

		double[] h2a = { 1, 3, 4, 6, 1, 5, 7, 8, 1, -0.1, 3, 4, -6, 4, 2, 1 };

		Homography_3D h2 = new Homography_3D(h2a, cov2);

		// Test of results:
		Matrix soll = h1.getTransformationMatrix().mult(
				h2.getTransformationMatrix());
		Homography_3D hsoll = new Homography_3D();
		hsoll.setV(soll.vec());
		hsoll.setCov(LinearAlgebraFactory.getMatrix(16,16));
		hsoll.normalizeSphere();
		Homography_3D hist = new Homography_3D(h1, h2);
		hist.normalizeSphere();
		if (!TestStatistics.check(hsoll.getV(), hist.getV(), 1e-3))
			fail();

		//RandomVector rvh2 = new RandomVector(h2.getV(), cov2);
		RandomVector rvh1 = new RandomVector(h1.getV(), cov1);

		TestStatistics stats = new TestStatistics(16);

		int MAXITER = 1000000;

		Vector tmp;
		Homography_3D hs;
		Homography_3D h1s = (Homography_3D) h1.clone();
		Homography_3D h2s = (Homography_3D) h2.clone();
		//System.out.println("Start Statistik mit MAXITER=" + MAXITER);
		
		//ProgressMonitor
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing statistic part",
														"of Homography_3D(Homo_3D,Homo_3D)",0,MAXITER);	


		// Test only varying one homography because otherwise MAXITER
		// must be too large
		for (int i = 0; i < MAXITER; i++) {
			//tmp = rvh2.getNextRandom();
			//h2s.setV(tmp);
			tmp = rvh1.getNextRandom();
			h1s.setV(tmp);

			hs = new Homography_3D(h1s, h2s);
			stats.addObservation(hs);
			//break if cancel button is pressed
			if(pm.isCanceled()) break;
			
			//update progress Monitor
			pm.setProgress(i);
		}

		Matrix sigma = stats.getSigma();
		//System.out.println("Statistik: " + stats.getMue());
		//System.out.println("Statistik: " + sigma);

		Homography_3D hsugr = new Homography_3D(h1, h2);
		//System.out.println("SUGR: " + hsugr);
		//System.out.println("DIFF Covariance: " + hsugr.getCov().minus(sigma));

		if (!TestStatistics.check(hsugr.getV(), stats.getMue(), hsugr.getCov(),
				sigma, 0.4, 6e-5)) fail();
	}


	

	/**
	 * Test for void Homography_3D(Translation_3D, Rotation_3D, Vector, Vector, Vector)
	 * As this method uses the same error propagation as Motion_3D, it isn't tested here.
	 */
	public void testHomography_3DTranslation_3DRotation_3DVectorVectorVector() {
		//set up parameter for construction
		Translation_3D trans = new Translation_3D(1,2,3);
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI/2,1/3*Math.PI);
		Vector scale = LinearAlgebraFactory.getVector(.3,.8,2);
		Vector shear = LinearAlgebraFactory.getVector(6);
		//Vector with 6 entries cannot be set through LAFactory 
		shear.set(0,4);
		shear.set(1,3);
		shear.set(2,5);
		shear.set(3,2);
		shear.set(4,8);
		shear.set(5,9);
		Vector proj = LinearAlgebraFactory.getVector(6,5,4);
		//construction with SUGR
		Homography_3D homSugr = new Homography_3D(trans,rot,scale,shear,proj);
		//construction from Affinity_3D
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear);
		Matrix homHand = aff.getTransformationMatrix();
		//set projective part
		homHand.set(3,0,6);
		homHand.set(3,1,5);
		homHand.set(3,2,4);
		//check constructions
		if(!TestStatistics.check(homSugr.getTransformationMatrix(),homHand,1e-10))fail();
	}

	/**
	 * Test for void Homography_3D(Translation_3D, Rotation_3D, Vector, Vector, Vector, Matrix)
	 */
	public void testHomography_3DTranslation_3DRotation_3DVectorVectorVectorMatrix() {
		//set up random covariance matrix (16 x 16)
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)covArray[i][j]=1e-3*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up parameter for construction
		Translation_3D trans = new Translation_3D(1,2,3);
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI/2,1/3*Math.PI);
		Vector scale = LinearAlgebraFactory.getVector(.3,.8,2);
		Vector shear = LinearAlgebraFactory.getVector(6);
		//Vector with 6 entries cannot be set through LAFactory 
		shear.set(0,4);
		shear.set(1,3);
		shear.set(2,5);
		shear.set(3,2);
		shear.set(4,8);
		shear.set(5,9);
		Vector proj = LinearAlgebraFactory.getVector(6,5,4);
		//construction with SUGR
		Homography_3D homSugr = new Homography_3D(trans,rot,scale,shear,proj,covMat);
		//construction from Affinity_3D
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear);
		Matrix homHandMat = aff.getTransformationMatrix();
		//set projective part
		homHandMat.set(3,0,6);
		homHandMat.set(3,1,5);
		homHandMat.set(3,2,4);
		//check constructions
		if(!TestStatistics.check(homSugr.getTransformationMatrix(),homHandMat,1e-10)){
			System.out.println("SUGR: "+homSugr.getTransformationMatrix());
			System.out.println("Hand: "+homHandMat);
			fail();
		}
		//check covariance matrix
		if(!TestStatistics.check(homSugr.getCov(),covMat,1e-10)){
			System.out.println(homSugr.getCov());
			System.out.println(covMat);
			fail();
		}
	}

	/**
	 * Test for void Homography_3D(Motion_3D, Vector, Vector, Vector)
	 */
	public void testHomography_3DMotion_3DVectorVectorVector() {
		//set up random covariance matrix (16 x 16)
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)covArray[i][j]=1e-3*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up parameter for construction
		Translation_3D trans = new Translation_3D(1,2,3);
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI/2,1/3*Math.PI);
		Motion_3D mot = new Motion_3D(trans,rot,covMat);
		Vector scale = LinearAlgebraFactory.getVector(.3,.8,2);
		Vector shear = LinearAlgebraFactory.getVector(6);
		//Vector with 6 entries cannot be set through LAFactory 
		shear.set(0,4);
		shear.set(1,3);
		shear.set(2,5);
		shear.set(3,2);
		shear.set(4,8);
		shear.set(5,9);
		Vector proj = LinearAlgebraFactory.getVector(6,5,4);
		//construction with SUGR
		Homography_3D homSugr = new Homography_3D(mot,scale,shear,proj);
		//construction from Affinity_3D
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear);
		Matrix homHand = aff.getTransformationMatrix();
		//set projective part
		homHand.set(3,0,6);
		homHand.set(3,1,5);
		homHand.set(3,2,4);
		//check constructions
		if(!TestStatistics.check(homHand,homSugr.getTransformationMatrix(),1e-10))fail();
		//check covariance matrix (should be the same as in mot)
		if(!TestStatistics.check(covMat,homSugr.getCov(),1e-10))fail();
	}

	/**
	 * Test for void Homography_3D(Motion_3D, Vector, Vector, Vector, Matrix)
	 */
	public void testHomography_3DMotion_3DVectorVectorVectorMatrix() {
		//set up random covariance matrix (16 x 16)
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)covArray[i][j]=1e-3*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up parameter for construction
		Translation_3D trans = new Translation_3D(1,2,3);
		Rotation_3D rot = new Rotation_3D(Math.PI,Math.PI/2,1/3*Math.PI);
		Motion_3D mot = new Motion_3D(trans,rot);
		Vector scale = LinearAlgebraFactory.getVector(.3,.8,2);
		Vector shear = LinearAlgebraFactory.getVector(6);
		//Vector with 6 entries cannot be set through LAFactory 
		shear.set(0,4);
		shear.set(1,3);
		shear.set(2,5);
		shear.set(3,2);
		shear.set(4,8);
		shear.set(5,9);
		Vector proj = LinearAlgebraFactory.getVector(6,5,4);
		//construction with SUGR
		Homography_3D homSugr = new Homography_3D(mot,scale,shear,proj,covMat);
		//construction from Affinity_3D
		Affinity_3D aff = new Affinity_3D(rot,trans,scale,shear);
		Matrix homHand = aff.getTransformationMatrix();
		//set projective part
		homHand.set(3,0,6);
		homHand.set(3,1,5);
		homHand.set(3,2,4);
		//check constructions
		if(!TestStatistics.check(homHand,homSugr.getTransformationMatrix(),1e-10))fail();
		//check covariance matrix
		if(!TestStatistics.check(covMat,homSugr.getCov(),1e-10))fail();
	}

	/**
	 * Test for Matrix getPointMatrix
	 */
	public void testGetPointMatrix() {
		//init Homography_3D with random covariance
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		double[][] covArray = new double [16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(j>=i)covArray[i][j]=1e-3*Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		Homography_3D hom = new Homography_3D(homArray,covMat);
		//check method getPointMatrix with method getTransformationMatrix
		Matrix poMat=hom.getPointMatrix();
		Matrix trMat=hom.getTransformationMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(poMat.get(i,j)!=trMat.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for Matrix getLineMatrix
	 */
	public void testGetLineMatrix() {
		//init Homography_3D
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		Homography_3D hom = new Homography_3D(homArray);
		//calculate line transformation matrix
		Vector[] H_l=new Vector[6];
		Vector[] H = new Vector [4];
		Matrix pointTransMatrix=hom.getTransformationMatrix();
		for(int i=0;i<4;i++){
			double[] array = new double[4];
			for(int j=0;j<4;j++){
				array[j]=pointTransMatrix.get(i,j);
			}
			H[i]=LinearAlgebraFactory.getVector(array);
		}
		H_l[0]=getPi(H[3]).mult(H[0]);
		H_l[1]=getPi(H[3]).mult(H[1]);
		H_l[2]=getPi(H[3]).mult(H[2]);
		H_l[3]=getPi(H[1]).mult(H[2]);
		H_l[4]=getPi(H[2]).mult(H[0]);
		H_l[5]=getPi(H[0]).mult(H[1]);
		double[][] array = new double[6][6];
		for(int i=0;i<6;i++){
			for(int j=0;j<6;j++){
				array[i][j]=H_l[i].get(j);
			}
		}
		Matrix trMatLineHand = LinearAlgebraFactory.getMatrix(array);
		Matrix lineMatSugr = hom.getLineMatrix();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(lineMatSugr.get(i,j)!=trMatLineHand.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for Matrix getPlaneMatrix
	 */
	public void testGetPlaneMatrix() {
		//init Homography_3D
		double[] homArray = {12,65,97,35,57,72,67,63,15,35,91,34,16,76,96,111};
		Homography_3D hom = new Homography_3D(homArray);
		//check method getPlaneMatrix with method getTransformationMatrix
		Matrix plMat=hom.getPlaneMatrix();
		Matrix trMatInvTrans=hom.getTransformationMatrix().invert().trans();
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				if(plMat.get(i,j)!=trMatInvTrans.get(i,j))fail();
			}
		}
	}
}
