/*
 * Created on 17.06.2004
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * Test for Incident Relation type.
 * 
 * @author Achim Knigs
 */
public class IncidentTest extends TestCase {

	/**
	 * Constructor for IncidentTest.
	 * @param arg0
	 */
	public IncidentTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for getJacobian
	 * Tests all possible Jacobians by value
	 */
	public void testGetJacobian() {
		//construct some entities:
		Point_2D p2D = new Point_2D(1,2,3);
		Line_2D l2D = new Line_2D(1,2,3);
		Point_3D p3D = new Point_3D(1,2,3,4);
		Plane_3D plane = new Plane_3D(1,2,3,4);
		Line_3D l3D = new Line_3D(1,2,3,4,5,6);
		
		//the Incident Object to be used:
		Incident inc;
		
		//Point_2D to Line_2D:
		inc = new Incident(p2D,Line_2D.class);
		
		//Jacobian is: x^T
		double[][] p2DL2DValues = {{1,2,3}};
		Matrix p2DL2DJac = LinearAlgebraFactory.getMatrix(p2DL2DValues);
		double p2DL2DZero = p2DL2DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(p2DL2DZero != 0) {
			System.out.println("Get Jacobian of Incident(Point_2D,Line_2D) failed.");
			fail();
		}
		
		//Line_2D to Point_2D:
		inc = new Incident(l2D,Point_2D.class);
		
		//Jacobian is: l^T
		double[][] l2DP2DValues = {{1,2,3}};
		Matrix l2DP2DJac = LinearAlgebraFactory.getMatrix(l2DP2DValues);
		double l2DP2DZero = l2DP2DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(l2DP2DZero != 0) {
			System.out.println("Get Jacobian of Incident(Line_2D,Point_2D) failed.");
			fail();
		}
		
		//Point_3D to Plane_3D:
		inc = new Incident(p3D,Plane_3D.class);
		
		//Jacobian is: X^T
		double[][] p3DPlaneValues = {{1,2,3,4}};
		Matrix p3DPlaneJac = LinearAlgebraFactory.getMatrix(p3DPlaneValues);
		double p3DPlaneZero = p3DPlaneJac.minus(inc.getJacobian()).vec().getLength();
		
		if(p3DPlaneZero != 0) {
			System.out.println("Get Jacobian of Incident(Point_3D,Plane_3D) failed.");
			fail();
		}
		
		//Plane_3D to Point_3D:
		inc = new Incident(plane,Point_3D.class);
		
		//Jacobian is: A^T
		double[][] planeP3DValues = {{1,2,3,4}};
		Matrix planeP3DJac = LinearAlgebraFactory.getMatrix(planeP3DValues);
		double planeP3DZero = planeP3DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(planeP3DZero != 0) {
			System.out.println("Get Jacobian of Incident(Plane_3D,Point_3D) failed.");
			fail();
		}
		
		//Point_3D to Line_3D:
		inc = new Incident(p3D,Line_3D.class);
		
		//Jacobian is: PiDual(X)^T (1,2,3,4)
		double[][] p3DL3DValues = {	{0,3,-2,0},
									{-3,0,1,0},
									{2,-1,0,0},
									{-4,0,0,1},
									{0,-4,0,2},
									{0,0,-4,3}};
		Matrix p3DL3DJac = LinearAlgebraFactory.getMatrix(p3DL3DValues).trans();
		double p3DL3DZero = p3DL3DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(p3DL3DZero != 0) {
			System.out.println("Get Jacobian of Incident(Point_3D,Line_3D) failed.");
			fail();
		}
		
		//Line_3D to Point_3D:
		inc = new Incident(l3D,Point_3D.class);
		
		//Jacobian is: GammaDual(L)^T
		double[][] l3DP3DValues = {	{0,-3,2,4},
									{3,0,-1,5},
									{-2,1,0,6},
									{-4,-5,-6,0}};
		Matrix l3DP3DJac = LinearAlgebraFactory.getMatrix(l3DP3DValues).trans();
		double l3DP3DZero = l3DP3DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(l3DP3DZero != 0) {
			System.out.println("Get Jacobian of Incident(Line_3D,Point_3D) failed.");
			fail();
		}
		
		//Line_3D to Line_3D:
		inc = new Incident(l3D,Line_3D.class);
		
		//Jacobian is: Dual(L)^T
		double[][] l3DL3DValues = {{-4,-5,-6,-1,-2,-3}};
		Matrix l3DL3DJac = LinearAlgebraFactory.getMatrix(l3DL3DValues);
		double l3DL3DZero = l3DL3DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(l3DL3DZero != 0) {
			System.out.println("Get Jacobian of Incident(Line_3D,Line_3D) failed.");
			fail();
		}
		
		//Line_3D to Plane_3D:
		inc = new Incident(l3D,Plane_3D.class);
		
		//Jacobian is: Gamma(L)^T
		double[][] l3DPlaneValues = {	{0,6,-5,-1},
										{-6,0,4,-2},
										{5,-4,0,-3},
										{1,2,3,0}};
		Matrix l3DPlaneJac = LinearAlgebraFactory.getMatrix(l3DPlaneValues).trans();
		double l3DPlaneZero = l3DPlaneJac.minus(inc.getJacobian()).vec().getLength();
		
		if(l3DPlaneZero != 0) {
			System.out.println("Get Jacobian of Incident(Line_3D,Plane_3D) failed.");
			fail();
		}
		
		//Plane_3D to Line_3D:
		inc = new Incident(plane,Line_3D.class);
		
		//Jacobian is: Pi(A)^T
		double[][] planeL3DValues = {	{4,0,0,-1},
										{0,4,0,-2},
										{0,0,4,-3},
										{0,-3,2,0},
										{3,0,-1,0},
										{-2,1,0,0}};
		Matrix planeL3DJac = LinearAlgebraFactory.getMatrix(planeL3DValues).trans();
		double planeL3DZero = planeL3DJac.minus(inc.getJacobian()).vec().getLength();
		
		if(planeL3DZero != 0) {
			System.out.println("Get Jacobian of Incident(Plane_3D,Line_3D) failed.");
			fail();
		}
	}

	/**
	 * Test for getDof
	 * Tests all degrees of Freedom for every combination of entities
	 */
	public void testGetDof() {
		//construct entities
		Point_2D p2D = new Point_2D(1,2,3);
		Line_2D l2D = new Line_2D(1,2,3);
		Point_3D p3D = new Point_3D(1,2,3,4);
		Plane_3D plane = new Plane_3D(1,2,3,4);
		Line_3D l3D = new Line_3D(1,2,3,4,5,6);
		
		//Incidence Object:
		Incident inc;
		
		//Point_2D to Line_2D:
		inc = new Incident(p2D,Line_2D.class);
		if(inc.getDof() != 1) fail();
		
		//Line_2D to Point_2D:
		inc = new Incident(l2D,Point_2D.class);
		if(inc.getDof() != 1) fail();
		
		//Point_3D to Plane_3D:
		inc = new Incident(p3D,Plane_3D.class);
		if(inc.getDof() != 1) fail();
		
		//Plane_3D to Point_3D:
		inc = new Incident(plane,Point_3D.class);
		if(inc.getDof() != 1) fail();
		
		//Point_3D to Line_3D:
		inc = new Incident(p3D,Line_3D.class);
		if(inc.getDof() != 2) fail();
		
		//Line_3D to Point_3D:
		inc = new Incident(l3D,Point_3D.class);
		if(inc.getDof() != 2) fail();
		
		//Line_3D to Line_3D:
		inc = new Incident(l3D,Line_3D.class);
		if(inc.getDof() != 1) fail();
		
		//Line_3D to Plane_3D:
		inc = new Incident(l3D,Plane_3D.class);
		if(inc.getDof() != 2) fail();
		
		//Plane_3D to Line_3D:
		inc = new Incident(plane,Line_3D.class);
		if(inc.getDof() != 2) fail();
	}

	/**
	 * Test for Incident(Entity, Class)
	 */
	public void testIncidentEntityClass() {
		//construct Incident
		Line_3D l = new Line_3D(2,2,4,1,1,-1);
		//second parameter should be Line_3D.class,
		//Point_3D.class or Plane_3D.class.
		Incident inc = new Incident(l,Line_3D.class);
		
		if(!inc.getFirstEntity().equals(l)) fail();
	}

	/**
	 * Test for Incident(Point_2D, Class)
	 * This tests a Constructor of Incident, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testIncidentPoint_2DClass() {
		double cov[][] = {{0.01,0,0},{0,0.01,0},{0,0,0.01}};
		//construct Incident
		Point_2D p = new Point_2D(2,2,1,
				LinearAlgebraFactory.getMatrix(cov));
		//second parameter should be Line_2D.class
		Incident inc = new Incident(p,Line_2D.class);
		
		if(!TestStatistics.check(inc.getFirstEntity(),p,1e-17)) fail();
	}

	/**
	 * Test for Incident(Point_3D, Class)
	 * This tests a Constructor of Incident, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testIncidentPoint_3DClass() {
		double cov[][] = {{0.01,0,0,0},
						  {0,0.01,0,0},
						  {0,0,0.01,0},
						  {0,0,0,0.01}};
		//construct Incident
		Point_3D p = new Point_3D(2,2,5,1,
				LinearAlgebraFactory.getMatrix(cov));
		//second parameter should be Line_3D.class or Plane_3D.class
		Incident inc = new Incident(p,Plane_3D.class);
		
		if(!TestStatistics.check(inc.getFirstEntity(),p,1e-17)) fail();
	}

	/**
	 * Test for void Incident(Line_3D, Class)
	 * This tests a Constructor of Incident, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testIncidentLine_3DClass() {
		double cov[][] = {{0.01,0,0,0,0,0},
						  {0,0.01,0,0,0,0},
						  {0,0,0.01,0,0,0},
						  {0,0,0,0.01,0,0},
						  {0,0,0,0,0.01,0},
						  {0,0,0,0,0,0.01}};
		//construct Incident
		Line_3D l = new Line_3D(2,7,2,5,6,1,
				LinearAlgebraFactory.getMatrix(cov));
		//second parameter should be Line_3D.class,
		//Point_3D.class or Plane_3D.class
		Incident inc = new Incident(l,Line_3D.class);
		
		if(!TestStatistics.check(inc.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for void Incident(Line_2D, Class)
	 * This tests a Constructor of Incident, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testIncidentLine_2DClass() {
		double cov[][] = {{0.01,0,0},{0,0.01,0},{0,0,0.01}};
		//construct Incident
		Line_2D l = new Line_2D(2,2,1,
				LinearAlgebraFactory.getMatrix(cov));
		//second parameter should be Point_2D.class
		Incident inc = new Incident(l,Line_2D.class);
		
		if(!TestStatistics.check(inc.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for void Incident(Plane_3D, Class)
	 * This tests a Constructor of Incident, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testIncidentPlane_3DClass() {
		double cov[][] = {{0.01,0,0,0},
						  {0,0.01,0,0},
						  {0,0,0.01,0},
						  {0,0,0,0.01}};
		//construct Incident
		Plane_3D p = new Plane_3D(2,2,1,7,
				LinearAlgebraFactory.getMatrix(cov));
		//second parameter should be Line_3D.class,
		//Point_3D.class or Line_3D.class
		Incident inc = new Incident(p,Line_3D.class);
		
		if(!TestStatistics.check(inc.getFirstEntity(),p,1e-17)) fail();
	}

}
