package sugr.test;
import junit.framework.TestCase;
import sugr.linalg.*;
import java.util.Random;
import java.lang.Math;

/*
 * Created on 19.12.2003
 *
 */

/**
 * @author Bastian Oehler
 *
 */
public class MatrixTest extends TestCase {

	/**
	 * Constructor for MatrixTest.
	 * @param arg0
	 */
	public MatrixTest(String arg0) {
		super(arg0);
	}

	static double abw=1e-15;
	
	/**
	 * Get a double array with values from 0 to (rowDim*colDim)-1
	 * @param row number of rows
	 * @param col number of columns
	 * @param c   c-1 Column/Row to delete 0 delets nothing
	 * @return double array with selected values
	 */
	public double[][] getDoubleArray(int row, int col, int c){
		double[][] array = new double[row][col];
		if (c==0){
			for (int i=0;i<row;i++){
				for (int j=0;j<col;j++){
					array[i][j]=i*col+j;
				}
			}
		}
		if (c<0){
			array = new double[row-1][col];
			for (int i=0;i<row;i++){
				for (int j=0;j<col;j++){
					if (i<-c-1) array[i][j]=i*col+j;
					if (i>-c-1) array[i-1][j]=i*col+j;
				}
			}
		}
		if (c>0){
			array = new double[row][col-1];
			for (int i=0;i<row;i++){
				for (int j=0;j<col;j++){
					if (j<c-1) array[i][j]=i*col+j;
					if (j>c-1) array[i][j-1]=i*col+j;
				}
			}
		}
		return array;
	}

	/**
	 * Get a double array with value val at every position
	 * @param row number of rows
	 * @param col number of columns
	 * @param val
	 * @return (a x b) Matrix with entry val at every position
	 */
	public double[][] getDoubleArrayEven(int row,int col,double val){
		double[][] array = new double[row][col];
		for (int i=0;i<row;i++){
			for (int j=0;j<col;j++){
				array[i][j]=val;
			}
		}
		return array;
	}

	/**
	 * Get a double array with random value from 0 to val
	 * @param row number of rows
	 * @param col number of columns
	 * @param val
	 * @return (a x b) Matrix with random value (0..val) at every position
	 */
	public double[][] getDoubleArrayRand(int row,int col,double val){
		double[][] array = new double[row][col];
		Random rand = new Random();
		for (int i=0;i<row;i++){
			for (int j=0;j<col;j++){
				array[i][j]=rand.nextDouble()*val;
			}
		}
		return array;
	}

	/**
	 * Test if Matrix A(n x m) = 0(n x m) +- abweichung
	 * @param A
	 * @param abweichung
	 * @return boolean (A==0?)
	 */
	boolean testNullMatrix(Matrix A,double abweichung){
		for (int i=0;i<A.getRowDimension();i++){
			for (int j=0;j<A.getColumnDimension();j++){
				if (Math.abs(A.get(i,j))<=abweichung)return true;
			}
		}
		return false;
	}
	
	/*
	 * Beginning tests 
	 */
	 
	/**
	 * Test for Matrix plus(Matrix)
	 */
	public void testPlus(){
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,2));
		Matrix B = LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,5));
		A = B.plus(A);
		B=LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,7));
		if (!A.equals(B)) fail();
	}

	/**
	 * Test for Matrix minus(Matrix)
	 */
	public void testMinus() {
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,2));
		Matrix B = LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,5));
		A = B.minus(A);
		B=LinearAlgebraFactory.getMatrix(getDoubleArrayEven(4,7,3));
		if (!A.equals(B)) fail();
	}

	/**
	 * Test for Matrix mult(Matrix)
	 */
	public void testMultMatrix() {
		Matrix A = LinearAlgebraFactory.getMatrixId(5);
		Matrix B = LinearAlgebraFactory.getMatrix(getDoubleArray(5,5,0));
		A=A.mult(B);
		if(!A.equals(B)) fail();
	}

	/**
	 * Test for Matrix trans()
	 */
	public void testTrans() {
		double[][] array = getDoubleArray(4,7,0);
		Matrix A =LinearAlgebraFactory.getMatrix(array);
		A=A.trans();
		for (int i=0;i<4;i++){
			for (int j=0;j<7;j++){
				if (A.get(j,i)!=array[i][j]) fail();
			}
		}
	}

	/**
	 * Test for double get(int, int)
	 */
	public void testGet(){
		double[][] array = getDoubleArray(4,7,0);
		Matrix A = LinearAlgebraFactory.getMatrix(array);
		for (int i=0;i<4;i++){
			for (int j=0;j<7;j++){
				if (A.get(i,j)!=array[i][j]) fail();
			}
		}
	}

	/**
	 * Test for void set(int, int, double)
	 */
	public void testSetintintdouble() {
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArrayEven(3,3,1));
		A.set(2,2,4);
		for (int i=0;i<A.getRowDimension();i++)
			for (int j=0;j<A.getColumnDimension();j++){
				if (A.get(i,j)!=1)
					if ((i==1)&&(j==1)&&(A.get(i,j)!=4)) fail();
			}
	}

	/**
	 * Test for int getRowDimension()
	 */
	public void testGetRowDimension() {
		Matrix A = LinearAlgebraFactory.getMatrix(4,6);
		if (A.getRowDimension()!=4) fail();
	}

	/**
	 * Test for int getColumnDimension()
	 */
	public void testGetColumnDimension() {
		Matrix A = LinearAlgebraFactory.getMatrix(4,6);
		if (A.getColumnDimension()!=6) fail();
	}

	/**
	 * Test for Matrix invert()
	 */
	public void testInvert() {
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArrayRand(6,6,5));
		Matrix B = LinearAlgebraFactory.getMatrixId(6);
		Matrix C = A.mult(A.invert());
		A=C.minus(B);
		if (!testNullMatrix(A,abw))fail();
	}

	/**
	 * Test for Matrix mult(double)
	 */
	public void testMultdouble() {
		double[][] array = getDoubleArray(5,5,0);
		Matrix A = LinearAlgebraFactory.getMatrix(array);
		A = A.mult(2);
		for (int i=0;i<5;i++){
			for (int j=0;j<5;j++){
				array[i][j]=array[i][j]*2;
				if (A.get(i,j)!=array[i][j]){
					fail();
				}
			}
		}
	}

	/**
	 * Test for Vector mult(Vector)
	 */
	public void testMultVector() {
		double[][] array=getDoubleArray(3,4,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vec=LinearAlgebraFactory.getVector(1,2,3,4);
		Vector v=A.mult(vec);
		double[] a=new double[3];
		for (int i=0;i<3;i++){
			a[i]=0;
			for (int j=0;j<4;j++){
				a[i]=a[i]+array[i][j]*vec.get(j);
			}
		}
		Vector w = LinearAlgebraFactory.getVector(a);
		if (!TestStatistics.check(v,w,1e-14)){
			System.out.println(v);
			System.out.println(w);
			fail();
		}
	}

	/**
	 * Test for Matrix[] svd()
	 */
	public void testSvd() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(2,2,0));
		Matrix[] B=A.svd();
		double[][][] U={{{2.297529205473612e-001,9.732489894677303e-001},
						{9.732489894677303e-001,-2.297529205473612e-001}},
						{{3.702459173643832e+000,0},
						{0,5.401815134754530e-001}},
						{{5.257311121191336e-001,-8.506508083520399e-001},
						{8.506508083520399e-001,5.257311121191336e-001}}};
		Matrix[] C=new Matrix[3];
		for (int i=0;i<3;i++){
			C[i]=LinearAlgebraFactory.getMatrix(U[i]);
			if (!testNullMatrix(B[i].minus(C[i]),abw)){
				System.out.println(B[i].minus(C[i]));
				fail();
			}
		}
	}

	/**
	 * Test for Matrix slice(int[], int[])
	 */
	public void testSliceintArrayintArray() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(3,3,0));
		int[] i={0,2};
		int[] j={1,2};
		A=A.slice(i,j);
		if ((A.get(0,0)!=1)||(A.get(0,1)!=2)||(A.get(1,0)!=7)||(A.get(1,1)!=8)) fail();
	}

	/**
	 * Test for void slice(int[], int[], Matrix)
	 */
	public void testSliceintArrayintArrayMatrix() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(4,4,0));
		Matrix B=LinearAlgebraFactory.getMatrix(2,2);
		int[] intarr0={1,3};
		int[] intarr1={0,2};
		A.slice(intarr0,intarr1,B);
		double[][] darr={{0,1,2,3},{0,5,0,7},{8,9,10,11},{0,13,0,15}};
		if (!A.equals(LinearAlgebraFactory.getMatrix(darr))) fail();
	}

	/**
	 * Test for void slice(int, int, int, int, Matrix)
	 */
	public void testSliceintintintintMatrix() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(4,4,0));
		Matrix B=LinearAlgebraFactory.getMatrix(3,3);
		A.slice(1,3,0,2,B);
		double[][] darr={{0,1,2,3},{0,0,0,7},{0,0,0,11},{0,0,0,15}};
		if (!A.equals(LinearAlgebraFactory.getMatrix(darr))) fail();
	}

	/**
	 * Test for Vector Vec()
	 */
	public void testVec() {
		double[][] array=getDoubleArray(2,2,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vecto=A.vec();
		double[] arr=new double[4];
		arr[0]=array[0][0];
		arr[1]=array[1][0];
		arr[2]=array[0][1];
		arr[3]=array[1][1];
		if ((vecto.get(0)!=arr[0])||(vecto.get(1)!=arr[1])||
				(vecto.get(2)!=arr[2])||(vecto.get(3)!=arr[3]))
			fail();
	}

	/**
	 * Test for void set(double)
	 */
	public void testSetdouble() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArrayRand(4,4,1));
		A.set(4);
		for (int i=0;i<4;i++)
			for (int j=0;j<4;j++)
				if (A.get(i,j)!=4) fail();
	}

	/**
	 * Test for boolean equals(Object)
	 */
	public void testEqualsObject() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(3,3,0));
		Matrix B=LinearAlgebraFactory.getMatrix(getDoubleArray(3,3,0));
		if (!A.equals(B)) fail();
	}

	/**
	 * Test for Matrix getMatrix(int,int,int,int)
	 */
	public void testGetMatrix() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(3,3,0));
		Matrix B=A.getMatrix(1,2,0,2);
		double[][] array={{3,4,5},{6,7,8}};
		for (int i=0;i<=1;i++)
			for (int j=0;j<=2;j++)
				if (B.get(i,j)!=array[i][j]) fail();
	}

	/**
	 * Test for Matrix eig()
	 */
	public void testEig(){
		double[][] MatA=new double[2][2];
		MatA[0][0]=0;
		MatA[0][1]=1;
		MatA[1][0]=1;
		MatA[1][1]=0;
		Matrix A = LinearAlgebraFactory.getMatrix(MatA);
		double[] eigenwerte =A.eig();
		if ((eigenwerte[0]!=-1) && (eigenwerte[1]!=1)){
			for (int i=0;i<eigenwerte.length;i++)
				System.out.println(eigenwerte[i]);
			fail();
		}
	}

	/**
	 * Test for boolean isSemiDefinite()
	 */
	public void testIsSemiDefinite() {
		double[][] array={{3,4,5},{3,5,2},{1,4,3}};
		Matrix A = LinearAlgebraFactory.getMatrix(array);
		if (!A.isSemiDefinite())fail();
	}

	/**
	 * Test for Matrix pseudoInverse(int)
	 */
	public void testPseudoInverse() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(3,2,0));
		Matrix B=A.pseudoInverse(2);
		Matrix C=LinearAlgebraFactory.getDiagMatrix(2,1);
		if (!testNullMatrix(B.mult(A).minus(C),abw))fail();
	}

	/**
	 * Test for Matrix Kronecker(Matrix)
	 */
	public void testKronecker() {
		double[][] array=getDoubleArray(2,2,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		double[][] arr=getDoubleArray(2,2,0);
		Matrix B=LinearAlgebraFactory.getMatrix(arr);
		B=B.plus(LinearAlgebraFactory.getMatrix(getDoubleArrayEven(2,2,5)));
		Matrix C=A.kronecker(B);
		double[][] kronArray={{0,0,5,6},{0,0,7,8},{10,12,15,18},{14,16,21,24}};
		Matrix D=LinearAlgebraFactory.getMatrix(kronArray);
		if (!testNullMatrix(D.minus(C),abw))fail();
	}

	/**
	 * Test for Matrix det()
	 */
	public void testDet(){
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArray(3,3,0));
		if (A.det()!=0) fail();
	}

	/**
	 * Test for Vector getColumn(int)
	 */
	public void testGetColumn() {
		double[][] array=getDoubleArray(3,3,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vec=A.getColumn(2);
		for (int i=0;i<vec.getDim();i++)
			if (vec.get(i)!=array[i][2]) fail();
	}

	/**
	 * Test for Vector getRow(int)
	 */
	public void testGetRow() {
		double[][] array=getDoubleArray(3,3,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vec=A.getRow(1);
		for (int i=0;i<vec.getDim();i++)
			if (vec.get(i)!=array[1][i]) fail();
	}

	/**
	 * Test for Matrix setRow(int,Vector)
	 */
	public void testSetRow() {
		double[][] array=getDoubleArray(3,3,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vec=LinearAlgebraFactory.getVector(3,2,1);
		A.setRow(1,vec);
		for (int i=0;i<3;i++){
			for (int j=0;j<3;j++){
				if (i==1) {if (vec.get(j)!=A.get(i,j))fail();} 
				else if (A.get(i,j)!=array[i][j])fail();
			}
		}
	}

	/**
	 * Test for Matrix setColumn(int,Vector)
	 */
	public void testSetColumn() {
		double[][] array=getDoubleArray(3,3,0);
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector vec=LinearAlgebraFactory.getVector(3,2,1);
		A.setColumn(1,vec);
		for (int i=0;i<3;i++){
			for (int j=0;j<3;j++){
				if (i==1) {if (vec.get(j)!=A.get(j,i))fail();} 
				else if (A.get(j,i)!=array[j][i])fail();
			}
		}
	}

	/**
	 * Test for Vector solve(Vector)
	 */
	public void testSolve() {
		double[][] array={{-1,-2},{3,2}};
		Matrix A=LinearAlgebraFactory.getMatrix(array);
		Vector b=LinearAlgebraFactory.getVector(2,2);
		if((Math.abs(A.solve(b).get(0)-2.0)>abw)||(Math.abs(A.solve(b).get(1)+2.0)>abw))fail();
	}

	/**
	 * Test for Matrix delColumn(int col)
	 */
	public void testDelColumn(){
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArray(5,8,0));
		A=A.delColumn(2);
		Matrix B = LinearAlgebraFactory.getMatrix(getDoubleArray(5,8,3));
		if (!B.equals(A)){
			System.out.println(A);
			System.out.println(B);
			fail();
		}
	}

	/**
	 * Test for Matrix delRow(int)
	 */
	public void testDelRow(){
		Matrix A = LinearAlgebraFactory.getMatrix(getDoubleArray(5,8,0));
		A=A.delRow(2);
		Matrix B = LinearAlgebraFactory.getMatrix(getDoubleArray(5,8,-3));
		if (!B.equals(A)){
			System.out.println(A);
			System.out.println(B);
			fail();
		}
	}

	/**
	 * Test for Matrix slice(int, int, int, int)
	 * No need for a test, because it is the same method as getMatrix
	 */
	public void testSliceintintintint() {
	}

	/**
	 * Test for Matrix[] qr()
	 */
	public void testQr() {
		Matrix A=LinearAlgebraFactory.getMatrix(getDoubleArray(2,2,0));
		A=A.plus(LinearAlgebraFactory.getMatrix(getDoubleArrayEven(2,2,1)));
		Matrix[] QR=A.qr();
		double[][][] array={{{-3.162277660168380e-001,-9.486832980505138e-001},
								{-9.486832980505138e-001,3.162277660168380e-001}},
							{{-3.162277660168380e+000,-4.427188724235731e+000},
								{0,-6.324555320336751e-001}}};
		Matrix[] qr_=new Matrix[2];
		for (int i=0;i<2;i++){
			qr_[i]=LinearAlgebraFactory.getMatrix(array[i]);
			if (!testNullMatrix(QR[i].minus(qr_[i]),abw)) fail();
		}
	}
}
