/*
 * Created on 03.08.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * @author Thomas Laebe, ipb
 *
 */
public class ProjectiveCameraTest extends TestCase {

	public int MAXITER=1000000;
	
	/**
	 * Constructor for ProjectiveCameraTest.
	 * @param arg0
	 */
	public ProjectiveCameraTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for int getDegreeOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		ProjectiveCamera cam = new ProjectiveCamera();
		if(cam.getDegreesOfFreedom()!=11)fail();
	}

	/**
	 * Test for Entity transform(Entity)
	 * No test needed as submethods are used only
	 */
	public void testTransformEntity() {
	}

	/**
	 * Test for int getDimRow()
	 */
	public void testGetDimRow() {
		ProjectiveCamera cam = new ProjectiveCamera();
		if(cam.getDimRow()!=3)fail();
	}

	/**
	 * Test for int GetDimCol()
	 */
	public void testGetDimCol() {
		ProjectiveCamera cam = new ProjectiveCamera();
		if(cam.getDimCol()!=4)fail();
	}

	/**
	 * Test for Matrix getJacobianAfterFirst()
	 */
	public void testGetJacobianAfterFirst() {
		//construct Point_2D and Camera
		Point_2D p=new Point_2D(2,5,7);
		ProjectiveCamera cam = new ProjectiveCamera(1,5.0/6.0,2,5,6);
		//calculate result
		Matrix jacPHand=p.getV().SkewSym().mult(cam.getPointMatrix());
		//get SUGR result
		Matrix jacPSUGR=cam.getJacobianAfterFirst(p);
		//check results
		if(!TestStatistics.check(jacPSUGR,jacPHand,1e-5))fail();
		
		//construct Line_2D
		Line_2D l=new Line_2D(1,2,5);
		//calculate result
		Matrix jacLHAND=Point_3D.Pi(cam.getTransformationMatrix().trans().mult(l.getV())).trans();
		//get SUGR result
		Matrix jacLSUGR=cam.getJacobianAfterFirst(l);
		//check results
		if(!TestStatistics.check(jacLSUGR,jacLHAND,1e-5))fail();
	}

	/**
	 * Test for Matrix getJacobianAfterSecond()
	 */
	public void testGetJacobianAfterSecond() {
		//construct Point_3D and Camera
		Point_3D p=new Point_3D(3,5,2);
		ProjectiveCamera cam=new ProjectiveCamera(1,5.0/6.0,2,5,6);
		//calculate result
		Matrix jacPHand=(cam.getPointMatrix().mult(p.getV())).SkewSym().mult(-1.0);
		//get SUGR result
		Matrix jacPSUGR=cam.getJacobianAfterSecond(p);
		//check results
		if(!TestStatistics.check(jacPSUGR,jacPHand,1e-5))fail();
		
		//construct Line_3D with two Point_3D
		Point_3D p1=new Point_3D(1,9,3);
		Point_3D p2=new Point_3D(2,7,7);
		Line_3D l=new Line_3D(p1,p2);
		//calculate result
		Matrix jacLHand=l.Gamma().trans().mult(cam.getPointMatrix().trans());
		//get SUGR result
		Matrix jacLSUGR=cam.getJacobianAfterSecond(l);
		//check results
		if(!TestStatistics.check(jacLSUGR,jacLHand,1e-5))fail();
	}

	/**
	 * Test for void ProjectiveCamera()
	 */
	public void testProjectiveCamera() {
		ProjectiveCamera cam = new ProjectiveCamera();
		if(cam==null)fail();
	}

	/**
	 * Test for void ProjectiveCamera(Matrix)
	 */
	public void testProjectiveCameraMatrix() {
		double[][] camArray = new double[3][4];
		for(int i=0;i<3;i++){
			for(int j=0;j<4;j++){
				camArray[i][j] = Math.random()*10;
			}
		}
		ProjectiveCamera cam = new ProjectiveCamera(LinearAlgebraFactory.getMatrix(camArray));
		for(int i=0;i<3;i++){
			for(int j=0;j<4;j++){
				if(camArray[i][j]!=cam.getTransformationMatrix().get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void ProjectiveCamera(Reader)
	 */
	public void testProjectiveCameraReader() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera camCheck=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11],covMat);
		//set up File
		java.io.File ffile=new java.io.File("camera.tmp");
		try{
			ffile.createNewFile();
			java.io.FileWriter fw=new java.io.FileWriter(ffile);
			for(int i=0;i<12;i++){
				fw.write(String.valueOf(da[i])+" ");
			}
			fw.write("\n");
			for(int i=0;i<12;i++){
				for(int j=0;j<12;j++){
					fw.write(String.valueOf(covArray[i][j])+" ");
				}
			}
			fw.flush();
			java.io.FileReader fr=new java.io.FileReader(ffile);
			//while(fr.ready())System.out.println(fr.read());
			ProjectiveCamera cam=new ProjectiveCamera(fr);
			//check results
			if(!TestStatistics.check(cam.getV(),camCheck.getV(),cam.getCov(),camCheck.getCov(),1e-9))fail();
		}catch(java.io.IOException e){
			System.out.println("IOException occured"+e.getMessage());
		}
		//clean up
		ffile.delete();
	}

	/**
	 * Test for void ProjectiveCamera(Matrix, Matrix)
	 */
	public void testProjectiveCameraMatrixMatrix() {
		//set up interior Oreintation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up exterior Orientation
		double[][] extOArray=new double[3][4];
		for(int i=0;i<3;i++){
			for(int j=0;j<4;j++){
				extOArray[i][j]=Math.random()*5;
			}
		}
		Matrix extO=LinearAlgebraFactory.getMatrix(extOArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(intO,extO);
		//check construction
		Matrix check=intO.mult(extO);
		if(!TestStatistics.check(cam.getPointMatrix(),check,1e-5)){
			System.out.println("testProjectiveCamera(2 x Matrix):");
			System.out.println("check-Matrix: "+check);
			System.out.println("cam.getPointMatrix: "+cam.getPointMatrix());
			fail();
		}
	}

	/**
	 * Test for void ProjectiveCamera(Matrix, Matrix, Matrix)
	 */
	public void testProjectiveCameraMatrixMatrixMatrix() {
		//set up interior Orientation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up exterior Orientation
		double[][] extOArray=new double[3][4];
		for(int i=0;i<3;i++){
			for(int j=0;j<4;j++){
				extOArray[i][j]=Math.random()*5;
			}
		}
		Matrix extO=LinearAlgebraFactory.getMatrix(extOArray);
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-4;
				else if(i==j)covArray[i][j]=Math.random()*1e-2;
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(intO,extO,covMat);
		//check construction
		Matrix check=intO.mult(extO);
		if(!TestStatistics.check(cam.getPointMatrix(),check,0.5)){
			System.out.println("testProjectiveCamera(3 x Matrix):");
			System.out.println("check-Matrix: "+check);
			System.out.println("cam.getPointMatrix: "+cam.getPointMatrix());
			fail();
		}
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Motion_3D)
	 */
	public void testProjectiveCameraMotion_3D() {
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//construct ProjectiveCamera
		ProjectiveCamera cam=new ProjectiveCamera(mot);
		//check constructed PointMatrix
		Matrix pm=cam.getPointMatrix();
		Matrix mm=mot.getTransformationMatrix();
		for(int i=0;i<3;i++)
			for(int j=0;j<4;j++)
				if(pm.get(i,j)!=mm.get(i,j))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Motion_3D, Matrix)
	 */
	public void testProjectiveCameraMotion_3DMatrix() {
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//construct ProjectiveCamera
		ProjectiveCamera cam=new ProjectiveCamera(mot,covMat);
		//check constructed PointMatrix
		Matrix pm=cam.getPointMatrix();
		Matrix mm=mot.getTransformationMatrix();
		for(int i=0;i<3;i++)
			for(int j=0;j<4;j++)
				if(pm.get(i,j)!=mm.get(i,j))fail();
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Matrix, Motion_3D)
	 */
	public void testProjectiveCameraMatrixMotion_3D() {
		//set up interior Oreintation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//construct camera
		ProjectiveCamera cam = new ProjectiveCamera(intO,mot);
		//check construction
		Matrix check=intO.mult(mot.getTransformationMatrix().slice(0,2,0,3));
		if(!TestStatistics.check(cam.getPointMatrix(),check,0.2))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Matrix, Motion_3D, Matrix)
	 */
	public void testProjectiveCameraMatrixMotion_3DMatrix() {
		//set up interior Oreintation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam = new ProjectiveCamera(intO,mot,covMat);
		//check construction
		Matrix check=intO.mult(mot.getTransformationMatrix().slice(0,2,0,3));
		if(!TestStatistics.check(cam.getPointMatrix(),check,0.2))fail();
		//check covariance
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double, double, double)
	 */
	public void testProjectiveCameradoubledoubledoubledoubledouble() {
		//set up parameters for construction
		double c=4.0;
		double s=1.3;
		double m=2.0;
		double ppX=5.0;
		double ppY=7.0;
		//set up Matrix for check
		Matrix camCheck=LinearAlgebraFactory.getMatrix(3,4);
		camCheck.set(0,0,c);
		camCheck.set(0,1,(c*s));
		camCheck.set(0,2,ppX);
		camCheck.set(1,1,(c*(1+m)));
		camCheck.set(1,2,ppY);
		camCheck.set(2,2,1);
		//construct Camera
		ProjectiveCamera cam=new ProjectiveCamera(c,s,m,ppX,ppY);
		//check results
		if(!TestStatistics.check(cam.getPointMatrix(),camCheck,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double, double, double, Matrix)
	 */
	public void testProjectiveCameradoubledoubledoubledoubledoubleMatrix() {
		//set up parameters for construction
		double c=4.0;
		double s=1.3;
		double m=2.0;
		double ppX=5.0;
		double ppY=7.0;
		//set up Matrix for check
		Matrix camCheck=LinearAlgebraFactory.getMatrix(3,4);
		camCheck.set(0,0,c);
		camCheck.set(0,1,(c*s));
		camCheck.set(0,2,ppX);
		camCheck.set(1,1,(c*(1+m)));
		camCheck.set(1,2,ppY);
		camCheck.set(2,2,1);
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		ProjectiveCamera cam=new ProjectiveCamera(c,s,m,ppX,ppY,covMat);
		//check results
		if(!TestStatistics.check(cam.getPointMatrix(),camCheck,1e-9))fail();
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Translation_3D, Rotation_3D)
	 */
	public void testProjectiveCameraTranslation_3DRotation_3D() {
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//construct ProjectiveCamera
		ProjectiveCamera cam=new ProjectiveCamera(trans,rot);
		//check constructed PointMatrix
		if(!TestStatistics.check(cam.getPointMatrix(),mot.getTransformationMatrix().slice(0,2,0,3),1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(Translation_3D, Rotation_3D, Matrix)
	 */
	public void testProjectiveCameraTranslation_3DRotation_3DMatrix() {
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up Motion_3D
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Motion_3D mot=new Motion_3D(trans,rot);
		//construct ProjectiveCamera
		ProjectiveCamera cam=new ProjectiveCamera(trans,rot,covMat);
		//check constructed PointMatrix
		if(!TestStatistics.check(cam.getPointMatrix(),mot.getTransformationMatrix().slice(0,2,0,3),1e-9))fail();
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double[])
	 */
	public void testProjectiveCameradoubleArray() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da);
		//check construction
		Matrix pm=cam.getPointMatrix();
		for(int i=0;i<12;i++)
			if(pm.get(i/4,i%4)!=da[i])fail();
	}

	/**
	 * Test for void ProjectiveCamera(double[], Matrix)
	 */
	public void testProjectiveCameradoubleArrayMatrix() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da,covMat);
		//check construction
		Matrix pm=cam.getPointMatrix();
		for(int i=0;i<12;i++){
			if(pm.get(i/4,i%4)!=da[i])fail();
		}
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double)
	 */
	public void testProjectiveCameradouble() {
		//setup parameters for construction
		double c=25.0;
		//set up check Matrix
		Matrix check=LinearAlgebraFactory.getMatrix(3,4);
		check.set(0,0,c);
		check.set(1,1,c);
		check.set(2,2,1);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(c);
		//check construction
		if(!TestStatistics.check(cam.getPointMatrix(),check,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, Matrix)
	 */
	public void testProjectiveCameradoubleMatrix() {
		//setup parameters for construction
		double c=25.0;
		//set up check Matrix
		Matrix check=LinearAlgebraFactory.getMatrix(3,4);
		check.set(0,0,c);
		check.set(1,1,c);
		check.set(2,2,1);
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(c);
		//check construction
		if(!TestStatistics.check(cam.getPointMatrix(),check,1e-9))fail();
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double)
	 */
	public void testProjectiveCameradoubledoubledouble() {
		//setup parameters for construction
		double c=25.0;
		double ppX=30.0;
		double ppY=32.0;
		//set up check Matrix
		Matrix check=LinearAlgebraFactory.getMatrix(3,4);
		check.set(0,0,c);
		check.set(0,2,ppX);
		check.set(1,1,c);
		check.set(1,2,ppY);
		check.set(2,2,1);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(c,ppX,ppY);
		//check construction
		if(!TestStatistics.check(cam.getPointMatrix(),check,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double, Matrix)
	 */
	public void testProjectiveCameradoubledoubledoubleMatrix() {
		//setup parameters for construction
		double c=25.0;
		double ppX=30.0;
		double ppY=32.0;
		//set up check Matrix
		Matrix check=LinearAlgebraFactory.getMatrix(3,4);
		check.set(0,0,c);
		check.set(0,2,ppX);
		check.set(1,1,c);
		check.set(1,2,ppY);
		check.set(2,2,1);
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(c,ppX,ppY);
		//check construction
		if(!TestStatistics.check(cam.getPointMatrix(),check,1e-9))fail();
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double, double, double, double, double, double, double, double, double, double)
	 */
	public void testProjectiveCameradoubledoubledoubledoubledoubledoubledoubledoubledoubledoubledoubledouble() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11]);
		//check construction
		Matrix pm=cam.getPointMatrix();
		for(int i=0;i<12;i++){
			if(pm.get(i/4,i%4)!=da[i])fail();
		}
	}

	/**
	 * Test for void ProjectiveCamera(double, double, double, double, double, double, double, double, double, double, double, double, Matrix)
	 */
	public void testProjectiveCameradoubledoubledoubledoubledoubledoubledoubledoubledoubledoubledoubledoubleMatrix() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11],covMat);
		//check construction
		Matrix pm=cam.getPointMatrix();
		for(int i=0;i<12;i++){
			if(pm.get(i/4,i%4)!=da[i])fail();
		}
		//check covariance Matrix
		if(!TestStatistics.check(cam.getCov(),covMat,1e-9))fail();
	}

	/**
	 * Test for Matrix getPointMatrix()
	 */
	public void testGetPointMatrix() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11]);
		//getPointMatrix
		Matrix pm=cam.getPointMatrix();
		//getTransformationMatrix
		Matrix tm=cam.getTransformationMatrix();
		//Matrices shuold be identical
		if(!TestStatistics.check(pm,tm,1e-12))fail();
	}

	/**
	 * Test for Matrix getLineMatrix()
	 */
	public void testGetLineMatrix() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11],covMat);
		//get PointMatrix
		Matrix pm=cam.getPointMatrix();
		//constuct Matrix mat for comparison
		Matrix mat=LinearAlgebraFactory.getMatrix(3,6);
		Matrix[] m=new Matrix[3];
		m[0]=LinearAlgebraFactory.getMatrix(pm.getRow(2)).trans().mult(Point_3D.Pi(pm.getRow(1)).trans());
		m[1]=LinearAlgebraFactory.getMatrix(pm.getRow(0)).trans().mult(Point_3D.Pi(pm.getRow(2)).trans());
		m[2]=LinearAlgebraFactory.getMatrix(pm.getRow(1)).trans().mult(Point_3D.Pi(pm.getRow(0)).trans());
		for(int i=0;i<6;i++){
			mat.set(0,i,m[0].get(0,i));
			mat.set(1,i,m[1].get(0,i));
			mat.set(2,i,m[2].get(0,i));
		}
		//check results
		if(!TestStatistics.check(cam.getLineMatrix(),mat,1e-9)){
			fail();
		}
	}

	/**
	 * Test for Matrix getDualLineMatrix()
	 */
	public void testGetDualLineMatrix() {
		//set up Array for construction
		double[] da={2,8,5,3,4,1,5,6,5,2,7,43};
		//set up random covariance Matrix
		double[][] covArray=new double[12][12];
		for(int i=0;i<12;i++){
			for(int j=0;j<12;j++){
				if (i<j)covArray[i][j]=Math.random()*1e-2;
				else if(i==j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11],covMat);
		//constuct Matrix mat for comparison
		Matrix C=LinearAlgebraFactory.getMatrix(6,6);
		C.set(0,3,1);
		C.set(1,4,1);
		C.set(2,5,1);
		C.set(3,0,1);
		C.set(4,1,1);
		C.set(5,2,1);
		Matrix QdualHand=cam.getLineMatrix().mult(C);
		//check results
		if(!TestStatistics.check(cam.getDualLineMatrix(),QdualHand,1e-9)){
			fail();
		}
	}

	/**
	 * Test for void swapRC()
	 */
	public void testSwapRC() {
		//set up Array for construction
		double[] da={1,2,3,4,5,6,7,8,9,10,11,12};
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11]);
		cam.swapRC();
		//sonstruct camera for comparison
		ProjectiveCamera compareWithThisCam=new ProjectiveCamera(da[4],da[5],da[6],da[7],da[0],da[1],da[2],da[3],da[8],da[9],da[10],da[11]);
		//System.out.println(cam.getTransformationMatrix());
		//System.out.println(compareWithThisCam.getTransformationMatrix());
		if(!TestStatistics.check(cam.getV(),compareWithThisCam.getV(),1e-14))fail();
	}

	/**
	 * Test for Matrix getInversePointMatrix(Vector)
	 */
	public void testGetInversePointMatrix() {
		//set up Array for construction
		double[] da={20,8,53,3,44,11,15,62,54,2,7,4};
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(da[0],da[1],da[2],da[3],da[4],da[5],da[6],da[7],da[8],da[9],da[10],da[11]);
		//construct 3 Point_3D for a Plane_3D
		Point_3D P1=new Point_3D(3,0,0);
		Point_3D P2=new Point_3D(0,3,0);
		Point_3D P3=new Point_3D(0,0,3);
		Plane_3D Plane=new Plane_3D(P1,P2,P3);
		//construct Point_3D somewhere else on the Plane
		Point_3D PoP=new Point_3D();
		PoP.setV(P1.getV().plus(P2.getV().minus(P1.getV()).mult(1.5)).plus(P3.getV().minus(P1.getV()).mult(2.31)));
		PoP.setCov(LinearAlgebraFactory.getMatrix(4,4));
		//project PoP
		Point_2D pop=cam.transform(PoP);
		//getInversePointMatrix
		Matrix ipm=cam.getInversePointMatrix(Plane.getV());
		//check result
		Vector revPro=ipm.mult(pop.getV());
		Point_3D revProj=new Point_3D(revPro.get(0),revPro.get(1),revPro.get(2),revPro.get(3));
		revProj.normalizeEuclideanComparable();
		PoP.normalizeEuclideanComparable();
		//System.out.println("Test getInversePointMatrix(Vector):");
		//System.out.println("Punkt vor der Transformation: "+PoP.getV());
		//System.out.println("rekonstruierter Punkt: "+revProj.getV());
		//System.out.println("beide Punkte sollten auf folgender Ebene liegen: "+Plane.getV());
		if(!TestStatistics.check(PoP.getV(),revProj.getV(),1e-5))fail();
	}

	/**
	 * Test for Matrix getRotationMatrix()
	 */
	public void testGetRotationMatrix() {
		//set up interior Orientation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up exterior Orientation
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4-0.1,Math.PI/2-0.1,Math.PI-0.1);
		Matrix extO=(rot.getTransformationMatrix().mult(trans.getTransformationMatrix())).delRow(3);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(intO,extO);
		//getRotationMatrix
		Matrix rotCamera=cam.getRotationMatrix();
		//check results
		//System.out.println("Test getRotationMatrix:");
		//System.out.println("SUGR rotation: "+rotCamera);
		//System.out.println("rotation set through exterior Orientation: "+rot.getTransformationMatrix().delRow(3).delColumn(3));
		if(!TestStatistics.checkAbsolute(rot.getTransformationMatrix().delRow(3).delColumn(3),rotCamera,1e-5))fail();
	}

	/**
	 * Test for Matrix getCalibrationMatrix()
	 */
	public void testGetCalibrationMatrix() {
		//set up interior Orientation
		double c=1;
		double s=5.0/6.0;
		double m=3.0;
		double ppX=0.2;
		double ppY=0.1;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//set up exterior Orientation
		Translation_3D trans=new Translation_3D(2,7,1);
		Rotation_3D rot=new Rotation_3D(Math.PI/4,Math.PI/2,Math.PI);
		Matrix extO=(rot.getTransformationMatrix().mult(trans.getTransformationMatrix())).delRow(3);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(intO,extO);
		//getCalibrationMatrix
		Matrix calib=cam.getCalibrationMatrix();
		//check results
		//System.out.println("Test getCalibrationMatrix():");
		//System.out.println("set Orientation: "+intO);
		//System.out.println("Calibration calculated from SUGR: "+calib);
		if(!TestStatistics.check(intO,calib,1e-5))fail();
	}

	/**
	 * Test for Plane_3D getPrincipalPlane
	 */
	public void testGetPrincipalPlane() {
	}

	/**
	 * Test for Point_3D getProjectionCenter()
	 */
	public void testGetProjectionCenter() {
		//set up interior Orientation
		double c=5;
		double s=1;
		double m=3.0;
		double ppX=0;
		double ppY=0;
		Matrix intO=LinearAlgebraFactory.getMatrix(3,3);
		intO.set(0,0,c);
		intO.set(1,1,(c*(1+m)));
		intO.set(0,1,(c*s));
		intO.set(0,2,ppX);
		intO.set(1,2,ppY);
		intO.set(2,2,1);
		//System.out.println(intO);
		//set up Matrix for exterior Orientation:
		// R* (I | -X0)
		Point_3D given_pc= new Point_3D(2,7,10);
		Translation_3D trans=new Translation_3D(-given_pc.getV().get(0),
				-given_pc.getV().get(1),-given_pc.getV().get(2));
		//System.out.println(trans.getTransformationMatrix());
		Rotation_3D rot=new Rotation_3D(0.1,-0.2,0.3);
		//System.out.println(rot.getTransformationMatrix());
		Matrix extO=(rot.getTransformationMatrix().mult(trans.getTransformationMatrix())).delRow(3);
		//construct camera
		ProjectiveCamera cam=new ProjectiveCamera(intO,extO);
		//System.out.println(cam.getTransformationMatrix());
		//getProjectionCenter
		Point_3D pc=cam.getProjectionCenter();
		//normalize results
		pc.normalizeEuclidean();
		if(!TestStatistics.check(pc.getV(),given_pc.getV(),1e-14)){
		System.out.println(pc.getV());
		fail();
		}
	}

	/**
	 * Test for Line_3D transform(Point_2D)
	 */
	public void testTransformPoint_2D() {
		//construct Point_2D
		double[][] pca={{0.05,0.001,0.000},{0.001,0.06,0.0000},{0.000,0.0000,0.00}};
		Matrix pointCov=LinearAlgebraFactory.getMatrix(pca);
		Point_2D p=new Point_2D(30,50,pointCov);
		//construct camera with random covariance Matrix
		Matrix camCov=LinearAlgebraFactory.getMatrix(12,12);
		double[] ca=new double[12];
		for(int i=0;i<12;i++){
			ca[i]=Math.random()*1e2;
			
			for(int j=0;j<12;j++){
				if(i<j)	camCov.set(i,i,Math.random()*1e-7);
			else if(i==j) camCov.set(i,i,Math.random()*1e-5+1e-5);
				else camCov.set(i,j,camCov.get(j,i));
			}
			
		}
		ProjectiveCamera cam=new ProjectiveCamera(ca[0],ca[1],ca[2],ca[3],ca[4],ca[5],ca[6],ca[7],ca[8],ca[9],ca[10],ca[11],camCov);
		//calculate result
		Line_3D l=cam.transform(p);
		//get result from sugr
		Vector lineVec=cam.getDualLineMatrix().trans().mult(p.getV());
		//check results
		if(!TestStatistics.check(l.getV(),lineVec,1e-9))fail();

		//test statistics
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformPoint_2D",
					"",0,MAXITER);
		TestStatistics stats = new TestStatistics(6);
		RandomVector rv = new RandomVector(p.getV(),pointCov);
		RandomVector rvHom = new RandomVector(LinearAlgebraFactory.getVector(ca),camCov);
		Point_2D pointStats;
		Line_3D lineStatsTrans;
		for (int i=0;i<MAXITER;i++){
			pointStats=new Point_2D(rv.getNextRandom(),pointCov);
			Vector rvTemp=rvHom.getNextRandom();
			ProjectiveCamera camStats=new ProjectiveCamera(rvTemp.get(0),rvTemp.get(1),rvTemp.get(2),rvTemp.get(3),rvTemp.get(4),rvTemp.get(5),rvTemp.get(6),rvTemp.get(7),rvTemp.get(8),rvTemp.get(9),rvTemp.get(10),rvTemp.get(11),camCov);
			lineStatsTrans=camStats.transform(pointStats);
			stats.addObservation(lineStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Line_3D simuLine = new Line_3D(stats.getMue(),stats.getSigma());
		simuLine.normalizeEuclideanComparable();
		l.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuLine,l,0.5)){
			System.out.println("testTransform(Point_2D)");
			System.out.println("SUGR: "+l);
			System.out.println("HAND: "+simuLine);
			fail();	
		}
	}

	/**
	 * Test for Plane_3D transform(Line_2D)
	 */
	public void testTransformLine_2D() {
		//construct Line_2D
		double[][] lca={{0.05,0.001,0.003},{0.001,0.06,0.0008},{0.003,0.0008,0.04}};
		Matrix lineCov=LinearAlgebraFactory.getMatrix(lca);
		Line_2D l=new Line_2D(3,6,1,lineCov);
		//construct camera with random covariance Matrix
		Matrix camCov=LinearAlgebraFactory.getMatrix(12,12);
		double[] ca=new double[12];
		for(int i=0;i<12;i++){
			ca[i]=Math.random()*1e2;
			for(int j=0;j<12;j++){
				if(i<j)camCov.set(i,j,Math.random()*1e-3);
				else if(i==j)camCov.set(i,j,Math.random()*1e-1);
				else camCov.set(i,j,camCov.get(j,i));
			}
		}
		ProjectiveCamera cam=new ProjectiveCamera(ca[0],ca[1],ca[2],ca[3],ca[4],ca[5],ca[6],ca[7],ca[8],ca[9],ca[10],ca[11],camCov);
		//calculate result
		Plane_3D p=cam.transform(l);
		//get result from sugr
		Vector planeVec=cam.getPointMatrix().trans().mult(l.getV());
		//check results
		if(!TestStatistics.check(p.getV(),planeVec,1e-9))fail();
		
		//test statistics
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformLine_2D",
					"",0,MAXITER);
		TestStatistics stats = new TestStatistics(4);
		RandomVector rv = new RandomVector(l.getV(),lineCov);
		RandomVector rvHom = new RandomVector(LinearAlgebraFactory.getVector(ca),camCov);
		Line_2D lineStats;
		Plane_3D planeStatsTrans;
		for (int i=0;i<MAXITER;i++){
			lineStats=new Line_2D(rv.getNextRandom(),lineCov);
			Vector rvTemp=rvHom.getNextRandom();
			ProjectiveCamera camStats=new ProjectiveCamera(rvTemp.get(0),rvTemp.get(1),rvTemp.get(2),rvTemp.get(3),rvTemp.get(4),rvTemp.get(5),rvTemp.get(6),rvTemp.get(7),rvTemp.get(8),rvTemp.get(9),rvTemp.get(10),rvTemp.get(11),camCov);
			planeStatsTrans=camStats.transform(lineStats);
			stats.addObservation(planeStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Plane_3D simuPlane = new Plane_3D(stats.getMue(),stats.getSigma());
		simuPlane.normalizeEuclideanComparable();
		p.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPlane,p,0.5)){
			System.out.println("testTransform(Line_2D)");
			System.out.println("SUGR: "+p);
			System.out.println("HAND: "+simuPlane);
			fail();	
		}
	}

	/**
	 * Test for Point_2D transform(Point_3D)
	 */
	public void testTransformPoint_3D() {
		//construct Point_3D
		double[][] pointCovArray={{0.1,0.001,0.002,0},{0.001,0.1,0.006,0},{0.002,0.006,0.1,0},{0,0,0,0}};
		Matrix PointCov=LinearAlgebraFactory.getMatrix(pointCovArray);
		Point_3D P=new Point_3D(3,6,1,PointCov);
		//construct camera with random covariance Matrix
		Matrix camCov=LinearAlgebraFactory.getMatrix(12,12);
		double[] ca=new double[12];
		for(int i=0;i<12;i++){
			ca[i]=Math.random()*1e2;
			for(int j=0;j<12;j++){
				if(i<j)camCov.set(i,j,Math.random()*1e-3);
				else if(i==j)camCov.set(i,j,Math.random()*1e-1);
				else camCov.set(i,j,camCov.get(j,i));
			}
		}
		ProjectiveCamera cam=new ProjectiveCamera(ca[0],ca[1],ca[2],ca[3],ca[4],ca[5],ca[6],ca[7],ca[8],ca[9],ca[10],ca[11],camCov);
		//calculate result
		Vector pVec=cam.getPointMatrix().mult(P.getV());
		//get Point_2D from camera transform
		Point_2D p=cam.transform(P);
		//check results
		if(!TestStatistics.check(p.getV(),pVec,1e-9))fail();

		//test statistics
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformPoint_3D",
					"",0,MAXITER);
		TestStatistics stats = new TestStatistics(3);
		RandomVector rv = new RandomVector(P.getV(),PointCov);
		RandomVector rvHom = new RandomVector(LinearAlgebraFactory.getVector(ca),camCov);
		Point_3D pointStats;
		Point_2D pointStatsTrans;
		for (int i=0;i<MAXITER;i++){
			pointStats=new Point_3D(rv.getNextRandom(),PointCov);
			Vector rvTemp=rvHom.getNextRandom();
			ProjectiveCamera camStats=new ProjectiveCamera(rvTemp.get(0),rvTemp.get(1),rvTemp.get(2),rvTemp.get(3),rvTemp.get(4),rvTemp.get(5),rvTemp.get(6),rvTemp.get(7),rvTemp.get(8),rvTemp.get(9),rvTemp.get(10),rvTemp.get(11),camCov);
			pointStatsTrans=camStats.transform(pointStats);
			stats.addObservation(pointStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Point_2D simuPoint = new Point_2D(stats.getMue(),stats.getSigma());
		simuPoint.normalizeEuclideanComparable();
		p.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuPoint,p,0.5)){
			System.out.println("testTransform(Point_3D)");
			System.out.println("SUGR: "+p);
			System.out.println("HAND: "+simuPoint);
			fail();	
		}
	}

	/**
	 * Test for Line_2D transform(Line_3D)
	 */
	public void testTransformLine_3D() {
		//construct Line_3D from two Point_3D
		Matrix pc1=LinearAlgebraFactory.getMatrix(4,4);
		Matrix pc2=LinearAlgebraFactory.getMatrix(4,4);
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(i<j){
					pc1.set(i,j,Math.random()*1e-4);
					pc2.set(i,j,Math.random()*1e-4);
				}else if(i==j){
					pc1.set(i,j,Math.random()*1e-2);
					pc2.set(i,j,Math.random()*1e-2);
				}else{
					pc1.set(i,j,pc1.get(j,i));
					pc2.set(i,j,pc2.get(j,i));
				}
			}
		}
		Point_3D p1=new Point_3D(10,15,13,pc1);
		Point_3D p2=new Point_3D(18,25,11,pc2);
		Line_3D l=new Line_3D(p1,p2);
		Matrix lineCov=l.getCov();
		//construct camera with random covariance Matrix
		Matrix camCov=LinearAlgebraFactory.getMatrix(12,12);
		double[] ca=new double[12];
		for(int i=0;i<12;i++){
			ca[i]=Math.random()*1e2;
			for(int j=0;j<12;j++){
				if(i<j)camCov.set(i,j,Math.random()*1e-7);
				else if(i==j)camCov.set(i,j,Math.random()*1e-5+1e-5);
				else camCov.set(i,j,camCov.get(j,i));
			}
		}
		ProjectiveCamera cam=new ProjectiveCamera(ca[0],ca[1],ca[2],ca[3],ca[4],ca[5],ca[6],ca[7],ca[8],ca[9],ca[10],ca[11],camCov);
		//calculate transformed line
		Vector v2d=cam.getLineMatrix().mult(l.getV());
		//get line from sugr
		Line_2D l2d=cam.transform(l);
		//check results
		if(!TestStatistics.check(l2d.getV(),v2d,1e-5))fail();
		
		//test statistics
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing transformLine_3D",
					"",0,MAXITER);
		TestStatistics stats = new TestStatistics(3);
		RandomVector rv = new RandomVector(l.getV(),lineCov);
		RandomVector rvHom = new RandomVector(LinearAlgebraFactory.getVector(ca),camCov);
		Line_3D lineStats;
		Line_2D lineStatsTrans;
		for (int i=0;i<MAXITER;i++){
			lineStats=new Line_3D(rv.getNextRandom(),lineCov);
			Vector rvTemp=rvHom.getNextRandom();
			ProjectiveCamera camStats=new ProjectiveCamera(rvTemp.get(0),rvTemp.get(1),rvTemp.get(2),rvTemp.get(3),rvTemp.get(4),rvTemp.get(5),rvTemp.get(6),rvTemp.get(7),rvTemp.get(8),rvTemp.get(9),rvTemp.get(10),rvTemp.get(11),camCov);
			lineStatsTrans=camStats.transform(lineStats);
			stats.addObservation(lineStatsTrans);
			//break if cancel button is pressed
			if(pm.isCanceled())break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Line_2D simuLine = new Line_2D(stats.getMue(),stats.getSigma());
		simuLine.normalizeEuclideanComparable();
		l2d.normalizeEuclideanComparable();
		if(!TestStatistics.check(simuLine,l2d,0.5)){
			System.out.println("testTransform(Line_3D)");
			System.out.println("SUGR(Line_2D): "+l2d);
			System.out.println("HAND(Line_2D): "+simuLine);
			fail();	
		}
	}
}
