/*
 * Created on 12.07.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class Rotation_3DTest extends TestCase {

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Rotation_3D rot = new Rotation_3D();
		int dof = rot.getDegreesOfFreedom();
		if(dof!=3)fail();
	}

	/**
	 * Test for void Rotation_3D()
	 */
	public void testRotation_3D() {
		Rotation_3D rot = new Rotation_3D();
		if(rot==null)fail();
	}

	/**
	 * Test for void Rotation_3D(double, double, double)
	 */
	public void testRotation_3Ddoubledoubledouble() {
		double omega = 1;
		double phi = 2;
		double kappa = 3;
		double oSin=Math.sin(omega); double oCos=Math.cos(omega);
		double pSin=Math.sin(phi); double pCos=Math.cos(phi);
		double qSin=Math.sin(kappa); double qCos=Math.cos(kappa);
		double[][] arrayX = {{1,0,0,0},{0,oCos,-oSin,0},{0,oSin,oCos,0},{0,0,0,1}};
		double[][] arrayY = {{pCos,0,pSin,0},{0,1,0,0},{-pSin,0,pCos,0},{0,0,0,1}};
		double[][] arrayZ = {{qCos,-qSin,0,0},{qSin,qCos,0,0},{0,0,1,0},{0,0,0,1}};
		Rotation_3D rot = new Rotation_3D(omega,phi,kappa);
		Matrix sugrRot=rot.getTransformationMatrix();
		Matrix rotX=LinearAlgebraFactory.getMatrix(arrayX);
		Matrix rotY=LinearAlgebraFactory.getMatrix(arrayY);
		Matrix rotZ=LinearAlgebraFactory.getMatrix(arrayZ);
		Matrix testRotZYX=rotZ.mult(rotY.mult(rotX));
		if(!TestStatistics.check(sugrRot.vec(),testRotZYX.vec(),1e-5))fail();
	}

	/**
	 * Test for void Rotation_3D(double, double, double, Matrix)
	 */
	public void testRotation_3DdoubledoubledoubleMatrix() {
		double[][] covArray = new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
                if(i<j)covArray[i][j]=Math.random()*1e-3;
                else {
                	if(i==j)covArray[i][j]=Math.random()*1e-1;
                	else covArray[i][j]=covArray[j][i];
                }
			}
		}
		Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
		double omega = 1;
		double phi = 2;
		double kappa = 3;
		double oSin=Math.sin(omega); double oCos=Math.cos(omega);
		double pSin=Math.sin(phi); double pCos=Math.cos(phi);
		double qSin=Math.sin(kappa); double qCos=Math.cos(kappa);
		double[][] arrayX = {{1,0,0,0},{0,oCos,-oSin,0},{0,oSin,oCos,0},{0,0,0,1}};
		double[][] arrayY = {{pCos,0,pSin,0},{0,1,0,0},{-pSin,0,pCos,0},{0,0,0,1}};
		double[][] arrayZ = {{qCos,-qSin,0,0},{qSin,qCos,0,0},{0,0,1,0},{0,0,0,1}};
		Rotation_3D rot = new Rotation_3D(omega,phi,kappa,covMat);
		Matrix sugrRot=rot.getTransformationMatrix();
		Matrix rotX=LinearAlgebraFactory.getMatrix(arrayX);
		Matrix rotY=LinearAlgebraFactory.getMatrix(arrayY);
		Matrix rotZ=LinearAlgebraFactory.getMatrix(arrayZ);
		Matrix testRotZYX=rotZ.mult(rotY.mult(rotX));
		if(!TestStatistics.check(sugrRot.vec(),testRotZYX.vec(),1e-5))fail();
		if(!TestStatistics.check(rot.getCov().vec(),covMat.vec(),1e-12))fail();
	}

	/**
	 * Test for void Rotation_3D(Vector)
	 */
	public void testRotation_3DVector() {
	}

	/**
	 * Test for void Rotation_3D(Vector, Matrix)
	 */
	public void testRotation_3DVectorMatrix() {
	}

	/**
	 * Test for void Rotation_3D(double, double, double, double)
	 */
	public void testRotation_3Ddoubledoubledoubledouble() {
		//set up quaternion parameters
		double q1=0.43595284407357;
		double q2=-0.71828701824341;
		double q3=0.31062245106570;
		double q4=0.44443511344300;
		//set up Rotation_3D with three angels
		Rotation_3D rot3Angles=new Rotation_3D(1,2,3);
		//set up Rotation_3D with quaternions
		Rotation_3D rotQuat=new Rotation_3D(q1,q2,q3,q4);
		//check results
		if(!TestStatistics.check(rotQuat.getTransformationMatrix(),rot3Angles.getTransformationMatrix(),1e-5))fail();
	}

	/**
	 * Test for void Rotation_3D(double, double, double, double, Matrix)
	 */
	public void testRotation_3DdoubledoubledoubledoubleMatrix() {
		//set up random covariance Matrix
		double[][] covArray = new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
                if(i<j)covArray[i][j]=Math.random()*1e-3;
                else {
                	if(i==j)covArray[i][j]=Math.random()*1e-1;
                	else covArray[i][j]=covArray[j][i];
                }
			}
		}
		Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
		//set up quaternion parameters
		double q1=0.43595284407357;
		double q2=-0.71828701824341;
		double q3=0.31062245106570;
		double q4=0.44443511344300;
		//set up Rotation_3D with three angels
		Rotation_3D rot3Angles=new Rotation_3D(1,2,3,covMat);
		//set up Rotation_3D with quaternions
		Rotation_3D rotQuat=new Rotation_3D(q1,q2,q3,q4,covMat);
		//check results
		if(!TestStatistics.check(rotQuat.getV(),rot3Angles.getV(),rotQuat.getCov(),rot3Angles.getCov(),1e-5))fail();
	}

	/**
	 * Test for void Rotation_3D(Vector, double)
	 */
	public void testRotation_3DVectordouble() {
		{
			//check if point p on spin-axis stays the same after transformation
			Vector spinAxis=LinearAlgebraFactory.getVector(1,1,1);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI*1.7);
			Point_3D p=new Point_3D(3,3,3);
			Point_3D rotP=(Point_3D)rot.transform(p);
			if(!TestStatistics.check(p,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (point on spin-axis) failed:");
				System.out.println("p"+p);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around first axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(1,0,0);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,3,0);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(3,-3,0);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 1st axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around second axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(0,1,0);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,3,0);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(-3,3,0);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 2nd axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around third axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(0,0,1);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,0,3);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(-3,0,3);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 3rd axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}
	}

	/**
	 * Test for void Rotation_3D(Vector, double, Matrix)
	 */
	public void testRotation_3DVectordoubleMatrix() {
		{
			//check if point p on spin-axis stays the same after transformation
			Vector spinAxis=LinearAlgebraFactory.getVector(1,1,1);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI*1.7);
			Point_3D p=new Point_3D(3,3,3);
			Point_3D rotP=(Point_3D)rot.transform(p);
			if(!TestStatistics.check(p,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (point on spin-axis) failed:");
				System.out.println("p"+p);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around first axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(1,0,0);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,3,0);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(3,-3,0);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 1st axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around second axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(0,1,0);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,3,0);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(-3,3,0);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 2nd axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if rotation around third axis is correct
			Vector spinAxis=LinearAlgebraFactory.getVector(0,0,1);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI);
			Point_3D p=new Point_3D(3,0,3);
			Point_3D rotP=(Point_3D)rot.transform(p);
			Point_3D p_strich=new Point_3D(-3,0,3);
			if(!TestStatistics.check(p_strich,rotP,1e-9)){
				System.out.println("Test spin-axis, angle (rotation around 3rd axis) failed:");
				System.out.println("p_strich"+p_strich);
				System.out.println("rotP"+rotP);
				fail();
			}
		}{
			//check if covariance matrix is set correct
			//set up random covariance Matrix
			double[][] covArray = new double[16][16];
			for(int i=0;i<16;i++){
				for(int j=0;j<16;j++){
	                if(i<j)covArray[i][j]=Math.random()*1e-3;
	                else {
	                	if(i==j)covArray[i][j]=Math.random()*1e-1;
	                	else covArray[i][j]=covArray[j][i];
	                }
				}
			}
			Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
			Vector spinAxis=LinearAlgebraFactory.getVector(1,1,1);
			Rotation_3D rot=new Rotation_3D(spinAxis,Math.PI,covMat);
			//check covariance matrix
			if(!TestStatistics.check(covMat,rot.getCov(),1e-12))fail();
		}
	}

}
