/*
 * Created on 05.07.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class Translation_2DTest extends TestCase {

	/**
	 * Test for int getDregreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Translation_2D trans = new Translation_2D();
		if(trans.getDegreesOfFreedom()!=2)fail();
	}

	/**
	 * Test for void Translation_2D()
	 */
	public void testTranslation_2D() {
		Translation_2D trans = new Translation_2D();
		if(trans==null)fail();
	}

	/**
	 * Test for void Translation_2D(double, double)
	 */
	public void testTranslation_2Ddoubledouble() {
		double[][] array = {{1,0,3},{0,1,2},{0,0,1}};
		Translation_2D trans = new Translation_2D(3,2);
		Matrix transMat=trans.getTransformationMatrix();
		for(int i=0;i<9;i++){
			if(transMat.get(i%3,i/3)!=array[i%3][i/3])fail();
		}
	}

	/**
	 * Test for void Translation_2D(double, double, Matrix)
	 */
	public void testTranslation_2DdoubledoubleMatrix() {
		double[][] array = {{1,0,3},{0,1,2},{0,0,1}};
		double[][] covArray = new double[9][9];
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(i>j)covArray[i][j]=covArray[j][i];
				else covArray[i][j]=Math.random();
			}
		}
		Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
		Translation_2D trans = new Translation_2D(3,2,covMat);
		Matrix transMat=trans.getTransformationMatrix();
		for(int i=0;i<9;i++){
			if(transMat.get(i%3,i/3)!=array[i%3][i/3])fail();
		}
		Matrix covMatSugr=trans.getCov();
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(covArray[i][j]!=covMatSugr.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void Translation_2D(Point_2D)
	 */
	public void testTranslation_2DPoint_2D() {
		double[][] array = {{1,0,2},{0,1,3},{0,0,1}};
		Point_2D p = new Point_2D(4,6,2);
		Translation_2D trans = new Translation_2D(p);
		p.normalizeEuclidean();
		Matrix transMat=trans.getTransformationMatrix();
		for(int i=0;i<9;i++){
			if(transMat.get(i%3,i/3)!=array[i%3][i/3])fail();
		}
	}

	/**
	 * Test for void Translation_2D(Point_2D, Matrix)
	 */
	public void testTranslation_2DPoint_2DMatrix() {
		double[][] covArray = new double[9][9];
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(i>j)covArray[i][j]=covArray[j][i];
				else covArray[i][j]=Math.random();
			}
		}
		Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
		double[][] array = {{1,0,2},{0,1,3},{0,0,1}};
		Point_2D p = new Point_2D(4,6,2);
		Translation_2D trans = new Translation_2D(p,covMat);
		p.normalizeEuclidean();
		Matrix transMat=trans.getTransformationMatrix();
		for(int i=0;i<9;i++){
			if(transMat.get(i%3,i/3)!=array[i%3][i/3])fail();
		}
		Matrix covMatSugr=trans.getCov();
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(covArray[i][j]!=covMatSugr.get(i,j))fail();
			}
		}
	}

}
