/*
 * Created on 17.06.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Test for TriRelationalProperty, the only really interesting test is the test
 * for getJacobian.
 * 
 * @author Achim Knigs
 */
public class TriRelationalPropertyTest extends TestCase {

	/**
	 * Constructor for TriRelationalPropertyTest.
	 * 
	 * @param arg0
	 */
	public TriRelationalPropertyTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for TriRelationalProperty()
	 */
	public void testTriRelationalProperty() {
		Point_2D p2 = new Point_2D(2, 3);
		Point_2D p1 = new Point_2D(1, 1);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		if (!rprop.getFirstEntity().equals(p1)
				|| !rprop.getSecondEntity().equals(p2)
				|| (rprop.getTargetType() != Translation_2D.class))
			fail();
	}

	/**
	 * Test for getTargetType
	 */
	public void testGetTargetType() {
		Point_2D p2 = new Point_2D(2, 3);
		Point_2D p1 = new Point_2D(1, 1);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		if (rprop.getTargetType() != Translation_2D.class)
			fail();
	}

	/**
	 * Test for getFirstEntity
	 */
	public void testGetFirstEntity() {
		Point_2D p2 = new Point_2D(2, 3);
		Point_2D p1 = new Point_2D(1, 1);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		if (!rprop.getFirstEntity().equals(p1))
			fail();
	}

	/**
	 * Test for getSecondEntity
	 */
	public void testGetSecondEntity() {
		Point_2D p2 = new Point_2D(2, 3);
		Point_2D p1 = new Point_2D(1, 1);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		if (!rprop.getSecondEntity().equals(p2))
			fail();
	}

	/**
	 * Test for getJacobian Test's the jacobians of the Relations
	 */
	public void testGetJacobian() {
		//Some entities with simple values to be easy to remember.
		Point_2D p2D = new Point_2D(1, 2);
		Point_2D p2DStrich = new Point_2D(2, 4);
		Point_3D p3D = new Point_3D(1, 2, 3);
		Point_3D p3DStrich = new Point_3D(2, 4, 6);
		Line_2D l2D = new Line_2D(1, 2, 3);
		Line_2D l2DStrich = new Line_2D(2, 4, 6);
		Line_3D l3D = new Line_3D(1, 2, 3, 4, 5, 6);
		Line_3D l3DStrich = new Line_3D(2, 4, 6, 8, 10, 12);
		Plane_3D plane = new Plane_3D(1, 2, 3, 4);
		Plane_3D planeStrich = new Plane_3D(2, 4, 6, 8);

		//The TriRelationalProperty Object
		TriRelationalProperty triProp;

		//Identity_3 is needed in some calculations
		Matrix Id3 = LinearAlgebraFactory.getDiagMatrix(3, 1.0);
		Matrix Id4 = LinearAlgebraFactory.getDiagMatrix(4, 1.0);

		//Projective Camera for Points:
		triProp = new TriRelationalProperty(p3D, p2D, ProjectiveCamera.class);
		//kronecker andersrum als in der diss:
		Matrix projectedPointJac = p2D.SkewSym().mult(
				p3D.getV().getMatrix().trans().kronecker(Id3));
		double projectedPointZero = projectedPointJac.minus(
				triProp.getJacobian()).vec().getLength();

		if (projectedPointZero != 0)
			fail();

		//Projective Camera for Lines
		triProp = new TriRelationalProperty(l3D, l2D, ProjectiveCamera.class);
		//kronecker andersrum als in der diss
		Matrix projectedLineJac = l3D.Gamma().trans().mult(
				Id4.kronecker(l2D.getV().getMatrix().trans()));
		double projectedLineZero = projectedLineJac
				.minus(triProp.getJacobian()).vec().getLength();

		if (projectedLineZero != 0)
			fail();

		//Homography 2D for Points:
		triProp = new TriRelationalProperty(p2D, p2DStrich, Homography_2D.class);
		//kronecker andersrum als in der diss
		Matrix homo2DPointJac = p2DStrich.SkewSym().mult(
				p2D.getV().getMatrix().trans().kronecker(Id3));
		double homo2DPointZero = homo2DPointJac.minus(triProp.getJacobian())
				.vec().getLength();

		if (homo2DPointZero != 0)
			fail();

		//Homography 2D for Lines:
		triProp = new TriRelationalProperty(l2D, l2DStrich, Homography_2D.class);
		//kronecker andersrum als in der diss
		Matrix homo2DLineJac = l2D.SkewSym().mult(
				Id3.kronecker(l2DStrich.getV().getMatrix().trans()));
		double homo2DLineZero = homo2DLineJac.minus(triProp.getJacobian())
				.vec().getLength();

		if (homo2DLineZero != 0)
			fail();

		//Homography 3D for Points:
		triProp = new TriRelationalProperty(p3D, p3DStrich, Homography_3D.class);
		//kronecker andersrum als in der diss I4 x X^T
		Matrix homo3DPointJac = p3DStrich.Pi().mult(
				p3D.getV().getMatrix().trans().kronecker(Id4));
		double homo3DPointZero = homo3DPointJac.minus(triProp.getJacobian())
				.vec().getLength();

		if (homo3DPointZero != 0)
			fail();

		//Homography 3D for Lines:
		triProp = new TriRelationalProperty(l3D, l3DStrich, Homography_3D.class);
		//kronecker andersrum als in der diss
		Matrix homo3DLineJac = l3D.Gamma().kronecker(
				l3DStrich.GammaDual().trans());
		double homo3DLineZero = homo3DLineJac.minus(triProp.getJacobian())
				.vec().getLength();

		if (homo3DLineZero != 0)
			fail();

		//Homography 3D for Planes:
		triProp = new TriRelationalProperty(plane, planeStrich,
				Homography_3D.class);
		//kronecker andersrum als in der diss
		Matrix homo3DPlaneJac = plane.PiDual().mult(
				Id4.kronecker(planeStrich.getV().getMatrix().trans()));
		double homo3DPlaneZero = homo3DPlaneJac.minus(triProp.getJacobian())
				.vec().getLength();

		if (homo3DPlaneZero != 0)
			fail();
	}

	/**
	 * Test for Object clone()
	 */
	public void testClone() {
		//construct some entity for the relation
		Point_2D p = new Point_2D(Double.MIN_VALUE, Double.POSITIVE_INFINITY);
		Point_2D p2 = new Point_2D(Double.MAX_VALUE, 7);

		//build the relations
		TriRelationalProperty triProp = new TriRelationalProperty(p, p2,
				Transformation.class);

		//clone objects:
		TriRelationalProperty triProp2 = (TriRelationalProperty) triProp
				.clone();

		//Compare values
		Vector zeroFirst = triProp.getFirstEntity().getV().minus(
				triProp2.getFirstEntity().getV());
		Vector zeroSecond = triProp.getSecondEntity().getV().minus(
				triProp2.getSecondEntity().getV());

		if (zeroFirst.getLength() >= 0.00000001)
			fail();
		if (zeroSecond.getLength() >= 0.00000001)
			fail();
		if (triProp.getTargetType() != triProp2.getTargetType())
			fail();
	}

	/**
	 * Test for getDof Tests the Degrees of Freedom for every allowed
	 * combination of entities and transformations
	 */
	public void testGetDof() {
		//Some entities with simple values to be easy to remember.
		Point_2D p2D = new Point_2D(1, 2);
		Point_2D p2DStrich = new Point_2D(2, 4);
		Point_3D p3D = new Point_3D(1, 2, 3);
		Point_3D p3DStrich = new Point_3D(2, 4, 6);
		Line_3D l3D = new Line_3D(1, 2, 3, 4, 5, 6);
		Line_3D l3DStrich = new Line_3D(2, 4, 6, 8, 10, 12);

		//The TriRelationalProperty Object
		TriRelationalProperty triProp;
		
		//Projection. Always the same:
		triProp = new TriRelationalProperty(p2D,p3D,ProjectiveCamera.class);
		double projectiv = triProp.getDof();
		
		if(projectiv != 2) fail();
		
		//Homographie 2D, for line and point the same:
		triProp = new TriRelationalProperty(p2D,p2DStrich,Homography_2D.class);
		double homo2D = triProp.getDof();
		
		if(homo2D != 2) fail();
		
		//Homographie 3D, for point and plane the same:
		triProp = new TriRelationalProperty(p3D,p3DStrich,Homography_3D.class);
		double homo3D = triProp.getDof();
		
		if(homo3D != 3) fail();
		
		//Homographie 3D for Lines
		triProp = new TriRelationalProperty(l3D,l3DStrich,Homography_3D.class);
		double homo3DLine = triProp.getDof();
		
		if(homo3DLine != 4) fail();
	}

	/**
	 * Test for getObservations
	 */
	public void testGetObservations() {
		Point_2D p2 = new Point_2D(2, 3, 1);
		Point_2D p1 = new Point_2D(1, 1, 8);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		//compare Vectors
		Vector sugr = rprop.getObservations();
		double[] values = {1, 1, 8, 2, 3, 1};
		Vector mine = LinearAlgebraFactory.getVector(values);
		double zero = sugr.minus(mine).getLength();

		if (zero != 0)
			fail();
	}

	/**
	 * Test for setObservations
	 */
	public void testSetObservations() {
		Point_2D p2 = new Point_2D(2, 3);
		Point_2D p1 = new Point_2D(1, 1);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		//generate some Vector and set as observation
		double[] values = {2, 632, 34, 576, 533, 7};
		Vector mine = LinearAlgebraFactory.getVector(values);
		rprop.setObservations(mine);
		Vector sugr = rprop.getObservations();
		//compare
		double zero = sugr.minus(mine).getLength();

		if (zero != 0)
			fail();
	}

	/**
	 * Test for getCovarianceOfObeservations The cov-matrices of the two
	 * entities are merged
	 */
	public void testGetCovarianceOfObservations() {
		//points with cov
		double[][] values = {{0.02, 0.001, 0.002}, {0.001, 0.016, 0.003},
				{0.002, 0.003, 0.017}};
		Matrix cov1 = LinearAlgebraFactory.getDiagMatrix(3, 0.016);
		Matrix cov2 = LinearAlgebraFactory.getMatrix(values);
		Point_2D p1 = new Point_2D(1, 1, cov1);
		Point_2D p2 = new Point_2D(2, 3, cov2);

		TriRelationalProperty rprop = new TriRelationalProperty(p1, p2,
				Translation_2D.class);

		//compare by values
		Matrix all = LinearAlgebraFactory.getMatrix(6, 6);
		all.slice(0, 2, 0, 2, cov1);
		all.slice(3, 5, 3, 5, cov2);
		Vector zero = all.minus(rprop.getCovarianceOfObservations()).vec();

		if (zero.getLength() != 0)
			fail();
	}
}