package sugr.test;

import junit.framework.TestCase;
import sugr.linalg.*;

/*
 * Created on 17.12.2003
 */

/**
 * This JUnit Test should test the funtionality of the 
 * Vector interface in sugr.linalg
 * with help of LinearAlgebraFactory which allows 
 * to test the current implementation without 
 * knowing it.
 * 
 * @author Achim Knigs
 */
public class VectorTest extends TestCase {

	/**
	 * Constructor for VectorImplJamaTest.
	 * @param arg0
	 */
	public VectorTest(String arg0) {
		super(arg0);
	}
	
	/**
	 * Check if two Vectors are identical.
	 * @param a
	 * @param b
	 * @return
	 */
	boolean check(Vector a, Vector b){
		if(a.getDim() != b.getDim()) return false;
		for(int i = 0; i < a.getDim(); i++){
			if(a.get(i) != b.get(i)) return false;
		}
		return true;
	}

	/**
	 * Simple Vector addition test.
	 */
	public void testPlus() {
		Vector a = LinearAlgebraFactory.getVector(.5,.5,.5,.5);
		Vector b = LinearAlgebraFactory.getVector(5,5,5,5);
		a = a.plus(b);
		b = LinearAlgebraFactory.getVector(5.5,5.5,5.5,5.5);
		if(!check(a,b))fail();
	}

	/**
	 * Simple Vector substraction test.
	 */
	public void testMinus() {
		Vector a = LinearAlgebraFactory.getVector(Double.MAX_VALUE,Double.MAX_VALUE,Double.MAX_VALUE);
		Vector b = LinearAlgebraFactory.getVector(1000,1000,1000);
		a = a.minus(b);
		double notFullMax = Double.MAX_VALUE-1000;
		b = LinearAlgebraFactory.getVector(notFullMax,notFullMax,notFullMax);
		if(!check(a,b))fail();
	}

	/**
	 * The dot product of two vectors is tested here.
	 */
	public void testScalarmult() {
		Vector a = LinearAlgebraFactory.getVector(3,2,1);
		Vector b = LinearAlgebraFactory.getVector(5,4,3);
		double res = a.scalarmult(b);
		if(res != 26)fail();
	}

	/**
	 * Test of the crossproduct of two three dimensional 
	 * vectors. 
	 */
	public void testCrossProduct() {
		double[] vecA = new double[3];
		vecA[0] = 4;
		vecA[1] = 8;
		vecA[2] = 7;
		Vector a = LinearAlgebraFactory.getVector(4,8,7);
		Vector b = LinearAlgebraFactory.getVector(9,4,2);
		a = a.crossProduct(b);
		b = LinearAlgebraFactory.getVector(-12,55,-56);
		if(!check(a,b))fail();
	}

	/**
	 * Test of the mult() funtion of vector. 
	 * Just multiplication of a vector with some scalar.
	 */
	public void testMult() {
    	Vector a = LinearAlgebraFactory.getVector(2,3,4);
		a = a.mult(7);
		Vector b = LinearAlgebraFactory.getVector(14,21,28);
		if(!check(a,b)) fail();
	}

	/**
	 * Test of the set(int,double) function of vector.
	 */
	public void testSetintdouble() {
		Vector a = LinearAlgebraFactory.getVector(2,4);
		a.set(1,5);
		if(!((a.get(0) == 2) && ( a.get(1) == 5))) fail();
	}

	/**
	 * Test for void set(int, Vector)
	 */
	public void testSetintVector() {
		Vector a = LinearAlgebraFactory.getVector(2,5);
		Vector b = LinearAlgebraFactory.getVector(0,0,7);
		b.set(0,a);
		if(!((b.get(0) == 2) && (b.get(1) == 5) && (b.get(2) == 7))) fail();
	}

	/**
	 * Test for double get(int)
	 */
	public void testGetint() {
		Vector a = LinearAlgebraFactory.getVector(Double.MAX_VALUE,Double.MAX_VALUE);
		double test = a.get(0);
		if(test != Double.MAX_VALUE) fail();
	}

	/**
	 * Test of the getDim() function of vector. 
	 */
	public void testGetDim() {
		Vector a = LinearAlgebraFactory.getVector(1,2,4,5);
		int dim = a.getDim();
		if(dim != 4) fail();
	}

	/**
	 * getLength() test. 
	 */
	public void testGetLength() {
		Vector a = LinearAlgebraFactory.getVector(7,8,9);
		double length = a.getLength();
		double length2 = Math.sqrt(a.get(0)*a.get(0) + a.get(1) * a.get(1) + a.get(2) * a.get(2));
		if(length != length2) fail();
	}

	/**
	 * Test for  print()
	 */
	public void testPrint() {
		Vector a = LinearAlgebraFactory.getVector(2,2);
		a.print(2,5);
	}

	/**
	 * Test for boolean equals(Object)
	 */
	public void testEqualsObject() {
		Vector a = LinearAlgebraFactory.getVector(2,3,5);
		Vector b = LinearAlgebraFactory.getVector(2,3,7);
		if(a.equals(b)) fail();
	}

	/**
	 * Test of skewSym()
	 * This is using the Matrix class
	 */
	public void testSkewSym() {
		Vector a = LinearAlgebraFactory.getVector(7,6,5);
		Matrix C  = a.SkewSym();
		
		double[][] values = {{ 0,-5, 6},
							 { 5, 0,-7},
							 {-6, 7, 0}};
		
		Matrix B  = LinearAlgebraFactory.getMatrix(values);

		if(!TestStatistics.check(C,B,0.00000001)) fail();
	}

	/**
	 * Test for Vector get(int, int)
	 */
	public void testGetintint() {
		Vector a = LinearAlgebraFactory.getVector(7,5,3,8);
		Vector b = LinearAlgebraFactory.getVector(5,3,8);
		Vector c = a.get(1,3);
		if(!check(b,c)) fail();
	}

	/**
	 * Test of normF()
	 * Should be the same as lenght.. 
	 */
	public void testNormF() {
		Vector a = LinearAlgebraFactory.getVector(2,4,3);
		double length = a.normF();
		double length2 = Math.sqrt(a.scalarmult(a));
		if(length != length2) fail();
	}

}
